/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       date.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       09-Jul-2009 19.54.51 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Module for dates/times                                                    *
* 20-Aug-2004 12.10.29 -36000 0.03    David M. Balean                       *
* Very similar to C version                                                 *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "date.h"

#include "logfile.h"
#include "quit.h"
#include "header.h"
#include "misc.h"
#include "string.h"

using namespace std;
using namespace misc;

const string Date::D_StampTXT = "DateStamp:";

// NOTE time_t is a long int in linux

const char *Date::MonthTxt[12] =
	{
	"January",
	"February",
	"March",
	"April",
	"May",
	"June",
	"July",
	"August",
	"September",
	"October",
	"November",
	"December"
	};

const char *Date::DayTxt[7] =
	{
	"Sunday",
	"Monday",
	"Tuesday",
	"Wednesday",
	"Thursday",
	"Friday",
	"Saturday"
	};

Date::Date(RevHead *revhead)
{
cout << "Date:" << endl;
rh = revhead;
}

Date::~Date(void)
{      
cout << "~Date:" << endl;
}

/********************************
 * Fix current date/time  *
 ********************************/
  // Not sure how to convert this to the format used by stat()
  // stat() gives time_t like time() above which is
  // time in seconds since Epoch (00:00:00 1st January 1970, but it may be local...)
  // int stat(char *fname, struct stat *buf)
  // use buf->st_ctime (time last changed)
  // Not sure how to then write the stat buf back to the file

  // if TRUE
  // the original date is in text as e.g. 30-Mar-1995 23.22.44.31
  // i.e.                             Day-Month-Year hour.minute.second.tick
  // where tick is in 50ths of a second
  // Unfortunately Linux only goes to seconds
  // if FALSE
  // the original date is in text as e.g Monday 15th January 1996
  // i.e.                          Day of week, Day, Month, Year

void Date::get_current_time(void)
{
struct tm *now_tm;

  
/* --- Get Time --- */
time (&rh->p_currentds);    // time in seconds since Epoch (00:00:00 UTC 1st January 1970) =GMT
                            // returns -1 for error 
if (rh->p_currentds == -1)
    {
    /* Deal with error */
    rh->quit->HandleError(TIME_F_ERR, NULL, NULL);
    return;
    }


now_tm = localtime (&rh->p_currentds);  // localtime sets external variable "timezone"
rh->p_currenttz = timezone;
  
rh->p_current_daylight = get_daylight_saving(rh->p_currentds, rh->p_currenttz);
// a copy of current date in form  Day of Week, Day, Month, Year
  
ds2date(&rh->p_currentds, &rh->p_currenttz, &rh->p_current_daylight, &rh->datebuffer, D_M_Y);

return;  
}

/***************************************
 * int get_daylight_saving(utc_ds, tz) *
 * Return the seconds of daylight      *
 * saving in force at UTC utc          *
 ***************************************/
int Date::get_daylight_saving(time_t utc_ds, time_t tz)
{
struct tm utc_tm;
struct tm local_tm;

gmtime_r(&utc_ds, &utc_tm);      // these return NULL for failure
localtime_r(&utc_ds, &local_tm);

time_t local_secs = dmyt2ds(local_tm.tm_mday,
                            local_tm.tm_mon,
                            local_tm.tm_year + 1900,
	                          local_tm.tm_hour,
                            local_tm.tm_min,
                            local_tm.tm_sec);
  
time_t utc_secs = dmyt2ds(utc_tm.tm_mday,
                          utc_tm.tm_mon,
                          utc_tm.tm_year + 1900,
	                        utc_tm.tm_hour,
                          utc_tm.tm_min,
                          utc_tm.tm_sec); 

return (local_secs - utc_secs + tz);
}




/*
  Convert day, month, year, hour, minute, second
  to seconds since start of epoch.
*/
 
time_t Date::dmyt2ds(long day, long month, long year, 
	     long hour, long minute, long second)
{
  long days;
  long i;
  time_t seconds;

int MonthDays[] =
	{
	31,
	28,
	31,
	30,
	31,
	30,
	31,
	31,
	30,
	31,
	30,
	31
	};

// Convert year to days from 1970
  for (days=0, i=1970; i < year; i++)
    {
//      if (LEAPYEAR(i)) days += 366;
      if (isleapyear(i)) days += 366;
      else days += 365;
    }

  // Add # of days end of last month
 
 if (isleapyear(year))   // THIS year; fix February days
   MonthDays[1] = 29;
 else
   MonthDays[1] = 28;

 for (i=0; i < month; i++)  // range 0 to 11
   {
     days += MonthDays[i];
   }

 days += (day - 1);  // days to end of yesterday

 seconds = (days * 24) + hour;        // convert days to hours
 seconds = (seconds * 60) + minute;   // convert hours to minutes
 seconds = (seconds * 60) + second;   // convert minutes to seconds
 return(seconds);

}


/******************************************
 * date2ds                                *
 * Convert date & time to DateStamp.      *
 * Expects format:                        *
 *   dd-Month-yyyy hh.mm.ss TZ	daylight  *
 * e.g.                                   *
 *	02-Jan-1978 10.15.02 -36000 3600      *
 * Return TRUE (OK) or FALSE (failed).		*
 ******************************************/

// NOTE: date/time portion of date is local time, not UTC

gboolean Date::date2ds(char *date, time_t *ds, time_t *tz, int *daylight)
{
long Day;
long Month;
long Year;
long Hour;
long Minute;
long Second;
long Zone;


int x;
char buffer[20];
char *c1, *c2;

 int MonthDays[] =
 {
   31,
   28,
   31,
   30,
   31,
   30,
   31,
   31,
   30,
   31,
   30,
   31
 };

c1 = date;

Day = 0;

if (!(isdigit(*c1)))
	goto error;

c2 = c1;
while ( isdigit(*c1) )
	{
	Day *= 10;
	Day += (*c1++ - '0');
	}

/* Skip the separator */
if ( ((c1 - c2) > 2) || !(*c1++))
	goto error;

/* Extract the month */
if (!(isalpha(*c1)))
	goto error;

for(x = 0; x < 3; x++)
	if ((*c1) && isalpha(*c1))
		buffer[x] = *c1++;
	else
		goto error;

buffer[x] = 0;

for (Month = 0; Month < 12; Month++)
	if (strncasecmp(buffer, MonthTxt[Month], 3) == 0)
		break;

 if (Month > 11)  // Range 0 to 11
	goto error;

/* Skip the separator */
if (!(*c1++))
	goto error;

/* Extract the year */
Year = 0;
c2 = c1;
if (!(isdigit(*c1)))
	goto error;

while ( isdigit(*c1) )
	{
	Year *= 10;
	Year += *c1++ - '0';
	}

// ALWAYS use a 4 digit number for the year

if ((c1-c2 != 4) || (Year < 1970))
	goto error;

if (isleapyear(Year))
	MonthDays[1] = 29;
else
	MonthDays[1] = 28;

if ((Day < 1) || Day > MonthDays[Month])
	goto error;

while (*c1 == ' ')
	c1++;

/* c1 now points to time */


/* Extract the hour */
c2 = c1;
Hour = 0;

if (!(isdigit(*c1)))
	goto error;

while ( isdigit(*c1) )
	{
	Hour *= 10;
	Hour += *c1++ - '0';
	}

/* Skip the separator */
if (((c1 - c2) > 2) || !(*c1++) || (Hour > 23) )
	goto error;

/* Extract the minute */
c2 = c1;
Minute = 0;

if (!(isdigit(*c1)))
	goto error;

while ( isdigit(*c1) )
	{
	Minute *= 10;
	Minute += *c1++ - '0';
	}

/* Skip the separator */
if (((c1 - c2) > 2) || !(*c1++) || (Minute > 59))
	goto error;

/* Extract the second */
c2 = c1;
Second = 0;

if (!(isdigit(*c1)))
	goto error;

while ( isdigit(*c1) )
	{
	Second *= 10;
	Second += *c1++ - '0';
	}

if ((c1 - c2 > 2) || (Second > 60)) // there can be a leap second...
	goto error;

while (*c1 == ' ')
	c1++;


x = 1; // sign
Zone = 0;
if (*c1 == '-')
  {
  x = -1;
  c1++;
  }

while ( isdigit(*c1) )
	{
	Zone *= 10;
	Zone += *c1++ - '0';
	}

*tz = Zone * x;

while (*c1 == ' ')
	c1++;

*daylight =  (isdigit(*c1)) ?  atoi(c1) : 0;

// Datestamp is LOCAL time in seconds from the start of 1970
// tz is the difference between UTC and local time so we add it to local time to get UTC

// Allow for the local daylight saving
*ds = dmyt2ds(Day, Month, Year, Hour, Minute, Second) + *tz - *daylight;

//cout << "Date::date2ds - SUCCESS" << endl;
return(TRUE);

error:
//cout << "Date::date2ds - FAILED" << endl;

return(FALSE);
}




/******************************************
 * ds2date(ds,buffer)                     *
 * Convert the datestamp ds to date       *
 * and time in the buffer subtracting     *
 * timezone. (ds is UTC)                  *
 * Format:                                *
 * if flag is TRUE...                     *
 * Day-Month-Year Hour/Minute/Second TZ	DL*
 * e.g. 02-Jan-1978 10.15.02 -36000       *
 * if flag is FALSE....                   *
 * Day of week, day, month, year          *
 * e.g. Monday 2nd January 1978	          *
 ******************************************/
// Change this:-
// flag = 0: Day of week, day, month, year        : Monday 2nd January 1978
// flag = 1: Day-Month-Year Hour/Minute/Second TZ : 02-Jan-1978 10.15.02 -36000
// flag = 2: yyyy-mm-dd hh:mm:ss                  : 1978-01-02 10:15:02
// RETURNED in buffer

// NOTE: original ds is supposed to be UTC
//       LEAP SECONDS are ignored...

void  Date::ds2date(time_t *ds, time_t *tz, int *daylight, string *datebuffer, int flag)
{
char buffer[512];

int MonthDays[] =
    {
    31,
    28,
    31,
    30,
    31,
    30,
    31,
    31,
    30,
    31,
    30,
    31
    };

int Day, Month, Year;
int Hour, Minute, Second;  //, Tick;
int WeekDay;

char *c;

// tz is the difference between UTC and local timezones
// so we subtract it from UTC for local standard time
// This means that we get the local STANDARD time.
// This does not allow for daylight saving.

//Second = *ds - *tz;
Second = *ds - *tz + *daylight; // Output the current local time

Minute = Second / 60;
Second = Second % 60;

Hour = Minute / 60;
Minute = Minute % 60;
Day = Hour / 24;
Hour = Hour % 24;

WeekDay = (Day + 4) % 7;  // 1-1-70 was Thursday

for (Year = 1970; ; Year++)
    {
    if(isleapyear(Year))
        {
        if (Day > 365) Day -= 366;
        else break;
        }
    else if (Day >= 365) Day -= 365;
    else break;
    }


if (isleapyear(Year))
    MonthDays[1] = 29;
else
    MonthDays[1] = 28;

for (Month = 0;
    Month < 12 && Day >= MonthDays[Month];
    Day -= MonthDays[Month++] ) ;

Day++; // Day starts at 1, not 0

switch (flag)
  {
  case D_M_Y:
    strcpy(buffer,DayTxt[WeekDay]);
    c = buffer + strlen(buffer);
    sprintf(c," %d",Day);
    c = buffer + strlen(buffer) - 2;

    if (*c++ != '1')
      {
      if (*c == '1') strcat(buffer,"st ");
      else if (*c == '2') strcat(buffer,"nd ");
      else if (*c == '3') strcat(buffer,"rd ");
      else strcat(buffer,"th ");
      }
    else strcat(buffer,"th ");

    strcat(buffer,MonthTxt[Month]);
    c = buffer + strlen(buffer);
    sprintf(c," %d", Year);
    break;

  case D_M_Y_T:
    sprintf(buffer,"%02d-%.3s-%04d %02d.%02d.%02d %ld %d",
            Day,MonthTxt[Month],Year,Hour,Minute,Second, *tz, *daylight);
    break;
      
  case Y_M_D_T:
  default:
    sprintf(buffer, "%04d-%02d-%02d %02d:%02d:%02d",
            Year,(Month+1),Day,Hour,Minute,Second);    
    break;
  }
  
*datebuffer = buffer;
return;
}


/********************************
 * ds_cmp(d1,d2)                *
 * Compare two DOS datestamps.  *
 * Return:                      *
 *	 0	  if identical          *
 * 	 1	  if d2 later           *
 *	-1	if d2 earlier           *
 ********************************/

int Date::ds_cmp(time_t *d1, time_t *d2)

{
int r;

if (*d1 == *d2) r = 0;
else if (*d2 > *d1) r = 1;
else r = -1;

return(r);
}



/****************************************
 * gboolean datefile(fname,ds)          *
 * Change the datestamp of fname to ds. *
 * Changes access & modification dates. *
 * Return TRUE (OK) or FALSE.           *
 ****************************************/
gboolean Date::date_file(string filename, time_t *ds)
{
struct utimbuf ut;

ut.actime = ut.modtime = *ds;
return( (utime( (char *)filename.c_str(), &ut)) ? FALSE : TRUE );  // errno is set
}



/**************************************** 
 * Get the datestamp from it's header   *
 * and put it into ds tz and daytime    *
 * Return TRUE if OK.                   *
 ****************************************/
gboolean Date::get_date(time_t *ds, time_t *tz, int *daytime, string pathname, int type)
{
fstream ifp;
gboolean rc = FALSE;

if (type != NO_TYPE)
  {
  ifp.open(pathname.c_str(), ios::in | ios::binary);
  if (ifp.is_open())
    {
    string buffer;
    string test_str;

    // Check for RevHead_Start first, make sure it is logged...
    test_str = Header::in_comment_key_start[type] + RevHead::RH_Start;

    if (rh->find_string(ifp, test_str, buffer) == 1)
      {
      test_str = Header::in_comment_key_start[type] + D_StampTXT;

      if (rh->find_string(ifp, test_str, buffer) == 1)
        {
        string tail = Header::in_comment_end[type];
        remove_head_and_tail(buffer, test_str, tail);
        
        UNLESS (buffer.empty())
          {
          if (date2ds( (char *)buffer.c_str(), ds, tz, daytime))
            {
            rc = TRUE;
            }
          }
        }
      }
    ifp.close();
    }
  }
return(rc);
}

/**************************************************************************
A year divisible by 4 is a leap year unless it is divisible by 100, except
that a year divisible by 400 is a leap year.
This means that the year 2000 is a leap year because it is divisible by 400.
 
    if (year % 4 == 0   &&  year % 100 != 0  ||  year % 400 == 0)
      THEN IT IS LEAP YEAR !!!
Therefore:
***************************************************************************/
gboolean Date::isleapyear(long year)
{
return( ( (!(year % 4)) && (year % 100) ) || (!(year % 400)) );
}


char *Date::get_date_now(void)
{
time_t ds;
struct tm *time_now;

time (&ds);                      // gets UTC time, no timezone...
time_now = localtime(&ds);       // convert to local time
return asctime(time_now);  // convert to readable form
}

/********************************************
 * void date_log_operation(string *opname)  *
 * Write an operation start notification    *
 * to the logfile which includes the type   *
 * of operation and the date and time.      *
 ********************************************/

void Date::date_log_operation(string *opname)
{
string buffer;
//time_t ds;
//struct tm *time_now;
char *datebuffer;
int len;

datebuffer = get_date_now();  
//time (&ds);                      // gets UTC time, no timezone...
//time_now = localtime(&ds);       // convert to local time
//datebuffer = asctime(time_now);  // convert to readable form
  
buffer = '\n';
buffer += *opname;
buffer += " - ";
buffer += datebuffer;
len = buffer.length() - 1;
//len--; // Allow for the newline that datebuffer has and our own initial newline
while (--len > 0) buffer += '=';

rh->logfile->write2log(buffer);
return;
}
