/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       defaults.cpp                                              *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       15-Apr-2013 20.12.03 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Separated from elsewhere.                                                 *
* It deals with revhead's definition/default file.                          *
* This is read from/written to the user's home directory.                   *
* The file has changed and this reflects the changes.                       *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "defaults.h"

#include "entrylist.h"
#include "date.h"
#include "header.h"
#include "logfile.h"
#include "header.h"
#include "misc.h"
#include "RH_version.h"

using namespace std;
using namespace misc;

string Defaults::P_Start_VersionTXT  = "PROJECT_Start_Version";
string Defaults::P_Auto2TXT          = "PROJECT_Auto";
string Defaults::P_Want_MicroTXT     = "PROJECT_Want_Micro";

string Defaults::F_AuthorTXT         = "FILE_Author";
string Defaults::F_AuthorEmailTXT    = "FILE_Author_Email";
string Defaults::F_StartVersion2TXT  = "FILE Start Version";
string Defaults::F_Auto2TXT          = "FILE_Auto";




string Defaults::P_AutoTXT          = "P_Auto:";

string Defaults::FileAuthorTXT      = "File Author:";
string Defaults::FileAuthorEmailTXT = "File Author's email:";
string Defaults::FileVersionsTXT    = "File Versions:";
string Defaults::F_StartVersionTXT  = "FILE Start Version:";
string Defaults::F_AutoTXT          = "F_Auto:";

string Defaults::Default_FileTXT    = "";

Defaults::Defaults(RevHead *revhead)
{
string fname = "/.revhead.defs";
  
cout << "Defaults:" << endl;
  
rh = revhead;
if (Default_FileTXT.empty())
  {
  char *home = getenv("HOME");
  if (home) Default_FileTXT = home;
  Default_FileTXT += fname;
  if (Default_FileTXT == fname)
    {
    string warning = "WARNING - Defaults::Defaults - HOME environmental variable has NO VALUE!\n  Using current path \"" +
                    Default_FileTXT + "\"";
    rh->logfile->write2log(warning);    
    }  
  }
}

Defaults::~Defaults(void)
{
cout << "~Defaults:" << endl;
}

// We quit keeping the existing values in RevHead
void Defaults::callback_cancel(GtkWidget *widget, Defaults *nd)
{
nd->rh->FrontPanel();
}

// Copy defaults to RevHead, USE THESE DEFAULTS
void Defaults::info_from_widgets(Defaults *nd)
{
nd->rh->np_owner = (char *)gtk_entry_get_text(GTK_ENTRY(nd->npowner_entry));
nd->rh->np_owner_email = (char *)gtk_entry_get_text(GTK_ENTRY(nd->npowner_email_entry));

nd->rh->np_major_version = (char *)gtk_entry_get_text(GTK_ENTRY(nd->np_major_vernum_entry));
nd->rh->np_minor_version = (char *)gtk_entry_get_text(GTK_ENTRY(nd->np_minor_vernum_entry));
nd->rh->np_micro_version = (char *)gtk_entry_get_text(GTK_ENTRY(nd->np_micro_vernum_entry));
nd->rh->flags.np_want_micro = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(nd->np_want_micro_check_button));
nd->rh->flags.np_auto_inc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(nd->np_auto_inc_check_button));
if (nd->rh->np_major_version.empty()) nd->rh->np_major_version = "?";
if (nd->rh->np_minor_version.empty()) nd->rh->np_minor_version = "?";          
if (nd->rh->np_micro_version.empty()) nd->rh->np_micro_version = "?";

nd->rh->nf_author = (char *)gtk_entry_get_text(GTK_ENTRY(nd->nfauthor_entry));
nd->rh->nf_author_email = (char *)gtk_entry_get_text(GTK_ENTRY(nd->nfauthor_email_entry));

nd->rh->flags.nf_auto_inc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(nd->f_auto_inc_checkbutton));
nd->rh->nf_major_version = (char *)gtk_entry_get_text(GTK_ENTRY(nd->nf_major_vernum_entry));
nd->rh->nf_minor_version = (char *)gtk_entry_get_text(GTK_ENTRY(nd->nf_minor_vernum_entry));
nd->rh->nf_micro_version = (char *)gtk_entry_get_text(GTK_ENTRY(nd->nf_micro_vernum_entry));

if (nd->rh->nf_major_version.empty()) nd->rh->nf_major_version = "?";
if (nd->rh->nf_minor_version.empty()) nd->rh->nf_minor_version = "?";          
if (nd->rh->nf_micro_version.empty()) nd->rh->nf_micro_version = "?";

if (EntryList::string_found(&nd->copyright_el))
    {
    nd->rh->np_copyright.clear();
    nd->copyright_el.createstringlist(&nd->copyright_el, nd->rh->np_copyright);
    }

if (EntryList::string_found(&nd->contact_el))
    {
    nd->rh->np_address.clear();
    nd->contact_el.createstringlist(&nd->contact_el, nd->rh->np_address);
    }
}

void Defaults::callback_use(GtkWidget *widget, Defaults *nd)
{
nd->info_from_widgets(nd);
callback_cancel(widget, nd);
}


void Defaults::callback_save_use(GtkWidget *widget, Defaults *nd)
{
nd->info_from_widgets(nd);
nd->do_save();
callback_cancel(widget, nd);
}


/*******************************************
 Save defaults for new project and new files
********************************************/
gboolean Defaults::do_save(void)
{
fstream ofp;
string buffer;
  
ofp.open(Default_FileTXT.c_str(), ios::out | ios::binary | ios::trunc );

if ( ofp.is_open() )
  {
  list<string>::iterator i;

  buffer = "Writing file \"" + Default_FileTXT + "\"";
  rh->logfile->report(buffer);

  buffer = "# Defaults file \"" +  Default_FileTXT + "\" created by " + RH_PROJECT_NAME + "-" + RH_PROJECT_VERSION + "\n\n";

  UNLESS ( rh->np_owner.empty() )
    {
    buffer += P_OwnerTXT  + "       = " + rh->np_owner + "\n";
    }
  UNLESS ( rh->np_owner_email.empty() )
    {
      buffer += P_EmailTXT + "         = " +  rh->np_owner_email + "\n";
    }
  if (rh->np_major_version.empty()) rh->np_major_version = "?"; // if all else fails!
  if (rh->np_minor_version.empty()) rh->np_minor_version = "?";          
  if (rh->np_micro_version.empty()) rh->np_micro_version = "?";

  buffer += P_Start_VersionTXT + " = ";
  buffer += rh->np_major_version;
  buffer += '.';
  buffer += rh->np_minor_version;
  buffer += '.';
  buffer += rh->np_micro_version;
  buffer += '\n';

  buffer += P_Want_MicroTXT + "    = ";
  buffer += (rh->flags.np_want_micro) ? "1\n" : "0\n";
  buffer += P_Auto2TXT + "          = ";
  buffer += (rh->flags.np_auto_inc) ? "1\n" : "0\n";

  if (!rh->np_copyright.empty())
    {
    buffer += Start_CopyrightTXT + "\n";
    for (i=rh->np_copyright.begin(); i!=rh->np_copyright.end(); ++i)
      {
      buffer += "    " + (*i) + "\n";
      }
    buffer += End_CopyrightTXT + "\n";
    }

  if (!rh->np_address.empty())
    {
    buffer += Start_ContactTXT + "\n";
    for (i=rh->np_address.begin(); i!=rh->np_address.end(); ++i)
      {
      buffer += "    " + (*i) + "\n";
      }
    buffer += End_ContactTXT + "\n";
    }

  if (!rh->nf_author.empty())
    {
    buffer += F_AuthorTXT + "           = " + rh->nf_author + "\n";
    }

  if (!rh->nf_author_email.empty())
    {
    buffer += F_AuthorEmailTXT + "     = " + rh->nf_author_email + "\n";
    }


  if (rh->nf_major_version.empty()) rh->nf_major_version = "?";
  if (rh->nf_minor_version.empty()) rh->nf_minor_version = "?";          
  if (rh->nf_micro_version.empty()) rh->nf_micro_version = "?";

  buffer += F_StartVersion2TXT + "    = ";
  buffer += rh->nf_major_version;
  buffer += '.';
  buffer += rh->nf_minor_version;
  buffer += '.';
  buffer += rh->nf_micro_version;
  buffer += '\n';
    
  buffer += F_Auto2TXT + "             = ";
  buffer += (rh->flags.nf_auto_inc) ? "1\n" : "0\n";

  buffer += '\n';

  if (rh->write_string(ofp, buffer))
    {
    ofp.close();
    buffer = "Wrote file \"" + Default_FileTXT + "\"";
    rh->logfile->report(buffer);
    return TRUE;
    }
  else
    {
    ofp.close();
    buffer = "FAILED to write file \"" + Default_FileTXT + "\"";
    rh->logfile->report(buffer);
    }
  }
else
  {
  // ERROR
  buffer = "Failed to open defaults file \"" + Default_FileTXT + "\" for write";
  rh->logfile->write2log(buffer);
  }
return FALSE;
}


void Defaults::load(void)
{
fstream ifp;
string buffer;
string line;
string value;

// Everything should already have been cleared when it gets here

ifp.open(Default_FileTXT.c_str(), ios::in | ios::binary );
UNLESS (ifp.is_open())
  {
  // use errno to get the reason for failure...
  buffer = "Failed to open project defaults file \"" + Default_FileTXT + "\"";
  rh->logfile->write2log(buffer);
  return;
  }

buffer = "Reading defaults file \"" + Default_FileTXT + "\"";
rh->logfile->write2log(buffer);

while ( misc::getline(ifp, line, TRUE) )
  {
  while (isblank(line[0])) line = line.substr(1);

  if ( (line[0] == '#') || (line[0] == ';') || (line.length() == 0) )
    {
    continue; // comment line or empty
    }

  if (getvalue(line, P_OwnerTXT, value))
    {
    rh->np_owner = value;
    }

  else if (getvalue(line, P_EmailTXT, value))
    {
    rh->np_owner_email = value;
    }

  else if (getvalue(line, P_Start_VersionTXT, value))
    {
    if (value.length())
      {
      int p;
                          
      for (p=0; p<value.length(); ++p)
        {
        if (value[p] == '.') break;
        rh->np_major_version += value[p];
        }
      p++; 
      for (; p<value.length(); ++p)
        {
        if (value[p] == '.') break;
        rh->np_minor_version += value[p];
        }
      p++; 
      for (; p<value.length(); ++p)
        {
        rh->np_micro_version += value[p];
        }
      }
    }
  else if (getvalue(line, P_Want_MicroTXT, value))
    {
    rh->flags.np_want_micro = (value == "1") ? 1 : 0;
    }
  else if (getvalue(line, P_Auto2TXT, value))
    {
    if (value == "1") rh->flags.np_auto_inc = 1;
    else rh->flags.np_auto_inc = 0;
    }
  else if (getvalue(ifp, line, Start_CopyrightTXT, End_CopyrightTXT, rh->np_copyright))
    {
    continue;
    }
    else if (getvalue(ifp, line, Start_ContactTXT, End_ContactTXT, rh->np_address))
      {
      continue;
      }

  else if (getvalue(line, F_AuthorTXT, value))
    {
    rh->nf_author = value;
    }
  else if (getvalue(line, F_AuthorEmailTXT, value))
    {
    rh->nf_author_email = value;
    }

  else if (getvalue(line, F_StartVersion2TXT, value))
    {
    if (value.length())
      {
      int p;
                          
      for (p=0; p<value.length(); ++p)
        {
        if (value[p] == '.') break;
        rh->nf_major_version += value[p];
        }
      p++; 
      for (; p<value.length(); ++p)
        {
        if (value[p] == '.') break;
        rh->nf_minor_version += value[p];
        }
      p++; 
      for (; p<value.length(); ++p)
        {
        rh->nf_micro_version += value[p];
        }
      }
    }

  else if (getvalue(line, F_Auto2TXT, value))
    {
    if (value == "1") rh->flags.nf_auto_inc = 1;
    else rh->flags.nf_auto_inc = 0;
    }
      
  else // Default if all else fails
    {
    buffer = "    Unrecognized line \"" + line + "\" in file \"";
    buffer += Default_FileTXT + "\" - SKIPPING";
    rh->logfile->write2log(buffer);
    }
  }
ifp.close();
buffer = "    Completed reading defaults file \"" + Default_FileTXT + "\"";
rh->logfile->write2log(buffer);
}

void Defaults::callback_np_want_micro_check_button_clicked(GtkWidget *widget, Defaults *nd)
{
if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(nd->np_want_micro_check_button)) )
  {
  gtk_button_set_label(GTK_BUTTON(nd->np_auto_inc_check_button), "Auto-increment MICRO version");
  gtk_widget_set_sensitive(nd->np_micro_vernum_entry, TRUE);
  }
else
  {
  gtk_button_set_label(GTK_BUTTON(nd->np_auto_inc_check_button), "Auto-increment MINOR version");
  gtk_widget_set_sensitive(nd->np_micro_vernum_entry, FALSE);
  }
}


//==============================================================================
//                                  PROJECT PAGE
//==============================================================================

void Defaults::fill_project_page(GtkWidget *vbox)
{
GtkWidget *widget, *grid, *grid2, *frame, *box;


	
// PROJECT EMAIL ADDRESS ====================================================

box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);

gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);
	
grid = gtk_grid_new();

gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
	
gtk_widget_show (grid);


gtk_box_pack_start(GTK_BOX(box), grid, TRUE, TRUE, 5);


widget = gtk_label_new( RevHead::P_EmailTXT.c_str());
gtk_misc_set_alignment (GTK_MISC (widget), 1, .5);
	
gtk_widget_show (widget);
	

gtk_grid_attach (GTK_GRID(grid), widget, 4, 1, 16, 5); // L, T, W, H
	
npowner_email_entry = gtk_entry_new();
	
gtk_entry_set_max_length(GTK_ENTRY(npowner_email_entry), 40);
if ( !rh->np_owner_email.empty() )
	{
  gtk_entry_set_text(GTK_ENTRY(npowner_email_entry), (char *)rh->np_owner_email.c_str() );
	}

gtk_grid_attach (GTK_GRID(grid), npowner_email_entry, 20, 1, 40, 5); // L, T, W, H


gtk_widget_set_tooltip_text (GTK_WIDGET(npowner_email_entry),
      "Email Address\n"
      "Enter the email address for a default project.");



	
// START VERSION FRAME =================================================

box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_set_homogeneous (GTK_BOX (box), FALSE);


gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);

frame = gtk_frame_new( "Start Version:" );
	
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);



gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Start Version\n"
      "Enter the default start version for a project.\n"
      "If auto-incrementation of the micro-version is required,\n"
      "tick the \"Auto-increment MICRO version\" checkbox.");




grid = gtk_grid_new();
gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_column_spacing( GTK_GRID(grid), 8);
		
gtk_container_set_border_width(GTK_CONTAINER(grid), 10);

gtk_widget_show (grid);

	
gtk_container_add(GTK_CONTAINER(frame), grid);

	
widget = gtk_label_new ( "Major" );
gtk_misc_set_alignment (GTK_MISC (widget), .5, 1);

gtk_grid_attach (GTK_GRID(grid), widget, 0, 0, 20, 1); // L, T, W, H


widget = gtk_label_new ( "Minor" );
gtk_misc_set_alignment (GTK_MISC (widget), .5, 1);

gtk_grid_attach (GTK_GRID(grid), widget, 20, 0, 20, 1); // L, T, W, H

  
widget = gtk_label_new ( "Micro" );
gtk_misc_set_alignment (GTK_MISC (widget), .5, 1);
   

gtk_grid_attach (GTK_GRID(grid), widget, 40, 0, 20, 1); // L, T, W, H

  
// This will need to be made longer
np_major_vernum_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(np_major_vernum_entry), 7);


	
if ( !rh->np_major_version.empty() )
	{
	gtk_entry_set_text(GTK_ENTRY(np_major_vernum_entry), (char *)rh->np_major_version.c_str() );
	}
	
gtk_grid_attach (GTK_GRID(grid), np_major_vernum_entry, 0, 1, 20, 1); // L, T, W, H


// This will need to be made longer
np_minor_vernum_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(np_minor_vernum_entry), 7);

	
if ( !rh->np_minor_version.empty() )
	{
	gtk_entry_set_text(GTK_ENTRY(np_minor_vernum_entry), (char *)rh->np_minor_version.c_str() );
	}

gtk_grid_attach (GTK_GRID(grid), np_minor_vernum_entry, 20, 1, 20, 1); // L, T, W, H


// This will need to be made longer
np_micro_vernum_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(np_micro_vernum_entry), 7);
	
if ( !rh->np_micro_version.empty() )
	{
	gtk_entry_set_text(GTK_ENTRY(np_micro_vernum_entry), (char *)rh->np_micro_version.c_str() );
	}
if (!rh->flags.np_want_micro)
	{
	gtk_widget_set_sensitive(np_micro_vernum_entry, FALSE);
	}
	

gtk_grid_attach (GTK_GRID(grid), np_micro_vernum_entry, 40, 1, 20, 1); // L, T, W, H


box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
gtk_box_set_homogeneous (GTK_BOX (box), FALSE);



gtk_grid_attach (GTK_GRID(grid), box, 0, 2, 60, 1); // L, T, W, H


np_auto_inc_check_button = gtk_check_button_new_with_label(
            (rh->flags.np_want_micro) ? "Auto-increment MICRO version" : "Auto-increment MINOR version");

gtk_box_pack_start(GTK_BOX(box), np_auto_inc_check_button, TRUE, TRUE, 0);  

	
gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (np_auto_inc_check_button), rh->flags.np_auto_inc);  
		
np_want_micro_check_button = gtk_check_button_new_with_label ("Want Micro Version");
gtk_box_pack_end( GTK_BOX(box), np_want_micro_check_button, FALSE, FALSE, 0);
gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (np_want_micro_check_button), rh->flags.np_want_micro);  


g_signal_connect(G_OBJECT (np_want_micro_check_button), "clicked",
        G_CALLBACK(callback_np_want_micro_check_button_clicked), this);



// CONTACT FRAME ===========================================================

box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_set_homogeneous (GTK_BOX (box), TRUE);

gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);



frame = gtk_frame_new( RevHead::ContactTXT.c_str() );

gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);


gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Contact\n"
      "Enter the default contact details for a project.");
	
	
widget = contact_el.create_entry_list();
	
gtk_container_set_border_width(GTK_CONTAINER(widget), 10);

gtk_container_add(GTK_CONTAINER(frame), widget);
	


if (rh->np_address.empty())
    {
    for (int n=3; n; n--)  // we want 3 entries anyway
        {
        contact_el.add_to_elist("", &contact_el);
        }
    }
else
    {
    list<string>::iterator i;
    for(i=rh->np_address.begin(); i!=rh->np_address.end(); ++i)
      {
      contact_el.add_to_elist( (*i) , &contact_el);
      }
    }

}

//==============================================================================
//                          COPYRIGHT PAGE
//==============================================================================

void Defaults::fill_copyright_page(GtkWidget *vbox)
{
GtkWidget *widget, *grid, *frame, *box;;

// Project Owner ======================================================

box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);

gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);
	
grid = gtk_grid_new();

gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
	
gtk_widget_show (grid);

gtk_box_pack_start(GTK_BOX(box), grid, TRUE, TRUE, 5);

// Project owner (copyright owner string)
widget = gtk_label_new( (char *)RevHead::P_OwnerTXT.c_str());
gtk_misc_set_alignment (GTK_MISC (widget), 1, .5);

gtk_grid_attach (GTK_GRID(grid), widget, 4, 1, 16, 5); // L, T, W, H


npowner_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(npowner_entry), 40);	
if ( !rh->np_owner.empty() )
	{
  gtk_entry_set_text(GTK_ENTRY(npowner_entry), (char *)rh->np_owner.c_str() );
	}


gtk_grid_attach (GTK_GRID(grid), npowner_entry, 20, 1, 40, 5); // L, T, W, H



gtk_widget_set_tooltip_text (GTK_WIDGET(npowner_entry),
      "Copyright Owner\n"
      "Enter the name of the copyright owner.");


	
// Project Copyright ======================================================

box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_set_homogeneous (GTK_BOX (box), TRUE);
gtk_grid_attach (GTK_GRID(grid), box, 2, 60, 60, 40); // L, T, W, H

frame = gtk_frame_new( RevHead::P_CopyrightTXT.c_str() );
	
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);


gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Project Copyright\n"
      "Enter the copyright details.");


widget = copyright_el.create_entry_list();
	
gtk_container_add(GTK_CONTAINER(frame), widget);
gtk_container_set_border_width(GTK_CONTAINER(widget), 10);


if (rh->np_copyright.empty())
    {
    for (int n=3; n; n--)  // we want 3 entries anyway
        {
        copyright_el.add_to_elist("", &copyright_el);
        }
    }
else
    {
    list<string>::iterator i;
    for(i=rh->np_copyright.begin(); i!=rh->np_copyright.end(); ++i)
      {
      copyright_el.add_to_elist( (*i) , &copyright_el);
      }
    }

}


//==============================================================================
//                             FILE PAGE
//==============================================================================

void Defaults::fill_file_page(GtkWidget *vbox)
{
GtkWidget *widget, *grid, *grid2, *frame, *box;


// FILE AUTHOR FRAME ===========================================================	
box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);


frame = gtk_frame_new( FileAuthorTXT.c_str() );
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);



	
grid = gtk_grid_new();

gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
gtk_container_set_border_width(GTK_CONTAINER(grid), 10);

gtk_widget_show (grid);

//gtk_box_pack_start(GTK_BOX(frame), grid, TRUE, TRUE, 5);

gtk_container_add(GTK_CONTAINER(frame), grid);




gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "File Author\n"
      "Enter default information about\n"
      "the author of the project's files");
 
// File Author
widget = gtk_label_new ( "Default File Author:"  );
gtk_misc_set_alignment (GTK_MISC (widget), 1, .5); 
gtk_widget_show (widget);
	

gtk_grid_attach (GTK_GRID(grid), widget, 4, 1, 16, 5);

nfauthor_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(nfauthor_entry), 40);	
	
if ( !rh->nf_author.empty() )
	{
  gtk_entry_set_text(GTK_ENTRY(nfauthor_entry), (char *)rh->nf_author.c_str() );
	}
	

gtk_grid_attach (GTK_GRID(grid), nfauthor_entry, 20, 1, 40, 5);

	
widget = gtk_label_new ( FileAuthorEmailTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (widget), 1, .5);
	

gtk_grid_attach (GTK_GRID(grid), widget, 4, 7, 16, 5);


nfauthor_email_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(nfauthor_email_entry), 40);
	
if ( !rh->nf_author_email.empty() )
	{
  gtk_entry_set_text(GTK_ENTRY(nfauthor_email_entry), (char *)rh->nf_author_email.c_str() );
	}

gtk_grid_attach (GTK_GRID(grid), nfauthor_email_entry, 20, 7, 40, 5);


// START FILE VERSION FRAME =================================================



frame = gtk_frame_new( "Start File Version:" );
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);


grid = gtk_grid_new();

gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
gtk_container_set_border_width(GTK_CONTAINER(grid), 10);
gtk_grid_set_column_spacing( GTK_GRID(grid), 8);
	
gtk_widget_show (grid);

gtk_container_add(GTK_CONTAINER(frame), grid);



gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "File Versions\n"
      "Enter a start version number\n"      
      "and tick \"Auto-increment FILE micro versions\"\n"
      "if auto-incrementaion is required.");
 
widget = gtk_label_new ( "Major" );
gtk_misc_set_alignment (GTK_MISC (widget), .5, 1);


gtk_grid_attach (GTK_GRID(grid), widget, 0, 0, 20, 1);	

widget = gtk_label_new ( "Minor" );
gtk_misc_set_alignment (GTK_MISC (widget), .5, 1);


gtk_grid_attach (GTK_GRID(grid), widget, 20, 0, 20, 1);	


	
widget = gtk_label_new ( "Micro" );
gtk_misc_set_alignment (GTK_MISC (widget), .5, 1);
 
gtk_grid_attach (GTK_GRID(grid), widget, 40, 0, 20, 1);	

  
  
// This will need to be made longer
nf_major_vernum_entry	= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(nf_major_vernum_entry), 7);
	
if ( !rh->nf_major_version.empty() ) 
	{
	gtk_entry_set_text(GTK_ENTRY(nf_major_vernum_entry), (char *)rh->nf_major_version.c_str() );
	}


gtk_grid_attach (GTK_GRID(grid), nf_major_vernum_entry, 0, 1, 20, 1);	
	

nf_minor_vernum_entry	= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(nf_minor_vernum_entry), 7);
	
if ( !rh->nf_minor_version.empty() )
	{
	gtk_entry_set_text(GTK_ENTRY(nf_minor_vernum_entry), (char *)rh->nf_minor_version.c_str() );
	}

gtk_grid_attach (GTK_GRID(grid), nf_minor_vernum_entry, 20, 1, 20, 1);	


// This will need to be made longer
//nf_micro_vernum_entry = gtk_entry_new_with_max_length(7);

nf_micro_vernum_entry	= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(nf_micro_vernum_entry), 7);

	
if ( !rh->nf_micro_version.empty() )
	{
	gtk_entry_set_text(GTK_ENTRY(nf_micro_vernum_entry), (char *)rh->nf_micro_version.c_str() );
	}

gtk_grid_attach (GTK_GRID(grid), nf_micro_vernum_entry, 40, 1, 20, 1);	



// AUTO-INCREMENT File Versions ------------------------------------------------
f_auto_inc_checkbutton = gtk_check_button_new_with_label("Auto-increment FILE micro versions");
gtk_grid_attach (GTK_GRID(grid), f_auto_inc_checkbutton, 0, 3, 32, 1);	
	
}

/****************************************
 * Get new project and file defaults    *
 * from the user and save if requested. *
 ****************************************/
void Defaults::get_new_defs(void)
{
GtkWidget *grid; // project version, file version, file author, copyright
GtkWidget *widget;
GtkWidget *box;
Date dt(rh);

using namespace misc;

rh->DestroyWidgets();

dt.date_log_operation(&RevHead::DefaultsTXT);

gtk_label_set_text (GTK_LABEL (rh->title), 
        "DEFAULTS FOR:  new project and new files");

grid = gtk_grid_new ();
gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous(GTK_GRID(grid), FALSE);
	
gtk_container_add (GTK_CONTAINER (rh->vbox), grid);
  
notebook = gtk_notebook_new ();  
gtk_notebook_set_tab_pos (GTK_NOTEBOOK(notebook), GTK_POS_TOP);
	
gtk_grid_attach (GTK_GRID(grid), notebook, 1, 0, 61, 70);
                 
widget = add_page(notebook, (gchar *)"Project Defaults");
fill_project_page(widget);


widget = add_page(notebook, (gchar *)"Copyright Defaults");
fill_copyright_page(widget);

  
widget = add_page(notebook, (gchar *)"File Defaults");
fill_file_page(widget);




/* Termination buttons */
box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
gtk_box_set_homogeneous (GTK_BOX (box), TRUE);


// CANCEL
widget = CreateButton(box, (char *)RevHead::CancelTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_CANCEL);
g_signal_connect(G_OBJECT (widget), "clicked",
        G_CALLBACK(callback_cancel), this);

// USE
widget = CreateButton(box, "USE");
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_APPLY);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(callback_use), this);

// SAVE & USE
widget = CreateButton(box, "SAVE & USE");
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_SAVE);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(callback_save_use), this);
	
gtk_grid_attach (GTK_GRID(grid), box, 0, 70, 64, 10);
 
 
gtk_widget_show_all(rh->vbox);
}





 
