/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       environment.h                                             *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       13-Apr-2013 18.50.19 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Header file for environment.cpp                                           *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef _ENVIRONMENT_H
#define _ENVIRONMENT_H

//#include "requester.h"
#include "entrylist.h"
#include "revhead.h"


class Environment
{
public:
  Environment(RevHead *revhead);
  ~Environment(void);

  void get_env(gboolean new_flag);
  void save_environment(Environment *ev);
  void load_environment(void);
  void update_changelog(void);

  static string Env_FileTXT;

private:
	static void callback_directory_selection(GtkWidget *widget, Environment *ev);

  static void callback_yes(GtkWidget *widget, Complain *cn);
  static void env_OK_func(GtkWidget *widget, Environment *ev);
  static void abort_func(GtkWidget *widget, Environment *ev);
  static void cancel_func(GtkWidget *widget, Environment *ev);
  static void callback_add_source(GtkWidget *widget, Environment *ev);
  static void callback_delete_source(GtkWidget *widget, Environment *ev);
  static void callback_assign_changelog(GtkWidget *widget, Environment *ev);
  static void callback_want_changelog(GtkWidget *widget, Environment *ev);
  static void callback_write_RH_run(GtkWidget *widget, Environment *ev);
  static void log_checkbox_clicked(GtkWidget *widget, Environment *ev);
  
  static void c_file_widget_clicked(GtkWidget *widget, Environment *ev);
  static void c_c_type_checkbox_clicked(GtkWidget *widget, Environment *ev);

  static void cpp_file_widget_clicked(GtkWidget *widget, Environment *ev);
  static void cpp_type_checkbox_clicked(GtkWidget *widget, Environment *ev);

  static void ada_file_widget_clicked(GtkWidget *widget, Environment *ev);
  static void ada_type_checkbox_clicked(GtkWidget *widget, Environment *ev);

  static void php_file_widget_clicked(GtkWidget *widget, Environment *ev);
  static void php_type_checkbox_clicked(GtkWidget *widget, Environment *ev);

  static void java_file_widget_clicked(GtkWidget *widget, Environment *ev);
  static void java_type_checkbox_clicked(GtkWidget *widget, Environment *ev);

  static void perl_file_widget_clicked(GtkWidget *widget, Environment *ev);
  static void perl_hash_type_checkbox_clicked(GtkWidget *widget, Environment *ev);

  static void fortran_file_widget_clicked(GtkWidget *widget, Environment *ev);
  static void fortran_type_checkbox_clicked(GtkWidget *widget, Environment *ev);

//  static void f_version_checkbutton_clicked(GtkWidget *widget, Environment *ev); // file versions

  // Callbacks for version directories
  static void callback_version_clicked(GtkWidget *widget, Environment *ev);
  
  static void callback_browse_c(GtkWidget *widget, Environment *ev);
  static void callback_delete_c(GtkWidget *widget, Environment *ev);

  static void callback_browse_ada(GtkWidget *widget, Environment *ev);
  static void callback_delete_ada(GtkWidget *widget, Environment *ev);

  static void callback_browse_php(GtkWidget *widget, Environment *ev);
  static void callback_delete_php(GtkWidget *widget, Environment *ev);

  static void callback_browse_java(GtkWidget *widget, Environment *ev);
  static void callback_delete_java(GtkWidget *widget, Environment *ev);

  static void callback_browse_perl(GtkWidget *widget, Environment *ev);
  static void callback_delete_perl(GtkWidget *widget, Environment *ev);

  static void callback_browse_fortran(GtkWidget *widget, Environment *ev);
  static void callback_delete_fortran(GtkWidget *widget, Environment *ev);

  static void callback_want_micro_clicked(GtkWidget *widget,  Environment *ev);

  list<string>::iterator insert_date_pname_version_comment(list<string>::iterator &i, bool overwrite);
  void add_columns (GtkTreeView *treeview, gchar *title);
  GtkTreeModel *create_model(list<string> mlist);
  GtkWidget *get_list_store(list<string> mlist);
  void add_new_source(string src, Environment *ev);

  void fill_project_page(GtkWidget *vbox);
  void fill_copyright_page(GtkWidget *vbox);
  void fill_language_page(GtkWidget *vbox);
  void fill_version_page(GtkWidget *vbox);
  void fill_directories_page(GtkWidget *vbox);
  void fill_logfile_page(GtkWidget *vbox);

  static GtkWidget *treeview; 

  static string RH_runTXT;


  RevHead *rh;
	
	string get_browse_directory(string title, Environment *ev);
	GtkWidget *directory_chooser;
	string dir_pathname;
	
  GtkWidget *pname_widget;
  GtkWidget *owner_email_entry;
  class EntryList contact_el;

  GtkWidget *owner_widget;
  class EntryList copyright_el;

  GtkWidget *c_file_widget;
  GtkWidget *c_c_type_checkbox;
  GtkWidget *c_version_checkbox;
  
  GtkWidget *cpp_file_widget;
  GtkWidget *cpp_c_type_checkbox;
  GtkWidget *cpp_cpp_type_checkbox;
  GtkWidget *cpp2_cpp_type_checkbox;
  
  GtkWidget *c_path_entry;
  GtkWidget *ada_path_entry;
  GtkWidget *php_path_entry;
  GtkWidget *java_path_entry;
  GtkWidget *perl_path_entry;
  GtkWidget *fortran_path_entry;

  GtkWidget *c_clear_button;
  GtkWidget *ada_clear_button;
  GtkWidget *php_clear_button;
  GtkWidget *java_clear_button;
  GtkWidget *perl_clear_button;
  GtkWidget *fortran_clear_button;

  GtkWidget *c_browse_button;
  GtkWidget *ada_browse_button;
  GtkWidget *php_browse_button;
  GtkWidget *java_browse_button;
  GtkWidget *perl_browse_button;
  GtkWidget *fortran_browse_button;



  
  GtkWidget *ada_file_widget;
  GtkWidget *ada_type_checkbox;
  GtkWidget *ada_version_checkbox;

  GtkWidget *php_file_widget;
  GtkWidget *php_c_type_checkbox;
  GtkWidget *php_cpp_type_checkbox;
  GtkWidget *php_cpp2_type_checkbox;
  GtkWidget *php_hash_type_checkbox;
  GtkWidget *php_version_checkbox;

  GtkWidget *java_file_widget;
  GtkWidget *java_c_type_checkbox;
  GtkWidget *java_cpp_type_checkbox;
  GtkWidget *java_cpp2_type_checkbox;
  GtkWidget *java_version_checkbox;

  GtkWidget *perl_file_widget;
  GtkWidget *perl_hash_type_checkbox;
  GtkWidget *perl_version_checkbox;

  GtkWidget *fortran_file_widget;
  GtkWidget *fortran_type_checkbox;
  GtkWidget *fortran_version_checkbox;

  GtkWidget *changelog_path_entry;
  GtkWidget *changelog_name_entry;
  GtkWidget *changelog_button;
  GtkWidget *changelog_checkbox;

  GtkWidget *none_log_checkbox;
  GtkWidget *append_log_checkbox;
  GtkWidget *overwrite_log_checkbox;
  GtkWidget *chosen_log_checkbox;
  GtkWidget *log2cerr_checkbox;

  GtkWidget *p_want_micro_checkbutton;
  GtkWidget *p_auto_inc_checkbutton;

  GtkWidget *f_auto_inc_checkbutton;
	
	GtkWidget *main_grid;

  GtkWidget *notebook;

  gboolean existing_project_flag;

    
protected:

};

#endif  // _ENVIRONMENT_H
