/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       filedetails.cpp                                           *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Apr-2013 10.04.48 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Split off from elsewhere.  Recursion was a problem.                       *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/


#include "filedetails.h"

#include "date.h"
#include "logfile.h"
#include "entrylist.h"
#include "tfile.h"
#include "misc.h"

using namespace std;
using namespace misc;


FileDetails::FileDetails(RevHead *revhead)
{
cout << "FileDetails:" << endl;
//GdkColor red  =   {0, 0xffff, 0x0000, 0x0000};
//GdkColor blue =   {0, 0x0000, 0x0000, 0xffff};
//GdkColor white =  {0, 0xffff, 0xffff, 0xffff};

rh = revhead;
author_entry = author_email_entry = version_frame = NULL;
file_logged = FALSE;
ud = NULL;

}

FileDetails::~FileDetails(void)
{      
cout << "~FileDetails:" << endl;

//if (comment_el) delete comment_el;
//comment_el = NULL;
}


// NOTE: "data" is "flag", 
// TRUE for existing file, FALSE for new file

void FileDetails::GetFDOKFunc(GtkWidget *widget,  FileDetails *fd)
{
gboolean    changed_flag = FALSE;
Date        dt(fd->rh);
class Files *f;
string      buffer;
string      fname = fd->rh->direntry->d_name;
  
remove_trailing(fname); // just in case...
remove_leading(fname);

buffer = "    Marking \"";
buffer += fname;
buffer +="\" for update";
fd->rh->logfile->report(buffer);

//cout << "FileDetails::GetFDOKFunc : " << buffer << endl;

f = new Files;
UNLESS (f)
    {
    fd->rh->quit->HandleError(MEMORY_F_ERR, fname.c_str(), NULL);
    return;
    }

cout << "FileDetails::GetFDOKFunc - 1" << endl;

f->author = (char *)gtk_entry_get_text(GTK_ENTRY(fd->author_entry));


cout << "FileDetails::GetFDOKFunc - 2" << endl;



  
remove_trailing(f->author);
remove_leading(f->author);
if (f->author != fd->ud->f_author)
  {
  changed_flag = TRUE;
  }

cout << "FileDetails::GetFDOKFunc - 3" << endl;


f->author_email = (char *)gtk_entry_get_text(GTK_ENTRY(fd->author_email_entry));

cout << "FileDetails::GetFDOKFunc - 4" << endl;

remove_trailing(f->author_email);
remove_leading(f->author_email);  
if (f->author != fd->ud->f_author_email)
  {
  changed_flag = TRUE;
  }

// VERSION
f->flags.version_wanted = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fd->f_version_checkbutton));
if (f->flags.version_wanted)
  {
  f->flags.no_auto_inc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fd->no_auto_checkbutton));
  }
else f->flags.no_auto_inc = 0;

//if (f->flags.version_wanted) cout << "FileDetails::GetFDOKFunc - version WANTED" << endl;
//else cout << "FileDetails::GetFDOKFunc - version NOT WANTED" << endl;

  
if (f->flags.version_wanted)
  {

cout << "FileDetails::GetFDOKFunc - 5" << endl;

  f->major_version = (char *)gtk_entry_get_text(GTK_ENTRY(fd->f_major_vernum_entry));

cout << "FileDetails::GetFDOKFunc - 6" << endl;

  remove_trailing(f->major_version);
  remove_leading(f->major_version);
  if (f->major_version != fd->ud->f_major_v)
    {
    changed_flag = TRUE;
    }
  f->minor_version = (char *)gtk_entry_get_text(GTK_ENTRY(fd->f_minor_vernum_entry));
  remove_trailing(f->minor_version);
  remove_leading(f->minor_version);
  if (f->minor_version != fd->ud->f_minor_v)
    {
    changed_flag = TRUE;
    }
  f->micro_version = (char *)gtk_entry_get_text(GTK_ENTRY(fd->f_micro_vernum_entry));
  remove_trailing(f->micro_version);
  remove_leading(f->micro_version);
  if (f->micro_version != fd->ud->f_micro_v)
    {
    changed_flag = TRUE;
    }  
  }

// SUMMARY
  
cout << "FileDetails::GetFDOKFunc - 7" << endl;


fd->summary_el.createstringlist(&fd->summary_el, f->summary); // WE NEED IT

cout << "FileDetails::GetFDOKFunc - 8" << endl;

if (!fd->summary_el.samestrings(fd->ud->f_summary, &fd->summary_el))
  {
cout << "FileDetails::GetFDOKFunc - 9.1 : summary CHANGED" << endl;
  f->flags.newsummary = 1;
  changed_flag = TRUE;
  }
else
  {
  f->flags.newsummary = 0;
  cout << "FileDetails::GetFDOKFunc - 9.2 :summary NOT CHANGED" << endl;
  }

cout << "FileDetails::GetFDOKFunc - 10" << endl;




/* Fix up details, COPY EVERYTHING!    */
f->datestamp = fd->ud->f_ds;
f->timezone  = fd->ud->f_tz;
f->daylight  = dt.get_daylight_saving(f->datestamp, f->timezone);

f->name = fname;
f->pathname = fd->rh->path + "/" + fname;


f->comment_type = fd->ud->f_comment_type;

cout << "FileDetails::GetFDOKFunc - 11" << endl;

fd->comment_el.createstringlist(&fd->comment_el, f->comment);

if (!f->comment.empty()) changed_flag = TRUE; // NOTE: comments not in header though

f->flags.logged = fd->file_logged;  // FALSE if not logged
                                    // TRUE if logged

f->flags.changed = changed_flag;    // TRUE if file details changed
fd->rh->file_list.push_back(*f);

  
while (! fd->ud->CheckFiles() ) {;}
UNLESS (fd->rh->rc == EXIT_SUCCESS) return;
fd->ud->FixFileData();

cout << "FileDetails::GetFDOKFunc - 9" << endl;


}



// We quit completely
void FileDetails::AbortFunc(GtkWidget *widget,  FileDetails *fd)
{
fd->rh->DestroyWidgets();
fd->rh->flags.abort = 1;
fd->rh->quit->HandleError(USER_ABORT_ERR, NULL, NULL);
return;
}


/* This is "DON'T UPDATE" i.e. skip this file */
void FileDetails::GetFDSkipFunc(GtkWidget *widget,  FileDetails *fd)
{
string buffer;
    
buffer = "    Skipping \"";
buffer += fd->rh->direntry->d_name;
buffer += "\" (user request)";
fd->rh->logfile->write2log(buffer);
  
while (! fd->ud->CheckFiles() ) {;}
UNLESS (fd->rh->rc == EXIT_SUCCESS) return;
fd->ud->FixFileData();
}

void FileDetails::callback_f_version_checkbutton_clicked(GtkWidget *widget, FileDetails *fd)
{
if (fd->version_frame)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fd->f_version_checkbutton))==TRUE)
    {
    gtk_widget_set_sensitive(GTK_WIDGET(fd->version_frame), TRUE);
    gtk_widget_set_sensitive(GTK_WIDGET(fd->no_auto_checkbutton), TRUE);
    }
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(fd->version_frame), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(fd->no_auto_checkbutton), FALSE);
    }
  }
}

void FileDetails::callback_no_auto_checkbutton_clicked(GtkWidget *widget, FileDetails *fd)
{
string buffer;
if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fd->no_auto_checkbutton))==TRUE)
  {
  buffer = (fd->ud->f_micro_v.empty()) ? "?" : fd->ud->f_micro_v.c_str();
  }
else
  {
  if (!fd->ud->f_micro_v.empty())
    {
    buffer = fd->ud->f_micro_v;
    }
  else if (!fd->rh->nf_micro_version.empty())
    {
    buffer = fd->rh->nf_micro_version;
    }
  else buffer = "?";
  if ((fd->file_logged) && (fd->rh->flags.f_auto_inc) && (!fd->ud->f_no_auto_inc) && (buffer != "?"))
    {
    TFile tf;
    long v = atol(  (char *)buffer.c_str() );
    v++;
    buffer = tf.itoa(v);
    }
  }

gtk_entry_set_text(GTK_ENTRY(fd->f_micro_vernum_entry), buffer.c_str());  
}

//==============================================================================
//                           AUTHOR PAGE
//==============================================================================
void FileDetails::fill_author_page(GtkWidget *vbox)
{
GtkWidget *wid, *frame, *grid, *box, *vbox2, *hbox;
string buffer;


box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_set_homogeneous (GTK_BOX(box), TRUE);
gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);
	

if (!ud->f_old_name.empty())
  {
  buffer = "WARNING: Filename has changed, previous name \"" + ud->f_old_name + "\"";
  wid = gtk_label_new( (gchar *)buffer.c_str() );			
	decorate_widget(wid, &red, NULL, NULL, NULL, NULL);
	gtk_box_pack_start(GTK_BOX(box), wid, TRUE, TRUE, 5);

  }

// FILE AUTHOR FRAME ===========================================================	
	
frame = gtk_frame_new("File's Author:");
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "File's Author\n"
      "Insert this file's author and email address");
                             
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);

vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_set_homogeneous (GTK_BOX(vbox2), TRUE);
gtk_container_add(GTK_CONTAINER(frame), vbox2);


hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_set_homogeneous (GTK_BOX(hbox), TRUE);
gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 5);


// AUTHOR
wid = gtk_label_new ( (gchar *)RevHead::AuthorTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);  // Align the label to the right

gtk_box_pack_start(GTK_BOX(hbox), wid, TRUE, TRUE, 5);


// This will need to be made longer

author_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(author_entry), 40);

if (file_logged) gtk_entry_set_text(GTK_ENTRY(author_entry), (char *)ud->f_author.c_str() );
else gtk_entry_set_text(GTK_ENTRY(author_entry), (char *)rh->nf_author.c_str() );

gtk_box_pack_start(GTK_BOX(hbox), author_entry, TRUE, TRUE, 5);



hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_set_homogeneous (GTK_BOX(hbox), TRUE);
gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 5);


	
// Author's Email Address
wid = gtk_label_new ( RevHead::AuthorEmailTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);

gtk_box_pack_start(GTK_BOX(hbox), wid, TRUE, TRUE, 5);


author_email_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(author_email_entry), 40);

if (file_logged) gtk_entry_set_text(GTK_ENTRY(author_email_entry), (char *)ud->f_author_email.c_str() );
else gtk_entry_set_text(GTK_ENTRY(author_email_entry), (char *)rh->nf_author_email.c_str() );


gtk_box_pack_start(GTK_BOX(hbox), author_email_entry, TRUE, TRUE, 5);



hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_set_homogeneous (GTK_BOX(hbox), TRUE);
gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 5);

f_version_checkbutton = gtk_check_button_new_with_label("FILE version required for this file");
g_signal_connect (G_OBJECT (f_version_checkbutton), "clicked",
            G_CALLBACK(callback_f_version_checkbutton_clicked), this);

gtk_box_pack_start(GTK_BOX(hbox), f_version_checkbutton, TRUE, TRUE, 5);


hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_set_homogeneous (GTK_BOX(hbox), TRUE);
gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 5);

no_auto_checkbutton = gtk_check_button_new_with_label("Don't auto-increment this file version");
g_signal_connect (G_OBJECT (no_auto_checkbutton), "clicked",
            G_CALLBACK(callback_no_auto_checkbutton_clicked), this);

gtk_box_pack_start(GTK_BOX(hbox), no_auto_checkbutton, TRUE, TRUE, 5);


	
gtk_widget_set_sensitive(GTK_WIDGET(no_auto_checkbutton), ud->f_want_version);
gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(no_auto_checkbutton), ud->f_no_auto_inc);





	
// FILE VERSION FRAME =================================================
version_frame = gtk_frame_new(RevHead::F_VersionTXT.c_str());
gtk_widget_set_tooltip_text (GTK_WIDGET(version_frame),
      "File Version\n"
      "Edit this file's major, minor and micro version numbers");
	
gtk_container_set_border_width(GTK_CONTAINER(version_frame), 10);

gtk_box_pack_start(GTK_BOX(box), version_frame, TRUE, TRUE, 5);

grid = gtk_grid_new();
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);


gtk_container_set_border_width(GTK_CONTAINER(grid), 10);
gtk_grid_set_column_spacing( GTK_GRID(grid), 8);
	
gtk_container_add(GTK_CONTAINER(version_frame), grid);
                           
wid = gtk_label_new( "Major" );
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);
gtk_grid_attach (GTK_GRID(grid), wid, 15, 0, 15, 1);	
  
 
wid = gtk_label_new( "Minor" );
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);
gtk_grid_attach (GTK_GRID(grid), wid, 30, 0, 15, 1);	
  
  
wid = gtk_label_new( "Micro" );
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);
gtk_grid_attach (GTK_GRID(grid), wid, 45, 0, 15, 1);	




wid = gtk_label_new( (char *)RevHead::F_VersionTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5); //Align the label to the left
gtk_grid_attach (GTK_GRID(grid), wid, 0, 1, 15, 1);	



f_major_vernum_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(f_major_vernum_entry), 7);
	
if (!ud->f_major_v.empty())
  {
  buffer = ud->f_major_v;
  }
else if (!rh->nf_major_version.empty())
  {
  buffer = rh->nf_major_version;
  }
else buffer = "?";
gtk_entry_set_text(GTK_ENTRY(f_major_vernum_entry), buffer.c_str());

gtk_grid_attach (GTK_GRID(grid), f_major_vernum_entry, 15, 1, 15, 1);	
	

f_minor_vernum_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(f_minor_vernum_entry), 7);
	
if (!ud->f_minor_v.empty())
  {
  buffer = ud->f_minor_v;
  }
else if (!rh->nf_minor_version.empty())
  {
  buffer = rh->nf_minor_version.c_str();
  }
else buffer = "?";
gtk_entry_set_text(GTK_ENTRY(f_minor_vernum_entry), buffer.c_str()); 

gtk_grid_attach (GTK_GRID(grid), f_minor_vernum_entry, 30, 1, 15, 1);	

  
f_micro_vernum_entry	= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(f_micro_vernum_entry), 7);

if (!ud->f_micro_v.empty())
  {
  buffer = ud->f_micro_v;
  }
else if (!rh->nf_micro_version.empty())
  {
  buffer = rh->nf_micro_version;
  }
else buffer = "?";
if ((file_logged) && (rh->flags.f_auto_inc) && (!ud->f_no_auto_inc) && (buffer != "?"))
  {
  TFile tf;
  long v = atol(  (char *)buffer.c_str() );
  v++;
  buffer = tf.itoa(v);
  }
gtk_entry_set_text(GTK_ENTRY(f_micro_vernum_entry), buffer.c_str());  

gtk_grid_attach (GTK_GRID(grid), f_micro_vernum_entry, 45, 1, 15, 1);	

                             
// Current Version
wid = gtk_label_new ("Current Version:");
decorate_widget(wid, &blue, NULL, NULL, NULL, NULL);
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5); //Align the label to the left
gtk_grid_attach (GTK_GRID(grid), wid, 0, 2, 15, 1);	



wid = gtk_label_new( (ud->f_major_v.empty()) ? "?" : ud->f_major_v.c_str() );
decorate_widget(wid, &blue, NULL, NULL, NULL, NULL);
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);  // Align to the centre
gtk_grid_attach (GTK_GRID(grid), wid, 15, 2, 15, 1);	
	
  
wid = gtk_label_new( (ud->f_minor_v.empty()) ? "?" : ud->f_minor_v.c_str() );  
decorate_widget(wid, &blue, NULL, NULL, NULL, NULL);
	
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 30, 2, 15, 1);	

wid = gtk_label_new( (ud->f_micro_v.empty()) ? "?" : ud->f_micro_v.c_str() );  
decorate_widget(wid, &blue, NULL, NULL, NULL, NULL);
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 45, 2, 15, 1);	
	
gtk_widget_set_sensitive(GTK_WIDGET(version_frame), ((ud->f_want_version) ? TRUE : FALSE) );
gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f_version_checkbutton), ud->f_want_version);
}




//==============================================================================
//                           COMMENT PAGE
//==============================================================================
void FileDetails::fill_comment_page(GtkWidget *vbox)
{
GtkWidget *wid, *grid, *frame, *box;

	
box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_set_homogeneous (GTK_BOX(box), TRUE),
gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);


// COMMENTS

frame = gtk_frame_new(RevHead::CommentsTXT.c_str());
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Comments\n"
      "Insert comments for this version of the file");
                             

gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);
	
wid = comment_el.create_entry_list();
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);

gtk_container_add(GTK_CONTAINER(frame), wid);


	
if (!ud->f_old_name.empty())
  {
  string buffer = "Filename has changed, previous name \"" + ud->f_old_name + "\"";
  comment_el.add_to_elist(buffer.c_str(), &comment_el);
  }
  
for (int n=3; n; n--) // give 3 empty lines as default
    {
    comment_el.add_to_elist("", &comment_el);
    }


// SUMMARY

frame = gtk_frame_new( (!file_logged) ? 
                      RevHead::F_SummaryTXT.c_str() :
                      ((ud->f_summary.empty()) ?
                      "CREATE File Summary" : "EDIT Existing Summary:") );
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
       "Summary\n"
      "Edit this file's summary if required");


gtk_container_add(GTK_CONTAINER(box), frame);

                            
wid = summary_el.create_entry_list();
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);

gtk_container_add(GTK_CONTAINER(frame), wid);


if (!ud->f_summary.empty())         // i.e. if this file's summary wasn't empty
  {
  list<string>::iterator i;
  for (i=ud->f_summary.begin(); i!=ud->f_summary.end(); ++i)
    {

// Could check here what the summary is

//cout << "FileDetails::fill_comment_page : " << (*i) << endl;

      
    summary_el.add_to_elist( (*i) , &summary_el);
    }
  }
else for (int n=3; n; n--) // give 3 empty lines as default
    {
    summary_el.add_to_elist("", &summary_el);
    }

// Can we read the entrylist here to check it out?
// EntryList::display_list(&summary_el);

}







/****************************************
 * void GetFDetails(fname,flag)         *
 * Get gadget input for file            *
 * flag = FALSE means not yet logged.   *
 ****************************************/
void FileDetails::get_file_details(class Update *update, char *fname, gboolean flag)
{
GtkWidget *grid;
GtkWidget *hbox;
GtkWidget *widget;
string buffer;

ud = update;

rh->DestroyWidgets();
//tooltips = gtk_tooltips_new();

file_logged = flag;
  
buffer = "Current directory is \"" + rh->path + "\"";
gtk_label_set_text (GTK_LABEL (rh->title), (char *)buffer.c_str() );


buffer = "    Getting details of ";
if (file_logged)
  {
  buffer += "EXISTING";
  }
else
  {
  buffer += "NEW";
  }
buffer += " file \"";
buffer += fname;
buffer += "\"";
rh->logfile->write2log(buffer);
gtk_label_set_text (GTK_LABEL (rh->status), (char *)buffer.c_str() );
  


grid = gtk_grid_new ();
gtk_box_pack_start (GTK_BOX(rh->vbox), grid, TRUE, TRUE, 0);
	
gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
	
notebook = gtk_notebook_new();  
gtk_notebook_set_tab_pos (GTK_NOTEBOOK(notebook), GTK_POS_TOP);

gtk_grid_attach (GTK_GRID(grid), notebook, 0, 0, 64, 72); // L, T, W, H

	
widget = add_page(notebook, (gchar *)"Author & Contact & Version"); 
fill_author_page(widget);
 
widget = add_page(notebook, (gchar *)"Summary & Comments");
fill_comment_page(widget);



/*
  Termination buttons  
  NOTE: flag is TRUE for existing project 
  and FALSE if it already exists
*/
	
hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
gtk_box_set_homogeneous (GTK_BOX(hbox), FALSE);
gtk_container_add(GTK_CONTAINER(rh->vbox), hbox);
	

// ABORT EVERYTHING
widget = CreateButton(hbox, (char *)RevHead::QuitAllTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_QUIT);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(AbortFunc), this);

// Middle button
widget = CreateButton(hbox, (flag) ? (char *)RevHead::NoUpdateTXT.c_str() : (char *)RevHead::NoLogTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_CANCEL);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(GetFDSkipFunc), this);

// OK, DO IT!
widget = CreateButton(hbox, (char *)RevHead::OKTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_OK);
g_signal_connect (G_OBJECT (widget), "clicked",
         G_CALLBACK(GetFDOKFunc), this);

//GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
//gtk_widget_grab_default(widget);
  
gtk_widget_show_all(rh->vbox);

// cout << "FileDetails::get_file_details - LEAVING" << endl;
gtk_main_quit(); // exit existing level
gtk_main();      // Start new level : seems we have to have this
}





/* END OF FILE */
