/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       getproject.cpp                                            *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Apr-2013 16.51.46 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Several input routines                                                    *
* 30-Sep-2001 16.21.33 -36000  0.00    David M. Balean                      *
* 30-Sep-2001 16.21.33 -36000  0.01    David M. Balean                      *
* Minor changes                                                             *
* 30-Sep-2001 16.21.33 -36000  0.02    David M. Balean                      *
* Dating major operations in the log file                                   *
* 30-Sep-2001 16.21.33 -36000  0.03    David M. Balean                      *
* Added code to delete RH.log & temp log if user doesn't want log           *
* 30-Sep-2001 16.21.33 -36000  0.04    David M. Balean                      *
* Added code for incrementing version numbers and user input                *
* 30-Sep-2001 16.21.33 -36000  0.05    David M. Balean                      *
* "flag" meanining was inverted if from front panel...                      *
* 30-Sep-2001 16.21.33 -36000  0.06    David M. Balean                      *
* Bugs cleared in version number incrementing                               *
* 30-Sep-2001 16.21.33 -36000  0.7     David M. Balean                      *
* Added a function call to get current datestamp                            *
* 15-Oct-2001 16.13.55 -36000  0.8     David M. Balean                      *
* The incorrect datestamp was passed to GetFDetails from CheckFiles, fixed  *
* 19-Oct-2001 14.14.40 -36000 0.9     David M. Balean                       *
* 20-Aug-2004 12.10.29 -36000 0.13    David M. Balean                       *
* Allows editting of the project part of the header, basically very similar *
* to previous versions.                                                     *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/


#include "getproject.h"

#include "entrylist.h"
#include "complain.h"
#include "date.h"
#include "update.h"
#include "tfile.h"
#include "misc.h"
#include "logfile.h"
#include "header.h"
#include "environment.h"
#include "misc.h"

extern char **environ;

using namespace std;
using namespace misc;


  
GetProject::GetProject(RevHead *revhead)
{
cout << "GetProject:" << endl;

GdkColor blue =   {0, 0x0000, 0x0000, 0xffff};
GdkColor white =  {0, 0xffff, 0xffff, 0xffff};

rh = revhead;
//contact_el = comment_el = NULL;

existing_project_flag = FALSE;

// Zero the widgets
pathname_widget =
pname_widget =
choose_widget =
chosen_widget =
c_file_widget =
cpp_file_widget =
a_file_widget =
p_auto_inc_widget =
f_auto_inc_widget =
shell_widget =
binary_widget =
c_straight_widget =
asm_straight_widget =
owner_widget =
owner_email_entry =
major_vernum_entry =
minor_vernum_entry =
micro_vernum_entry = 
log_widget = NULL;

//force_ud_widget = NULL;

ev_list = NULL;
ud = NULL;

//blue_style = CreateStyle(  blue,  blue,  white);
//                           fg   text      bg

}

GetProject::~GetProject(void)
{
GList *node;

cout << "~GetProject:" << endl;

for (node=ev_list; node; node = node->next)
    {
    delete( (Environment *)node->data);
    }
g_list_free(ev_list);
ev_list = NULL;

//if (copyright_el) delete copyright_el;
//copyright_el = NULL;

//if (contact_el) delete contact_el;
//contact_el = NULL;

//if (comment_el) delete comment_el;
//comment_el = NULL;

//if (summary_el) delete summary_el;
//summary_el = NULL;

}


// We quit completely
void GetProject::callback_abort_all(GtkWidget *widget, GetProject *gp)
{
gp->rh->DestroyWidgets();
gp->rh->flags.abort = 1;
gp->rh->quit->HandleError(USER_ABORT_ERR, NULL, NULL);
}


// We quit keeping the existing values in RevHead
void GetProject::callback_cancel(GtkWidget *widget, GetProject *gp)
{
gp->rh->FrontPanel();
}



void GetProject::continue_update(GetProject *gp)
{
string buffer;

if (!gp->existing_project_flag)  // NOT AN EXISTING PROJECT
  {

  if (gp->ud->CheckFiles()) // returns TRUE when finished
    {
    gp->ud->FixFileData();
    }
    
  gp->rh->FrontPanel();
  return;
  }


// Now check out which files have changed etc
gp->rh->dp = NULL;
gp->rh->direntry = NULL;

gp->rh->DestroyWidgets();

buffer = gp->rh->p_name + " : current path= \"" + rh->path + "\"";
gtk_label_set_text(GTK_LABEL(gp->rh->title), buffer.c_str() );
  
while (! gp->ud->CheckFiles() ) {;}
UNLESS (gp->rh->rc == EXIT_SUCCESS) return;
gp->ud->FixFileData();
}




void GetProject::callback_yes(GtkWidget *widget, Complain *cn)
{
GetProject *gp =  (GetProject *)(cn->data);
cn->end_complain(widget, cn);
  
gp->continue_update(gp);  

}



// NOTE: "flag" is TRUE for existing project, FALSE for new project
void GetProject::callback_OK(GtkWidget *widget, GetProject *gp)
{
char *c;
string buffer;
Date dt(gp->rh);

gp->ud = gp->rh->ud;

using namespace misc;


// Project Name
c = (char *)gtk_entry_get_text(GTK_ENTRY(gp->pname_widget));

UNLESS (gp->existing_project_flag)
    {
    buffer = "LOGGING NEW PROJECT \"";
    buffer += c;
    buffer += '\"';
    dt.date_log_operation(&buffer);
    }

if (!gp->rh->p_name.empty() && STREQ(c, (char *)gp->rh->p_name.c_str()))
    gp->rh->flags.newname = 0;
else
    {
    gp->rh->p_name = c;
    gp->rh->flags.newname = 1;
    }

/* Project Version */
c = (char *)gtk_entry_get_text(GTK_ENTRY(gp->major_vernum_entry));

if (gp->rh->p_major_version == c)
    gp->rh->flags.new_major_version = 0;
else
    {
    gp->rh->p_major_version = c;
    gp->rh->flags.new_major_version = 1;
    }
  
c = (char *)gtk_entry_get_text(GTK_ENTRY(gp->minor_vernum_entry));
if (gp->rh->p_minor_version == c)
    gp->rh->flags.new_minor_version = 0;
else
    {
    gp->rh->p_minor_version = c;
    gp->rh->flags.new_minor_version = 1;
    }

if (gp->rh->flags.p_want_micro)
  {
  c = (char *)gtk_entry_get_text(GTK_ENTRY(gp->micro_vernum_entry));
  if (gp->rh->p_micro_version == c)
    gp->rh->flags.new_micro_version = 0;
  else
    {
    gp->rh->p_micro_version = c;
    gp->rh->flags.new_micro_version = 1;
    }
  }
else gp->rh->flags.new_micro_version = 0;

// The copyright owner 
c = (char *)gtk_entry_get_text(GTK_ENTRY(gp->owner_widget));
if (!gp->rh->p_owner.empty() && STREQ(c, (char *)gp->rh->p_owner.c_str()))
  {
  gp->rh->flags.newowner = 0;
  }
else
  {
  gp->rh->p_owner = c;
  gp->rh->flags.newowner = 1;
  }


// Project Email  
c = (char *)gtk_entry_get_text(GTK_ENTRY(gp->owner_email_entry));
if ( STREQ(c, (char *)gp->rh->p_owner_email.c_str()))
  {
  gp->rh->flags.newemail = 0;
  }
else 
  {
  gp->rh->p_owner_email = c;
  gp->rh->flags.newemail = 1;
  }
  
  
// Project Copyright Notice
if (gp->copyright_el.samestrings(gp->rh->p_copyright, &gp->copyright_el))
  {
  gp->rh->flags.newcopyright = 0;
  }
else
  {
  gp->rh->flags.newcopyright = 1;
  gp->copyright_el.createstringlist(&gp->copyright_el, gp->rh->p_copyright);
  }  
//delete gp->copyright_el;
//gp->copyright_el = NULL;


// Contact Address
if (gp->contact_el.samestrings(gp->rh->p_address, &gp->contact_el))
  { 
  gp->rh->flags.newaddress = 0;
  }
else gp->rh->flags.newaddress = 0;
  {
  gp->rh->flags.newaddress = 1;
  gp->contact_el.createstringlist(&gp->contact_el, gp->rh->p_address);    
  }  
//delete gp->contact_el;
//gp->contact_el = NULL;
  

// Comments
gp->comment_el.createstringlist(&gp->comment_el, gp->rh->p_comment);
gp->rh->flags.pc_changed = (gp->rh->p_comment.empty()) ? 0 : 1;
//delete gp->comment_el;
//gp->comment_el = NULL;
	
	
// SUMMARY
if (gp->summary_el.samestrings(gp->rh->p_summary, &gp->summary_el))
  {	
  gp->rh->flags.newsummary = 0;
  }
else
  {  
	gp->rh->flags.newsummary = 1;
	gp->summary_el.createstringlist(&gp->summary_el, gp->rh->p_summary);
  }
//delete gp->summary_el;
//gp->summary_el = NULL;

//if ( (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (gp->force_ud_widget))!=0) ||
if (
      gp->rh->flags.newname           ||
      gp->rh->flags.newowner          ||    
      gp->rh->flags.newemail          ||
      gp->rh->flags.new_major_version ||
      gp->rh->flags.new_minor_version ||
      gp->rh->flags.new_micro_version ||
      gp->rh->flags.newcopyright      ||
      gp->rh->flags.newaddress        ||
      gp->rh->flags.newsummary        ||
      gp->rh->flags.pc_changed )
  {
  gp->rh->flags.p_changed = 1;
  }    
else   gp->rh->flags.p_changed = 0;  

// We need a check here that ChangeLog and source files are in the project path
list<string>::iterator src;
buffer.clear();
int num_warns = 0;  

if ( ( (gp->rh->changelog_path.length() >= gp->rh->project_path.length()) &&
  (gp->rh->changelog_path.substr(0, gp->rh->project_path.length()) != gp->rh->project_path) ) ||
  ((!gp->rh->changelog_path.empty()) && (gp->rh->changelog_path.length() < gp->rh->project_path.length())) )    
  {
  num_warns++;
  buffer += "\nChangeLog path \"" +  gp->rh->changelog_path + "\"";
  }

for (src = gp->rh->source_path.begin();
          src!=gp->rh->source_path.end();
          ++src)
  {
  if ( ( (*src).length() >= gp->rh->project_path.length() ) &&
    ( (*src).substr(0, gp->rh->project_path.length()) != gp->rh->project_path ) ||
      ( (*src).length() < gp->rh->project_path.length() ) )
    {
    num_warns++;
    buffer += "\nSource path \"" +  (*src) + "\"";
    }
  }
if (num_warns)
  {
  Complain *cn = new Complain(gp->rh->icon_pixbuf, &lightgrey, &pink, GTK_STOCK_CANCEL, GTK_STOCK_OK);

  string warning =   "    WARNING!\nThe following ";
  warning += ((num_warns > 1) ? "are" : "is");
  warning += " not in the project path\nwhich is \"" +
             gp->rh->project_path + "\".\n" + buffer + "\nIs this OK?"; 
    
  cn->complain(CN_YES_NO,
                (char *)"PATH WARNING",
                (char *)warning.c_str(),
                callback_yes, NULL, gp);
  return;
  }

// ==========================
// IF NOT AN EXISTING PROJECT
// ==========================
if (!gp->existing_project_flag)  // NOT AN EXISTING PROJECT
  {

  if (gp->ud->CheckFiles()) // returns TRUE when finished
    {
    gp->ud->FixFileData();
    }
    
  gp->rh->FrontPanel();
  return;
  }

// =============================
// OTHERWISE AN EXISTING PROJECT
// =============================

// Now check out which files have changed etc
gp->rh->dp = NULL;
gp->rh->direntry = NULL;

gp->rh->DestroyWidgets();

buffer = gp->rh->p_name + " : current path= \"" + gp->rh->path + "\"";
gtk_label_set_text(GTK_LABEL(gp->rh->title), buffer.c_str() );
  
while (! gp->ud->CheckFiles() ) {;}
UNLESS (gp->rh->rc == EXIT_SUCCESS) return;
gp->ud->FixFileData();

}



// We want to fix the environment
void GetProject::callback_get_env(GtkWidget *widget,  GetProject *gp)
{
Environment *ev = new Environment(gp->rh);
gp->ev_list = g_list_append(gp->ev_list, ev);
ev->get_env(TRUE);
}


//========================================================================
//                      THE PAGES
//========================================================================

//===============
// PROJECT PAGE
//===============
void GetProject::fill_name_page(GtkWidget *vbox)
{
string buffer;
GtkWidget *wid, *grid,  *frame, *box;
	
grid = gtk_grid_new();
gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_column_spacing (GTK_GRID(grid), 4);
gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, TRUE, 5);


// PROJECT
frame = gtk_frame_new("Project:");
		
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Project\n"
      "Project Name:\n  The name for this project without any version number\n"
      "Project's Email:\n  The email address for the whole project");

gtk_grid_attach (GTK_GRID(grid), frame, 0, 0, 64, 14);


                             
// PPROJECT NAME
wid = gtk_label_new( RevHead::P_NameTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);
gtk_grid_attach (GTK_GRID(grid), wid, 0, 2, 10, 6);



// This will need to be made longer
pname_widget = gtk_entry_new();

gtk_entry_set_max_length(GTK_ENTRY(pname_widget), 30);
	
if ( existing_project_flag && (!rh->p_name.empty()) )		// EXISTING PROJECT
    gtk_entry_set_text(GTK_ENTRY(pname_widget),  rh->p_name.c_str() );
else
    gtk_entry_set_text(GTK_ENTRY(pname_widget), "(UNNAMED PROJECT)" );


gtk_grid_attach (GTK_GRID(grid), pname_widget, 10, 2, 30, 6);



// Project Email Address
wid = gtk_label_new( RevHead::P_EmailTXT.c_str());
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 0, 8, 10, 6);



owner_email_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(owner_email_entry), 40);


	
if (existing_project_flag) gtk_entry_set_text(GTK_ENTRY(owner_email_entry), (char *)rh->p_owner_email.c_str() );
else gtk_entry_set_text(GTK_ENTRY(owner_email_entry), rh->np_owner_email.c_str() );

gtk_grid_attach (GTK_GRID(grid), owner_email_entry, 10, 8, 30, 6);






// PROJECT VERSION
frame = gtk_frame_new("Version:");
		
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Version\n"
      "The major, minor and micro version numbers for this update");


gtk_grid_attach (GTK_GRID(grid), frame, 0, 17, 64, 20);



	 
wid = gtk_label_new( "Major" );
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);


gtk_grid_attach (GTK_GRID(grid), wid, 10, 18, 15, 6);

wid = gtk_label_new( "Minor" );
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);


gtk_grid_attach (GTK_GRID(grid), wid, 25, 18, 15, 6);

if (rh->flags.p_want_micro)
  {
	wid = gtk_label_new( "Micro" );
	gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);
 
	gtk_grid_attach (GTK_GRID(grid), wid, 40, 18, 15, 6);

 }

	


wid = gtk_label_new( (char *)RevHead::P_VersionTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 0, 24, 10, 6);
	



//  WHAT"S WRONG? 
//  Whatever I do increases the width of the containing widget dramatically
//  The problem is that an entry widget has a minimum width of 15!

	
major_vernum_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(major_vernum_entry), 6);

gtk_grid_attach (GTK_GRID(grid), major_vernum_entry, 10, 24, 15, 6);
	

if (rh->p_major_version.empty() || rh->p_major_version=="?")
  {
  gtk_entry_set_text(GTK_ENTRY(major_vernum_entry), rh->np_major_version.c_str());
  }
else
  {
  gtk_entry_set_text(GTK_ENTRY(major_vernum_entry), rh->p_major_version.c_str());
  }


 
minor_vernum_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(minor_vernum_entry), 7);

	
gtk_widget_show (minor_vernum_entry);
if (rh->p_minor_version.empty() || rh->p_minor_version=="?")
  {
  gtk_entry_set_text(GTK_ENTRY(minor_vernum_entry), rh->np_minor_version.c_str());
  }

else if ((!rh->flags.p_want_micro) && rh->flags.p_auto_inc)  // autoincrement existing project, no micro version
  {
  TFile tf;
  long v = atol(  (char *)rh->p_minor_version.c_str() );
  v++;
  buffer = tf.itoa(v);
  gtk_entry_set_text(GTK_ENTRY(minor_vernum_entry), buffer.c_str()); 
  } 
else gtk_entry_set_text(GTK_ENTRY(minor_vernum_entry), rh->p_minor_version.c_str()); 


gtk_grid_attach (GTK_GRID(grid), minor_vernum_entry,25, 24, 15, 6);



if (rh->flags.p_want_micro)
  {

	micro_vernum_entry = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(micro_vernum_entry), 7);

		
  gtk_widget_show (micro_vernum_entry);

  if (rh->p_micro_version.empty() || rh->p_micro_version=="?")
    {
    gtk_entry_set_text(GTK_ENTRY(micro_vernum_entry), rh->np_micro_version.c_str());
    }
  else if (rh->flags.p_auto_inc) // existing project
    {
    TFile tf;
    long v = atol(  (char *)rh->p_micro_version.c_str() );
    v++;
    buffer = tf.itoa(v);
    gtk_entry_set_text(GTK_ENTRY(micro_vernum_entry), buffer.c_str()); 
    }
  else gtk_entry_set_text(GTK_ENTRY(micro_vernum_entry), rh->p_micro_version.c_str() ); 
 
	gtk_grid_attach (GTK_GRID(grid), micro_vernum_entry, 40, 24, 15, 6);

 }                  
                          

// CURRENT VERSION
 
wid = gtk_label_new ("Current Version:");
decorate_widget(wid, &blue, NULL, NULL, NULL, GTK_STOCK_CANCEL);


gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 0, 31, 10, 6);


wid = gtk_label_new( rh->p_major_version.c_str());
decorate_widget(wid, &blue, NULL, NULL, NULL, GTK_STOCK_CANCEL);

gtk_misc_set_alignment (GTK_MISC (wid), 0.5, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 10, 31, 15, 6);
      
      
wid = gtk_label_new( rh->p_minor_version.c_str());
decorate_widget(wid, &blue, NULL, NULL, NULL, GTK_STOCK_CANCEL);
gtk_misc_set_alignment (GTK_MISC (wid), 0.5, .5);
gtk_grid_attach (GTK_GRID(grid), wid, 25, 31, 15, 6);
      
      
if (rh->flags.p_want_micro)
  {      
  wid = gtk_label_new( rh->p_micro_version.c_str());
  decorate_widget(wid, &blue, NULL, NULL, NULL, GTK_STOCK_CANCEL);

  gtk_misc_set_alignment (GTK_MISC (wid), 0.5, .5);
 
	gtk_grid_attach (GTK_GRID(grid), wid, 40, 31, 15, 6);
      
	}


// CONTACT
  
frame = gtk_frame_new(RevHead::ContactTXT.c_str());
	
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_widget_set_tooltip_text(GTK_WIDGET(frame),
      "Contact\n"
      "Contact information\n"
      "  Maybe email and snail addresses");
	
gtk_grid_attach (GTK_GRID(grid), frame, 0, 40, 64, 32);
	

wid = contact_el.create_entry_list();

gtk_container_set_border_width(GTK_CONTAINER(wid), 10);

gtk_container_add (GTK_CONTAINER(frame), wid);
	
if ( (existing_project_flag) && (!rh->p_address.empty()) ) // i.e. existing not empty
  {
  list<string>::iterator i;

  for (i=rh->p_address.begin(); i!=rh->p_address.end(); ++i)
    {
    contact_el.add_to_elist( (*i) , &contact_el);
    }
  }
else if (!rh->np_address.empty())         // i.e. defaults not empty
  {
  list<string>::iterator i;

  for (i=rh->np_address.begin(); i!=rh->np_address.end(); ++i)
    {
    contact_el.add_to_elist( (*i) , &contact_el);
    }
  }
else
    {
    for (int n=3; n; n--)                 // we want 3 entries anyway
        {
        contact_el.add_to_elist("", &contact_el);
        }
    }

}

// ==============
// COPYRIGHT PAGE
// ==============
void GetProject::fill_copyright_page(GtkWidget *vbox)
{
//GtkWidget *wid, *table, *table2, *frame, *box;
GtkWidget *widget, *grid, *frame, *box;;

gtk_widget_set_tooltip_text(GTK_WIDGET(vbox),
      "Copyright\n"
      "Enter copyright information here");

// Project Owner ===============================================================
	
grid = gtk_grid_new();

gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
	
gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, TRUE, 5);


// COPYRIGHT OWNER
widget = gtk_label_new ( (char *)RevHead::P_OwnerTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (widget), 1, .5);

gtk_grid_attach (GTK_GRID(grid), widget, 4, 0, 16, 5); // L, T, W, H


owner_widget = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(owner_widget), 40);

gtk_grid_attach (GTK_GRID(grid), owner_widget, 20, 0, 40, 5); // L, T, W, H


	
if (existing_project_flag && (!rh->p_owner.empty()))
    gtk_entry_set_text(GTK_ENTRY(owner_widget), (char *)rh->p_owner.c_str() );
else gtk_entry_set_text(GTK_ENTRY(owner_widget), (char *)rh->np_owner.c_str() );
  
 
cout << "GetProject::fill_copyright_page - 3" << endl;


gtk_widget_set_tooltip_text(GTK_WIDGET(owner_widget),
      "Copyright Owner\n"
      "Enter copyright owner here");

                            
// PROJECT COPYRIGHT ===========================================================

	
frame = gtk_frame_new(RevHead::P_CopyrightTXT.c_str());
gtk_grid_attach (GTK_GRID(grid), frame, 2, 5, 60, 60); // L, T, W, H
 

	
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
		
gtk_widget_set_tooltip_text(GTK_WIDGET(frame),
      "Copyright Notice\n"
      "Enter copyright information here");



widget = copyright_el.create_entry_list();
	
gtk_container_add(GTK_CONTAINER(frame), widget);
gtk_container_set_border_width(GTK_CONTAINER(widget), 10);


if ( (existing_project_flag) && (!rh->p_copyright.empty()) ) // i.e. existing not empty
  {
  list<string>::iterator i;
  for (i=rh->p_copyright.begin(); i!=rh->p_copyright.end(); ++i)
    {
    copyright_el.add_to_elist( (*i) , &copyright_el);
    }
  }
else if (!rh->np_copyright.empty())         // i.e. defaults not empty
  {
  list<string>::iterator i;
  for (i=rh->np_copyright.begin(); i!=rh->np_copyright.end(); ++i)
    {
    copyright_el.add_to_elist( (*i) , &copyright_el);
    }
  } 
else for (int n=3; n; n--)                  // we want 3 entries anyway
  {
  copyright_el.add_to_elist("", &copyright_el);
  }
}


// ==============
// COMMENTS PAGE
// ==============
void GetProject::fill_comment_page(GtkWidget *vbox)
{
GtkWidget *wid, *grid, *frame, *box;


box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);
	
grid = gtk_grid_new();
gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);

gtk_box_pack_start(GTK_BOX(box), grid, TRUE, TRUE, 5);



// COMMENTS

frame = gtk_frame_new(RevHead::CommentsTXT.c_str()); 
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_widget_set_tooltip_text(GTK_WIDGET(frame),
      "Comments\n"
      "Enter comments about this update");
gtk_grid_attach (GTK_GRID(grid), frame, 0, 0, 64, 36);	


wid = comment_el.create_entry_list();
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);
gtk_container_add(GTK_CONTAINER(frame), wid);
	
for (int n=3; n; n--) // give 3 empty lines as default
    {
    comment_el.add_to_elist("", &comment_el);
    }

// SUMMARY

frame = gtk_frame_new((existing_project_flag) ?
                        "EDIT Existing Summary:" : RevHead::P_SummaryTXT.c_str());

gtk_container_set_border_width(GTK_CONTAINER(frame), 10); 
gtk_widget_set_tooltip_text(GTK_WIDGET(frame),
      "Summary\n"
      "Edit the project summary if required");

gtk_grid_attach (GTK_GRID(grid), frame, 0, 36, 64, 36);	

wid = summary_el.create_entry_list();
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);

gtk_container_add(GTK_CONTAINER(frame), wid);

if (!rh->p_summary.empty())         // i.e. defaults not empty
  {
  list<string>::iterator i;
  for (i=rh->p_summary.begin(); i!=rh->p_summary.end(); ++i)
    {
    summary_el.add_to_elist( (*i) , &summary_el);
    }
  }
else for (int n=3; n; n--) // give 3 empty lines as default
    {
    summary_el.add_to_elist("", &summary_el);
    }
}


/********************************
 * void GetProject(flag)        *
 * flag=FALSE means new project *
 *      TRUE means existing one *
 * Get gadget input for project	*
 ********************************/
// I think this should make use of gtk_notebook
void GetProject::get_project(gboolean flag)
{
GtkWidget *grid;
GtkWidget *widget;
GtkWidget *box;

string buffer;


using namespace misc;

buffer = "PATH = " + rh->path;
rh->logfile->report(buffer);

existing_project_flag = flag;

rh->DestroyWidgets();
//tooltips = gtk_tooltips_new();

if (flag)		// EXISTING PROJECT
  {
  buffer = RevHead::UpdateProjectTXT;
  buffer +=  ": ";
  if ( rh->p_name.empty() ) buffer += "(UNNAMED PROJECT)";
  else buffer += rh->p_name;

  gtk_label_set_text (GTK_LABEL (rh->title), (char *)buffer.c_str() );

  }

grid = gtk_grid_new ();
gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
	
gtk_container_add (GTK_CONTAINER (rh->vbox), grid);
	
notebook = gtk_notebook_new ();  
gtk_notebook_set_tab_pos (GTK_NOTEBOOK(notebook), GTK_POS_TOP);

gtk_grid_attach (GTK_GRID(grid), notebook, 0, 0, 64, 72); // L, T, W, H


widget = add_page(notebook, (gchar *)"Name, Version & Contact");
fill_name_page(widget);


widget = add_page(notebook, (gchar *)"Copyright");
fill_copyright_page(widget);


widget = add_page(notebook, (gchar *)"Summary & Comments");
fill_comment_page(widget);


/*
  Termination buttons  
  NOTE: flag is TRUE for existing project 
  and FALSE if it already exists
*/

//======================
// hbox for YES / NO etc
//======================

box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
gtk_box_set_homogeneous (GTK_BOX(box), FALSE);
gtk_container_set_border_width(GTK_CONTAINER(box), 8);

gtk_grid_attach (GTK_GRID(grid), box, 0, 72, 64, 8); // L, T, W, H
	

	
// Project Environment
if (!rh->flags.rh_starting)
  {
  widget = CreateButton(box, "    PROJECT\nENVIRONMENT");
  decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_PREFERENCES);
	g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(callback_get_env), this);
  }

// ABORT EVERYTHING
widget = CreateButton(box, (char *)RevHead::QuitAllTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_QUIT);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(callback_abort_all), this);

// CANCEL
widget = CreateButton(box, (char *)RevHead::CancelTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_CANCEL);
g_signal_connect (G_OBJECT (widget), "clicked",
         G_CALLBACK(callback_cancel), this);

// OK, DO IT!
widget = CreateButton(box, (char *)RevHead::OKTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_OK);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(callback_OK), this);


gtk_widget_show_all(rh->vbox);
}


/* END OF FILE */
