/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       logfile.cpp                                               *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       06-Apr-2013 12.17.58 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Greatly improved but very similar to previous versions                    *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/


#include <string.h>

#include "logfile.h"

#include "date.h"
#include "tfile.h"
#include "complain.h"

#include "RH_version.h"

using namespace std;


fstream Logfile::logfs;
string Logfile::RH_logname   = "RH.log";

Logfile::Logfile(RevHead *revhead)
{
cout << "Logfile:\n";
rh = revhead;
}
        
Logfile::~Logfile(void)
{
cout << "~Logfile:\n";
}
         

/****************************************
 * Send report to window status bar,    *
 * also to log file if flag is FALSE.   *
 ****************************************/
void Logfile::report(string &s)
{
if (rh->status)
    {
    gtk_label_set_text(GTK_LABEL(rh->status), s.c_str() );
    }
else cerr << s << endl;

write2log( s );

return;
}



/****************************************
 * Open the log file if requested.      *
 * Put it in same directory.            *
 ****************************************/
 
// If first we open a temporary log file and use that
// Later in Getopts::ReadEnv this may become the actual file
// or be appended to the pre-existing log file.
// The tempoaray file is then eliminated.
// If no log is wanted, both the temorary and the original are removed.


void Logfile::open(gboolean first)
{
string buffer;
string *t;


// We assume that the original fstream has been deleted
// if we have changed from a previous directory.
// If first we open a temporary log file and use that
// later in Getopts::ReadEnv this may become the actual file
// or be appended to the pre-existing log file.
// The tempoaray file is then eliminated.
// If no log is wanted, both the temorary and the original are removed.


if (first)
    {
    TFile tf;
    Date dt(rh);
    unsigned int len;

//cout  <<  "Logfile::open - first,  rh->path = \"" << rh->path << "\"" << endl;
    
    logname = rh->path + "/";
    logname += RH_logname;

    t = tf.tfile(&logname, &logfs, 3);
    if (t) tname = *t;
    else
        {
        rh->flags.want_log = 0;
        rh->quit->HandleError(NO_OPEN_WARN, (char *)logname.c_str(), NULL);  // Can't open file
        }
    string s = RH_VERSION_STRING;
    buffer = "\n" + s;
    s.clear();
    for (len=buffer.length()-1; len>0; len--)
        {
        s += '-';
        }
    buffer = s + buffer + "\n" + s;
        
    write2log( buffer );
                      
    buffer = "OPENED startup temporary log";
    dt.date_log_operation(&buffer);
    buffer = "    Log name \"" + tname + "\"\n";
    write2log( buffer );
   }
else
  {
  UNLESS (logfs.is_open())
    {
    if (rh->flags.want_log)  // always opened in append mode after initial stuff
      {
      logfs.open( (char *)logname.c_str(), ios::out | ios::app | ios::binary );

      UNLESS (logfs.is_open())
        if (errno)
          cerr << "Logfile::open : Can't open \"" << logname << "\" : " << strerror(errno) << endl;
      }
    }
  }
return;
}

void Logfile::close(void)
{
if (logfs.is_open()) logfs.close();
}

// Write ane entry in log file
void Logfile::write2log(string &s)
{
string buffer;

buffer += s + "\n";
if (rh->flags.log_to_cerr)
  {
  cerr << buffer;
  }

// This shouldn't be necessary
UNLESS (logfs) open(FALSE);  // If we want a log file it gets opened if not already opened and we can use it

if ( (rh->flags.want_log) && logfs.is_open() )
  {
    
  UNLESS (logfs.write( (char *)buffer.c_str(), buffer.length() ) )
    {
    cerr << "Logfile::write2log - FAILED to write " << buffer << "\n   : errno=" << errno << " : " << strerror(errno) << endl;
    }
  logfs.flush();
  }
changed = TRUE;
return;
}

// Rename the temporary file as RH.log
void Logfile::rename_tfile(void)
{
string buffer;

UNLESS (tname.empty())
  {
  cout << "Logfile::rename_tfile" << endl;

  if (logfs.is_open()) logfs.close();

  if (rename( (char *)tname.c_str(), (char *)logname.c_str()) )
    {
    open(FALSE);
    rh->quit->HandleError(RENAME_F_ERR, (char *)tname.c_str(), (char *)logname.c_str());
    return;
    }

  open(FALSE);
  buffer =  "Rename of \"" + tname + "\"\n";
  buffer += "       as \"" + logname + "\" succeded";

  write2log(buffer);
  cout << "Logfile::rename_tfile - " << buffer << endl;

  tname.clear();
  }
}

// Append tfile to logfile
gboolean Logfile::append_tfile(void)
{
UNLESS (tname.empty())
  {
  TFile tf;
  fstream ofp;

  // Open logfile for append
  ofp.open( (char *)logname.c_str(), ios::out | ios::app | ios::binary );
  UNLESS (ofp.is_open())
    {
    cerr << "Logfile::append_tfile - Logfile::open : Can't open \"" << logname << "\" : " << strerror(errno) << endl;
    rh->flags.want_log = 0;  // Prevents lots of future errors
    return FALSE;
    }
    
  UNLESS (logfs.is_open())
    {
    logfs.open( (char *)tname.c_str(), ios::in | ios::binary);
    }
  if (logfs.is_open())
    {
    tf.do_copy(&ofp, &logfs);
    if (errno)
      {
      logfs.close();
      ofp.close();
      cerr << "Logfile::append_tfile - Logfile::open : Can't append onto \"" << logname << "\" : " << strerror(errno) << endl;
      rh->flags.want_log = 0;  // Prevents lots of future errors
      return FALSE;
      }
    logfs.close();
    }
  else
    {
    cerr << "Logfile::append_tfile - Can't open \"" << tname << "\" : " << strerror(errno) << endl;
    return FALSE;
    }
  }
else
  {
  // Can't append a non-existent file!
  cerr << "Logfile::append_tfile - NO TEMPORARY LOGFILE!" << endl;
  }
return TRUE;
}

void Logfile::delete_tfile(void)
{
UNLESS (tname.empty()) // using tfile, not logfile
  {
  if (logfs.is_open()) logfs.close();
  if (remove( (char *)tname.c_str() ))
    {
    cerr << "Logfile::delete_tfile - Failed to remove \"" << tname << "\" : " << strerror(errno) << endl;
    }
  tname.clear();
  }
}


void Logfile::delete_logfile(void)
{
if (tname.empty()) // using logfile, not tfile
  {
  if (logfs.is_open()) logfs.close();
  if (remove( (char *)logname.c_str() ))
    {
    cerr << "Logfile::delete_tfile - Failed to remove \"" << logname << "\" : " << strerror(errno) << endl;
    }
  logname.clear();
  }
}
