/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       revhead.cpp                                               *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Apr-2013 10.41.50 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* The main entry point and several supporting routines                      *
* 30-Sep-2001 16.29.29 -36000  0.00    David M. Balean                      *
* 30-Sep-2001 16.29.29 -36000  0.01    David M. Balean                      *
* Added RH_ to the start of the RH_version definitions                      *
* 30-Sep-2001 16.29.29 -36000  0.02    David M. Balean                      *
* Added terminal quotes for the contact address...                          *
* 30-Sep-2001 16.29.29 -36000  0.03    David M. Balean                      *
* Trying "Contact" again...                                                 *
* 30-Sep-2001 16.29.29 -36000  0.04    David M. Balean                      *
* Should have fixed the contact bug...                                      *
* 30-Sep-2001 16.29.29 -36000  0.05    David M. Balean                      *
* Minor changes                                                             *
* 30-Sep-2001 16.29.29 -36000  0.06    David M. Balean                      *
* Improved logging                                                          *
* 30-Sep-2001 16.29.29 -36000  0.07    David M. Balean                      *
* Fixed the double credits header entry in the log file                     *
* 30-Sep-2001 16.29.29 -36000  0.08    David M. Balean                      *
* Changed FrontPanel to pass correct flag to get environment                *
* 30-Sep-2001 16.29.29 -36000  0.09    David M. Balean                      *
* Fixed "flag" to Project Environment                                       *
* 30-Sep-2001 16.29.29 -36000  0.10    David M. Balean                      *
* ReadEnv didn't get the incrementing choices...                            *
* 30-Sep-2001 16.29.29 -36000  0.11    David M. Balean                      *
* Now opens the logfile as default, eliminates later if not required        *
* 15-Oct-2001 21.42.43 -36000 0.12    David M. Balean                       *
* Added TimeZn to headings and lined them up better in WriteDTVA()          *
* 19-Oct-2001 15.03.05 -36000 0.13    David M. Balean                       *
* Changes to logging functions                                              *
* 20-Aug-2004 12.10.29 -36000 0.18    David M. Balean                       *
* Several functions deleted or moved elsewhere                              *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/


#include <linux/limits.h>

#include "revhead.h"
#include "complain.h"
#include "date.h"
#include "tfile.h"
#include "misc.h"
#include "header.h"
#include "fixfiles.h"
#include "defaults.h"
#include "update.h"
#include "environment.h"
#include "logfile.h"
#include "about.h"
#include "RH_version.h"


using namespace misc;
using namespace std;

// static data
Quit *RevHead::quit = NULL;
GtkWidget *RevHead::window = NULL;
GtkWidget *RevHead::menu = NULL;
gulong RevHead::menu_id = 0;

// static GtkWidget *menu_bar;
// static GtkWidget *root_menu;

gint RevHead::width = 0;
gint RevHead::height = 0;

char    **RevHead::argv = NULL;
int     RevHead::argc = 0;
string RevHead::progname;
string RevHead::path;

string RevHead::RH_Start           = "RevHead_Start";
string RevHead::RH_End             = "RevHead_End";

string RevHead::PathTXT            = "PATH";
string RevHead::UpdateProjectTXT   = "UPDATE PROJECT";
string RevHead::NoUpdateTXT        = "  Don't UpDate  ";
string RevHead::NoLogTXT           = "   Don't log    ";

string RevHead::P_NameTXT          = "Project Name:";
string RevHead::P_OwnerTXT         = "Copyright Owner:";
string RevHead::P_EmailTXT         = "Project's Email:";

string RevHead::P_CopyrightTXT     = "Copyright Notice:";
string RevHead::ContactTXT         = "Contact:";
string RevHead::Start_DSTXT        = "Start DateStamp:";

string RevHead::P_DateTXT          = "Project Date:";
string RevHead::P_VersionTXT       = "Project Version:";
string RevHead::P_SummaryTXT       = "Project Summary:";
string RevHead::CommentsTXT        = "Comments:";

string RevHead::F_ThisFileTXT      = "This File:";
string RevHead::F_SummaryTXT       = "File Summary:";

string RevHead::F_VersionTXT       = "File Version:";
string RevHead::P_Start_VersionTXT = "PROJECT Start Version:";
string RevHead::AuthorTXT          = "Author:";
string RevHead::AuthorEmailTXT     = "Author's Email:";

    
    
string RevHead::P_EnvironmentTXT   = "PROJECT ENVIRONMENT";
string RevHead::DefaultsTXT        = "DEFAULTS";
string RevHead::NewProjectTXT      = "NEW PROJECT";
string RevHead::SourceDirsTXT      = "Source Directories:";
string RevHead::ChangeLogDirTXT    = "ChangeLog Directory:";
string RevHead::OKTXT              = "OK, DO IT!";
string RevHead::QuitAllTXT         = "    QUIT ALL    ";
string RevHead::CancelTXT          = "     CANCEL     ";


GdkPixbuf *RevHead::icon_pixbuf = NULL;

/* GdkPixbuf RGBA C-Source image dump 1-byte-run-length-encoded */

// Created with the command:-
//         gdk-pixbuf-csource --extern Jen.png > rh_image.cpp
// then copied here
// The image is of my Jensen Interceptor manipulated with the GIMP

#ifdef __SUNPRO_C
#pragma align 4 (my_pixbuf)
#endif
#ifdef __GNUC__
const guint8 RevHead::rh_pixbuf_source[] __attribute__ ((__aligned__ (4))) = 
#else
const guint8 RevHead::rh_pixbuf_source[] = 
#endif
{ ""
  /* Pixbuf magic (0x47646b50) */
  "GdkP"
  /* length: header (24) + pixel_data (10672) */
  "\0\0)\310"
  /* pixdata_type (0x2010002) */
  "\2\1\0\2"
  /* rowstride (400) */
  "\0\0\1\220"
  /* width (100) */
  "\0\0\0d"
  /* height (100) */
  "\0\0\0d"
  /* pixel_data: */
  "\250\0\0\0\0\24\377\0\0\30\377\0\0F\377\0\0n\377\0\0\222\377\0\0\261"
  "\377\0\0\312\377\0\0\337\377\0\0\356\377\0\0\370\377\0\0\377\377\0\0"
  "\375\377\0\0\370\377\0\0\356\377\0\0\337\377\0\0\312\377\0\0\261\377"
  "\0\0\222\377\0\0n\377\0\0F\377\0\0\30\314\0\0\0\0\4\377\0\0'\377\0\0"
  "i\377\0\0\247\377\0\0\337\224\377\0\0\377\4\377\0\0\337\377\0\0\247\377"
  "\0\0i\377\0\0'\305\0\0\0\0\3\377\0\0""6\377\0\0\210\377\0\0\324\234\377"
  "\0\0\377\3\377\0\0\324\377\0\0\210\377\0\0""6\277\0\0\0\0\3\377\0\0\23"
  "\377\0\0t\377\0\0\317\242\377\0\0\377\3\377\0\0\317\377\0\0t\377\0\0"
  "\23\272\0\0\0\0\3\377\0\0,\377\0\0\227\377\0\0\375\212\377\0\0\377\223"
  "\363\357\361\377\211\377\0\0\377\3\377\0\0\375\377\0\0\227\377\0\0,\266"
  "\0\0\0\0\2\377\0\0,\377\0\0\241\211\377\0\0\377\233\363\357\361\377\210"
  "\377\0\0\377\2\377\0\0\241\377\0\0,\262\0\0\0\0\2\377\0\0\23\377\0\0"
  "\222\210\377\0\0\377\241\363\357\361\377\207\377\0\0\377\2\377\0\0\222"
  "\377\0\0\23\257\0\0\0\0\2\377\0\0i\377\0\0\356\207\377\0\0\377\245\363"
  "\357\361\377\206\377\0\0\377\2\377\0\0\356\377\0\0i\254\0\0\0\0\2\377"
  "\0\0'\377\0\0\266\206\377\0\0\377\253\363\357\361\377\205\377\0\0\377"
  "\2\377\0\0\266\377\0\0'\251\0\0\0\0\2\377\0\0d\377\0\0\370\206\377\0"
  "\0\377\255\363\357\361\377\205\377\0\0\377\2\377\0\0\370\377\0\0d\247"
  "\0\0\0\0\1\377\0\0\227\206\377\0\0\377\261\363\357\361\377\205\377\0"
  "\0\377\1\377\0\0\227\244\0\0\0\0\2\377\0\0\35\377\0\0\300\205\377\0\0"
  "\377\265\363\357\361\377\204\377\0\0\377\2\377\0\0\300\377\0\0\35\241"
  "\0\0\0\0\2\377\0\0""6\377\0\0\337\205\377\0\0\377\267\363\357\361\377"
  "\204\377\0\0\377\2\377\0\0\337\377\0\0""6\237\0\0\0\0\2\377\0\0F\377"
  "\0\0\363\205\377\0\0\377\271\363\357\361\377\204\377\0\0\377\2\377\0"
  "\0\363\377\0\0F\235\0\0\0\0\2\377\0\0K\377\0\0\375\204\377\0\0\377\275"
  "\363\357\361\377\203\377\0\0\377\2\377\0\0\375\377\0\0K\233\0\0\0\0\2"
  "\377\0\0F\377\0\0\375\204\377\0\0\377\277\363\357\361\377\203\377\0\0"
  "\377\2\377\0\0\375\377\0\0F\231\0\0\0\0\2\377\0\0""6\377\0\0\363\204"
  "\377\0\0\377\301\363\357\361\377\203\377\0\0\377\2\377\0\0\363\377\0"
  "\0""6\227\0\0\0\0\2\377\0\0\35\377\0\0\337\204\377\0\0\377\303\363\357"
  "\361\377\203\377\0\0\377\2\377\0\0\337\377\0\0\35\226\0\0\0\0\1\377\0"
  "\0\300\204\377\0\0\377\305\363\357\361\377\203\377\0\0\377\1\377\0\0"
  "\300\225\0\0\0\0\1\377\0\0\227\204\377\0\0\377\307\363\357\361\377\203"
  "\377\0\0\377\1\377\0\0\227\223\0\0\0\0\1\377\0\0d\204\377\0\0\377\311"
  "\363\357\361\377\203\377\0\0\377\1\377\0\0d\221\0\0\0\0\2\377\0\0'\377"
  "\0\0\370\204\377\0\0\377\311\363\357\361\377\203\377\0\0\377\2\377\0"
  "\0\370\377\0\0'\220\0\0\0\0\1\377\0\0\266\204\377\0\0\377\313\363\357"
  "\361\377\203\377\0\0\377\1\377\0\0\266\217\0\0\0\0\1\377\0\0i\204\377"
  "\0\0\377\315\363\357\361\377\203\377\0\0\377\1\377\0\0i\215\0\0\0\0\2"
  "\377\0\0\23\377\0\0\356\203\377\0\0\377\317\363\357\361\377\202\377\0"
  "\0\377\2\377\0\0\356\377\0\0\23\214\0\0\0\0\1\377\0\0\222\204\377\0\0"
  "\377\317\363\357\361\377\203\377\0\0\377\1\377\0\0\222\213\0\0\0\0\1"
  "\377\0\0,\204\377\0\0\377\321\363\357\361\377\203\377\0\0\377\1\377\0"
  "\0,\212\0\0\0\0\1\377\0\0\241\204\377\0\0\377\321\363\357\361\377\203"
  "\377\0\0\377\1\377\0\0\241\211\0\0\0\0\1\377\0\0,\204\377\0\0\377\323"
  "\363\357\361\377\203\377\0\0\377\1\377\0\0,\210\0\0\0\0\1\377\0\0\227"
  "\203\377\0\0\377\325\363\357\361\377\202\377\0\0\377\1\377\0\0\227\207"
  "\0\0\0\0\2\377\0\0\23\377\0\0\375\203\377\0\0\377\325\363\357\361\377"
  "\202\377\0\0\377\2\377\0\0\375\377\0\0\23\206\0\0\0\0\1\377\0\0t\204"
  "\377\0\0\377\325\363\357\361\377\203\377\0\0\377\1\377\0\0t\206\0\0\0"
  "\0\1\377\0\0\317\203\377\0\0\377\327\363\357\361\377\202\377\0\0\377"
  "\1\377\0\0\317\205\0\0\0\0\1\377\0\0""6\204\377\0\0\377\327\363\357\361"
  "\377\203\377\0\0\377\1\377\0\0""6\204\0\0\0\0\1\377\0\0\210\203\377\0"
  "\0\377\331\363\357\361\377\202\377\0\0\377\1\377\0\0\210\204\0\0\0\0"
  "\1\377\0\0\324\203\377\0\0\377\256\363\357\361\377\12\257\235\246\15"
  "\254\246\257\27\267\272\306\30\276\305\323'\274\312\325'\274\311\327"
  "6\275\306\3276\272\303\323!\266\276\316\10\272\272\306\5\241\363\357"
  "\361\377\202\377\0\0\377\1\377\0\0\324\203\0\0\0\0\1\377\0\0'\204\377"
  "\0\0\377\251\363\357\361\377\33\270\270\302^\274\304\316\317\247\260"
  "\303\342\224\244\273\330\220\244\273\350\211\240\271\376\204\232\263"
  "\377\201\225\257\377~\224\257\377\177\224\256\377\177\223\256\377}\223"
  "\260\377}\223\261\377x\220\255\377x\217\256\375t\212\251\344i\202\241"
  "\330d}\234\330g\202\242\343n\211\251\350o\214\253\321t\220\256\304\202"
  "\240\276\265\214\246\306\252\234\246\273\252\233\216\226\241\253\211"
  "\213.\225\363\357\361\377\203\377\0\0\377\1\377\0\0'\202\0\0\0\0\1\377"
  "\0\0i\203\377\0\0\377\250\363\357\361\377\40\265\230\223\21\320\326\335"
  "\231\263\320\353\377\247\266\306\377{\204\217\377w\202\205\377Tfo\377"
  "Tcm\377Ubm\377Sak\377P_i\377N_h\377N^e\377O`e\377Rcg\377Pcj\377Qdj\377"
  "Xlq\377cw{\377q\201\206\377s\201\210\377\213\234\250\377w\203\210\377"
  "o{\205\377u\200\215\377v\204\232\377k{\226\377Zn\215\377[w\230\363a}"
  "\233\262{\222\252\217\277\324\3474\223\363\357\361\377\202\377\0\0\377"
  "\1\377\0\0i\202\0\0\0\0\1\377\0\0\247\203\377\0\0\377\247\363\357\361"
  "\377#\310\307\315<\273\323\352\343\254\307\345\377\262\273\306\377\212"
  "\222\242\377\202\217\244\377\267\304\322\377px\224\377mu\221\377`m\214"
  "\377gn\213\377op\215\377cl\210\377bj\206\377fl\207\377`h\203\377Sb\203"
  "\377A\\|\377F]~\377D]|\377Wl\211\377\304\324\344\377Sk\215\3779On\377"
  "B[u\377K_o\377LYa\377Wed\377hxq\377w\203\200\377W`i\377Yi|\377\225\244"
  "\270\376\262\304\350\245\242\261\320+\221\363\357\361\377\202\377\0\0"
  "\377\1\377\0\0\247\202\0\0\0\0\1\377\0\0\337\203\377\0\0\377\245\363"
  "\357\361\377(\255\217\212\3\305\330\345z\261\317\356\373\260\275\331"
  "\377\271\275\305\377\227\231\244\377j~\230\377\255\257\276\377\216\230"
  "\263\377rt\217\377pt\216\377`j\207\377jn\211\377rr\211\377jl\205\377"
  "hk\207\377qo\210\377jl\205\377H_\177\377F\\\177\377K^\200\377J_\177\377"
  "\244\256\301\377\314\323\340\377CSt\377;Op\377/Ee\3770De\3770Ee\377/"
  "Cc\377:Hh\377k\200\223\377T^q\3774>R\377=GY\377z\220\246\377\216\251"
  "\321\376e\204\247\272\214\243\277P\226qx\1\216\363\357\361\377\202\377"
  "\0\0\377\3\377\0\0\337\0\0\0\0\377\0\0\30\204\377\0\0\377\244\363\357"
  "\361\377\33\275\277\305(\275\330\353\313\241\300\343\377\262\271\322"
  "\377\363\357\361\377\276\266\273\377v\211\237\377en\216\377\276\310\324"
  "\377rv\220\377nq\212\377mq\212\377\\g\203\377ei\204\377pp\207\377gi\203"
  "\377cj\204\377\\i\206\377Ub\201\377;Xy\377CZ}\377H\\}\377Sg\207\377\336"
  "\342\350\377\237\246\267\377AVw\3773Ii\377\203/Dd\377\14/Cd\377<Mk\377"
  "\252\265\302\377AMe\3776CY\3775AV\377@M]\377\221\266\322\377_m\223\377"
  "F\\\177\377g\200\244\312q\222\2710\215\363\357\361\377\203\377\0\0\377"
  "\2\377\0\0\30\377\0\0F\203\377\0\0\377\243\363\357\361\377.\254\247\177"
  "\30\267\277\313\200\264\323\355\363\233\262\323\377\245\252\304\377\301"
  "\301\312\377\327\322\326\377\232\245\265\377]q\221\377\226\230\253\377"
  "\217\235\261\377rs\213\377no\206\377jn\206\377[d\201\377eh\203\377oo"
  "\206\377fi\203\377\\e\177\3779W{\377\77Yy\377=[\177\377\77Z\177\377G"
  "[|\377{\216\246\377\320\333\345\377Oc\203\3779Rs\3771Hj\3771Eg\377/F"
  "f\377/Hf\3775Ig\377\205\215\234\377\262\257\274\377=Jc\377>Na\377APh"
  "\3773>R\377Nbr\377\233\275\343\377\77Mo\377AQo\377K_}\375Yu\230\263q"
  "\212\245O\214\363\357\361\377\202\377\0\0\377\2\377\0\0F\377\0\0n\203"
  "\377\0\0\377\241\363\357\361\3773\225\247\227\4\231\230\236{\252\274"
  "\311\367\247\316\350\377\237\263\312\377\230\237\270\377\273\276\310"
  "\377\236\234\245\377\221\232\247\377au\222\377`q\215\377\302\304\314"
  "\377kr\214\377qr\213\377mm\206\377mk\204\377\\e\203\377gi\204\377qn\207"
  "\377ai\204\377Se\205\377<W{\377CZ{\377=Y\177\377\77Z\177\377I_~\377\255"
  "\271\306\377\264\267\306\377BUw\377:Rr\3774Ml\3771Jk\3774Ml\3778Om\377"
  "BUr\377\272\274\304\377\264\257\271\377BOc\377>Oc\377BRo\3773>T\3775"
  "AV\377q\221\246\377y\217\270\3777Ij\377>Ml\377AMi\377M^}\377[q\216\323"
  "f}\221mlq_\23\211\363\357\361\377\202\377\0\0\377\2\377\0\0n\377\0\0"
  "\222\203\377\0\0\377\234\363\357\361\377:\226sf\2\233zq\40\237\206\204"
  "4\235\222\231I\241\237\253e\237\243\253\311\247\311\343\377\222\275\346"
  "\377\220\257\313\377\213\246\306\377\267\276\314\377\224\226\236\377"
  "\217\234\254\377w\214\243\377\220\243\255\377\225\233\247\377\254\262"
  "\274\377s}\226\377\214\223\252\377\221\230\256\377\220\226\255\377\205"
  "\221\253\377\201\210\241\377}\201\232\377f~\240\377`x\231\377cw\230\377"
  "g\201\241\377i\206\247\377Xs\222\377gx\221\377\330\332\340\377}\210\240"
  "\377Qb\201\377M_}\377\77Us\3776Op\3776Oo\3775Nm\377M^y\377\335\337\343"
  "\377\214\206\224\377=K_\3775BX\3773@U\3773>U\3775CX\377;IZ\377\223\276"
  "\331\377O\\\203\3778Fc\377<Hd\377@Mg\377AQk\377E\\|\377Rk\214\347d\177"
  "\215\177iv[\17\207\363\357\361\377\202\377\0\0\377\2\377\0\0\222\377"
  "\0\0\261\203\377\0\0\377\215\363\357\361\377K\236\222m\6\231\212N8\230"
  "\202mf\236\222vh\233\220t]\232\233\211]\205\205}w\205ws\257~yt\316qk"
  "m\345rhn\350~ry\365vow\367pmx\350uw\206\331\212\211\230\351\220\223\243"
  "\377\216\223\240\377\221\232\255\377\236\266\320\377\237\305\342\377"
  "\223\267\324\377\234\272\325\377\254\306\337\377\302\317\336\377\217"
  "\222\236\377\222\240\260\377_z\235\377\202\231\245\377\210\235\236\377"
  "\272\300\307\377^w\224\377Yt\222\377h}\231\377i\200\233\377`w\226\377"
  "e|\230\377n\202\235\377ax\230\377^v\224\377[q\220\377cw\225\377h\200"
  "\236\377m\212\251\377r\216\257\377\260\271\305\377\302\304\320\377bt"
  "\220\377\\p\217\377^p\216\377ct\215\377J[z\377Pc\200\377K\\x\377\233"
  "\247\263\377\327\322\333\377JRe\377CLY\377OZh\377MYf\377P[h\377>HW\377"
  "2>Q\377Kaq\377\225\267\330\377a|\241\377Zo\217\377Vf\204\377N^{\377E"
  "Vu\377\77Sw\377Ic\206\377[p\211\367akg\244bhM\"\205\363\357\361\377\202"
  "\377\0\0\377\2\377\0\0\261\377\0\0\312\203\377\0\0\377\210\363\357\361"
  "\377QkdB\1o]B-\224\230ZU\230\243\177\200\236\226\213\254\235\216\213"
  "\360\254\245\226\377\274\274\262\377\262\264\256\377\264\270\253\377"
  "\244\263\250\377\221\246\252\377\210\242\266\377\213\251\306\377\205"
  "\244\305\377\204\243\306\377\215\246\310\377\215\246\300\377\215\264"
  "\333\377\216\267\340\377\213\266\341\377\210\265\337\377\207\264\337"
  "\377\206\262\337\377\200\254\334\377|\243\316\377l\214\263\377c~\236"
  "\377ct\214\377z\201\216\377\230\236\246\377\205\227\247\377\210\233\252"
  "\377\212\234\254\377\210\233\252\377\215\236\253\377\177\215\237\377"
  "\177\215\241\377\202\221\244\377\203\222\245\377\204\223\245\377\205"
  "\222\242\377\206\223\243\377\205\224\246\377\204\223\245\377\205\222"
  "\243\377\202\214\232\377}\211\230\377{\210\230\377}\217\240\377\217\231"
  "\246\377\242\251\261\377\232\243\252\377\236\250\260\377\245\255\272"
  "\377\251\260\274\377\253\260\274\377\252\261\274\377\247\257\270\377"
  "\214\223\240\377NYn\3779H]\377:FT\3771<I\377/;H\377.;G\3774@L\377>GU"
  "\377;DY\377@O`\377\\s\210\377f\200\234\377k\205\244\377n\212\252\377"
  "m\210\252\377n\210\251\377l\205\246\377j\202\244\377n\206\245\377r\205"
  "\236\367eozd\204\363\357\361\377\202\377\0\0\377\2\377\0\0\312\377\0"
  "\0\337\203\377\0\0\377\206\363\357\361\3775>8'\2wid[}vq\324\206\201\204"
  "\377\232\241\245\377\226\250\262\377\251\265\304\377\306\316\332\377"
  "\252\274\320\377\242\271\323\377\242\275\330\377\244\276\334\377\233"
  "\263\320\377\217\250\304\377\204\236\271\377{\223\261\377y\215\251\377"
  "w\213\247\377p\200\233\377hv\216\377gv\217\377dr\214\377_n\210\377\\"
  "k\205\377Vh\200\377Tg~\377Qc{\377O_v\377Xh|\377I[s\377FWo\377FUm\377"
  "DUl\377@Tj\377ATl\377BRk\377ASk\377\77Ri\377\77Qi\377>Qi\377>Ph\377="
  "Ph\377<Nf\377;Md\377:Ld\3779Jc\3778Jc\3778Jb\3778Ja\3778Ha\3778H`\377"
  "9H`\3778Ha\377\2026F_\377\24""6G]\3777G\\\3775G[\3776H]\3778F\\\3775"
  "GZ\3777E\\\3777D\\\3778E[\377>J^\377KTf\377[ds\377gn|\377]ds\377LUi\377"
  "BK^\3772<N\3770<L\377(0A\377&/=\377\202&/>\377\10%/\77\377&-=\377%.="
  "\377&1\77\377,7I\3779CU\377Q[\\\265c`O\77\202\363\357\361\377\202\377"
  "\0\0\377\2\377\0\0\337\377\0\0\356\203\377\0\0\377\204\363\357\361\377"
  "\37e[]\35nnu\200tz\214\326{\207\236\377\216\236\271\377\235\256\307\377"
  "\245\263\313\377\273\307\327\377\346\352\363\377\371\372\373\377\332"
  "\342\347\377\262\301\321\377\221\241\265\377\205\224\250\377s\202\233"
  "\377`q\211\377Yj\203\377Sf\200\377Qc}\377Nay\377M_w\377K]v\377I[u\377"
  "FYr\377EXp\377BWp\377BVo\377BUn\377CVn\377>Nc\377>Qj\377\202\77Ph\377"
  "\202<Nf\3776;Og\377=Og\377\77Ph\377@Qh\377@Qg\377CTh\377JYm\377KYn\377"
  "LZp\377JXm\377CTi\377@Qg\377\77Of\377ARf\377ETk\377APh\377>Nf\377>Mc"
  "\377;Lb\377:Lb\377=Kb\377GUj\377HVj\377FTh\377GTi\377KUh\377LXi\377P"
  "]n\377[du\377io~\377ks\200\377py\206\377jq\201\377bkz\377U_p\377HSf\377"
  "@Ma\377BM`\377@K]\3777DU\3776AT\3770<N\377-;L\377)7H\377*5F\377)5F\377"
  "*5F\377*5H\377,8M\3778CX\377>La\377PYc\374KMFf\363\357\361\377\202\377"
  "\0\0\377\2\377\0\0\356\377\0\0\370\203\377\0\0\377\202\363\357\361\377"
  "Vd]P)UPI\263BAF\373EN[\377OVd\377\203\210\235\377\221\235\261\377\212"
  "\227\255\377\217\234\263\377\236\254\277\377\257\267\311\377\274\303"
  "\323\377\250\265\304\377\207\226\251\377l|\220\377Sez\377JYr\377FWo\377"
  "DUl\377GVo\377IWl\377JXo\377JYp\377L[q\377M]q\377P`s\377Sav\377Vbv\377"
  "Zaw\377\\_u\377\\]t\377QSe\377]\\p\377_]q\377`\\o\377b]q\377c\\q\377"
  "e\\q\377eVj\377dVg\377n^m\377tdr\377wer\377vcq\377t`n\377wan\377wak\377"
  "p\\h\377lYd\377m[g\377lZh\377jXc\377q]i\377wcq\377wfr\377o_k\377n]i\377"
  "^Q[\377k[c\377jZa\377gX_\377bT]\377dT[\377cRY\377fSZ\377dQX\377bOU\377"
  "`MS\377dOU\377bKS\377_JQ\377[FM\377WAI\377U\77I\377VAJ\377V@I\377R=E"
  "\377T>D\377R:A\377N5<\377L5;\377J59\377H38\377F27\377B26\377<.4\377\202"
  "8,2\377\3<27\377I73\377;0#Y\202\377\0\0\377\2\377\0\0\370\377\0\0\375"
  "\203\377\0\0\3773\363\357\361\377/%\35\245@76\376NDG\377TCH\377`LS\377"
  "dOV\377iPW\377_LQ\377TAF\377]HO\377oYc\377\232\203\223\377\277\262\277"
  "\377\233\221\240\377\204v\210\377vew\377dSb\377S=H\377G*0\377G*/\377"
  "G.3\377B,.\377\77)-\377:$)\3779$'\3778&(\3772\40\"\377(\25\31\377$\20"
  "\24\377\"\17\20\377\235mX\377\275\201h\377\36\26\21\377\34\16\17\377"
  "\33\17\17\377\31\20\17\377\26\21\15\377\32\25\25\377\35\31\31\377\26"
  "\21\20\377\22\16\16\377\25\24\21\377\24\24\20\377\23\21\17\377\20\21"
  "\17\377\17\17\15\377\15\15\13\377\16\16\14\377\17\17\15\377\16\16\15"
  "\377\203\17\17\15\377\12OTU\377\224\234\243\377\222\227\233\377\210\216"
  "\222\377X^`\377\34\35\26\377\34!\26\377\34\"\27\377\33!\25\377\34\"\26"
  "\377\203\32\40\24\377\32#($\3774:\77\377,5<\377&.5\377&*2\377%(1\377"
  "$(0\377!%.\377\40(.\377\35!%\377\22\22\21\377\22\23\16\377\26\27\22\377"
  "\34\34\33\377\23\23\23\377\32\33\27\377\16\17\11\377\14\15\10\377\13"
  "\14\7\377\14\15\10\377\11\12\5\377\11\12\6\377\14\14\11\377\27\22\16"
  "\377\77)\37\3771\40\30c\202\377\0\0\377\202\377\0\0\375\203\377\0\0\377"
  ".\207Z8\7""0'\30\373,\33\32\377\36\16\17\377\26\14\14\377\21\12\12\377"
  "\21\16\15\377\17\17\14\377\14\13\14\377\25\27\30\377!&&\377\36\40\40"
  "\377\"#&\377224\377\30\32\31\377\16\17\21\377\16\21\23\377\26\32\35\377"
  "(-7\377*4>\377\22\24\25\377\16\17\21\377\16\16\15\377\16\17\16\377\15"
  "\14\14\377YUP\377~l[\377f\\R\377\33\37\25\377\35\37\24\377\35\40\30\377"
  "NI5\377YR<\377\37!\26\377\40!\26\377\37!\26\377!$\30\377\35\40\25\377"
  "\32\34\23\377\36\40\27\377\35\36\25\377\32\33\21\377\36!\24\377\37#\25"
  "\377\34!\24\377\34\"\25\377\202\35\"\26\377\17\40#\27\377\37$\30\377"
  "\36\"\26\377\40\"\27\377\37!\26\377\35\"\26\377\40%\32\377/7/\377-5-"
  "\377181\377%(\40\377\35\"\26\377!%\30\377\35\"\26\377\34\"\26\377\202"
  "\37\"\27\377\20\37\"\26\377\"('\377'-7\377\12\14\16\377\7\7\7\377\12"
  "\13\15\377\15\20\23\377\15\20\21\377\13\14\15\377\3\6\6\377\5\5\5\377"
  "\15\17\17\377\37'(\377\27\26\24\377\23\24\17\377\21\21\15\377\202\16"
  "\17\13\377\12\22\23\16\377\20\21\14\377\14\15\10\377\16\17\12\377\21"
  "\22\14\377\31\33\20\377%&\33\377>6/\377P\40\17\377\34\27\14\\\202\377"
  "\0\0\377\2\377\0\0\375\377\0\0\370\203\377\0\0\3777\0\0\0\0P>'\242\13"
  "\14\7\377\16\16\15\377\21\21\15\377\26\27\17\377\31\32\22\377\31\33\22"
  "\377\23\24\22\377\31\35\40\377\10\11\10\377\10\11\11\377\17\20\21\377"
  "\31\32\34\377\36\40%\377\33!$\377\26\34\35\377\16\21\22\377\6\6\6\377"
  "\10\11\11\377'*.\377\26\27\23\377\35\36\23\377\36\40\25\377\34\37\23"
  "\377895\377\24\27\24\377)'&\377&*!\377\33\35\22\377\35\37\24\377\36\37"
  "\25\377\34\40\25\377\32\35\22\377\33\37\24\377\32\36\23\377\35\40\27"
  "\377\36!\26\377\35\40\25\377\35\37\24\377\34\40\25\377\35!\25\377\34"
  "\40\24\377\33\36\22\377\32\36\21\377\31\37\23\377\31\36\22\377\32\36"
  "\22\377\33\36\23\377\32\37\24\377\32\37\23\377\33\36\23\377\34\37\24"
  "\377\32\37\23\377\31\37\23\377\202\33!\25\377\4\32\40\24\377\30\36\22"
  "\377\32\40\24\377\33\37\24\377\202\33\40\24\377\36\34\40\24\377\36\""
  "\27\377\36!\30\377\21\23\25\377\5\6\6\377\20\21\23\377\37$)\37715:\377"
  "8<A\377>CG\377EHL\377\77CG\377\34\35\36\377\10\10\10\377\6\7\6\377\26"
  "\26\23\377\23\24\16\377\24\26\21\377\22\25\17\377\30\32\26\377\23\24"
  "\16\377\20\21\14\377\20\21\13\377\26\30\20\377\36\37\24\377\40\"\25\377"
  "0/$\377D;3\377Q9'\377.1,\234\202\377\0\0\377\2\377\0\0\370\377\0\0\356"
  "\203\377\0\0\377!\0\0\0\0\207\177nu\35\"\34\377\23\23\15\377\32\33\22"
  "\377\34\36\23\377\33\35\22\377\31\32\21\377\17\17\15\377\6\6\6\377\12"
  "\12\12\37757:\377MPT\377@CH\3779=A\37737;\3776:>\37737=\377\35#'\377"
  "\16\17\17\377\5\6\7\377!$#\377\33\35\22\377\32\34\22\377\32\33\21\377"
  "!\"\33\377:=<\377(((\3779;5\377\35\37\24\377\32\34\21\377\33\35\22\377"
  "\32\34\21\377\202\24\32\16\377\202\27\35\21\377\1\30\36\22\377\202\32"
  "\35\22\377\10\26\34\20\377\27\34\20\377\30\36\22\377\31\36\23\377\32"
  "\34\21\377\27\34\20\377\30\34\21\377\34\36\23\377\202\31\37\23\377\10"
  "\30\35\22\377\31\36\22\377\30\36\22\377\27\35\21\377\32\40\24\377\30"
  "\36\22\377\31\37\23\377\30\36\22\377\202\31\37\23\377\202\32\40\24\377"
  "\27\31\37\23\377\30\36\22\377\31\37\23\377\24\27\22\377\7\7\7\377\26"
  "\32\36\37717=\377=BD\377000\377895\377:96\377)*'\377598\377UX[\37744"
  "6\377\5\6\4\377\16\17\12\377\26\30\22\377\31\35\25\377\33!\30\377\33"
  "!\27\377\34\37\23\377\36\40\25\377\202\40\"\27\377\12!#\32\377-,&\377"
  "+'$\377(%#\377BH:\377KR:\372\372\1\0\377\377\0\0\377\377\0\0\356\377"
  "\0\0\337\203\377\0\0\377\26qcg#\214\210\220\355;AC\377/04\377@A\77\377"
  "\30\33\25\377\26\27\20\377\31\32\21\377\7\11\6\377\11\10\11\377VTY\377"
  "SW\\\377-12\377((#\377CB7\377DE9\377,-*\37788:\377IMP\3778;\77\377\16"
  "\21\21\377\10\10\10\377\202\31\32\24\377\16\27\30\22\377\31\32\24\377"
  "373\377\"&(\3779=9\377\30\30\22\377\27\33\21\377\24\31\16\377\25\33\17"
  "\377\23\31\15\377\25\33\17\377\23\31\15\377\25\33\17\377\26\34\20\377"
  "\202\24\32\16\377\21\25\33\17\377\30\32\17\377\30\34\20\377\25\33\17"
  "\377\27\35\21\377\27\32\17\377\32\34\21\377\31\33\20\377\32\34\21\377"
  "\26\33\17\377\25\31\16\377\30\32\20\377\24\31\22\377\26\33\21\377\27"
  "\35\21\377\26\34\20\377\27\35\21\377\202\24\32\16\377\1\26\34\20\377"
  "\202\30\36\22\377#\27\35\21\377\25\33\17\377\32\34\22\377\12\13\10\377"
  "\20\25\24\377:\77C\377::>\377LOD\377\230\242\216\377\207\211q\377\177"
  "\204e\377\206\211k\377b_S\377334\377MNP\377###\377\21\22\15\377\27\34"
  "\25\377\32\40\25\377\33!\25\377\34\"\26\3778:1\377GLJ\377JQU\377OW^\377"
  "R[d\377Yai\377hnx\377z}\205\377\227\232\230\377\235\227\222\377\350\21"
  "\14\377\377\0\0\377\377\0\0\337\377\0\0\312\203\377\0\0\377*IGI.fed\371"
  "\211\214\204\377\262\260\244\377\276\276\263\377SWQ\377\40\36\27\377"
  "\31\32\23\377\5\6\6\377GAC\377RVX\377#%\"\377WWB\377\200\201`\377\203"
  "\212u\377hkN\377y~`\377ejW\377688\377UUY\377\77BD\377\11\13\12\377\16"
  "\17\13\377\26\30\21\377\21\25\16\377\23\27\20\377)-(\377#&&\377/31\377"
  "\27\35\25\377\22\30\17\377\22\27\17\377\22\30\17\377\21\27\16\377\24"
  "\27\17\377\25\30\20\377\24\31\20\377\23\31\20\377\22\30\17\377\22\27"
  "\17\377\21\26\16\377\21\25\15\377\202\21\26\16\377\21\22\30\17\377\24"
  "\30\17\377\25\32\20\377\23\27\14\377\25\31\21\377\25\32\21\377\22\27"
  "\17\377\24\31\21\377\24\31\22\377\24\31\21\377\23\31\20\377\22\27\17"
  "\377\25\27\17\377\25\31\21\377\25\33\21\377\24\32\16\377\23\30\20\377"
  "\202\23\30\17\377\"\22\27\17\377\20\20\16\377\12\14\14\37759<\377>\77"
  "B\377NPC\377uxb\377\232\247\243\377\224\227\230\377\37\40\32\377RWJ\377"
  "\207\213m\377gdW\377797\3778::\377\10\11\7\377\23\25\17\377\37\"\27\377"
  "\35!\26\377\36\40\27\37787-\377gfT\377kkY\377nmZ\377pk[\377wnc\377|r"
  "l\377\222\210~\272\272\265\237~\303\304\244A\375\3\2\377\377\0\0\377"
  "\377\0\0\312\377\0\0\261\203\377\0\0\377\"\363\357\361\377>D65<>*\272"
  "2/%\352@8.\377;4+\37790(\377%#\33\377\17\15\16\377ijj\377,/-\377WSA\377"
  "uxW\377GJ9\377\217\240\261\3777\77;\377+.#\377ilX\377y\200g\377BAA\377"
  "TWX\377'*)\377\5\5\6\377\20\22\15\377\20\24\15\377\22\26\17\377\30\33"
  "\25\377'*(\377\36!\40\377*+$\377\23\27\16\377\22\27\17\377\23\30\20\377"
  "\22\26\17\377\202\24\27\20\377\2\25\31\22\377\24\27\20\377\202\22\26"
  "\17\377\10\22\26\16\377\22\26\17\377\22\27\20\377\21\26\17\377\23\27"
  "\20\377\26\33\23\377\25\32\17\377\26\32\17\377\202\23\30\21\377\202\22"
  "\27\20\377\1\24\31\21\377\202\24\30\21\377#\23\27\21\377\25\30\22\377"
  "\23\27\21\377\26\31\22\377\24\31\17\377\24\26\20\377\22\25\17\377\21"
  "\24\16\377\20\23\15\377\11\11\10\377\33\35\36\377QTX\377/1.\377{\200"
  "b\377!\"\36\377emh\377\265\271\252\377\217\224\221\377\241\245\246\377"
  "wyk\377\215\221t\377BBA\37789:\377\15\15\15\377\20\20\20\377$#\33\377"
  ".+!\37740&\37771)\37760)\37750(\377>80\377G\77""6\377MC;\375[TH\202\203"
  "\363\357\361\377\202\377\0\0\377\2\377\0\0\261\377\0\0\222\203\377\0"
  "\0\377\204\363\357\361\377\40IJ<kWN@\370MA3\3771,#\377,((\377UXX\377"
  "%$#\377\212\211j\377DG9\377//\"\377\227\236\216\377\206\214\204\377\213"
  "\214\212\377\232\241\250\377\204\206p\377WZN\377OOQ\377*.-\377\11\12"
  "\12\377\11\12\7\377\22\23\15\377\23\24\16\377\24\25\17\377250\377:><"
  "\377DEC\377\25\26\20\377\26\25\20\377\31\31\22\377\26\27\20\377\22\26"
  "\17\377\24\27\21\377\202\26\27\21\3773\27\27\22\377\27\30\22\377\34\34"
  "\27\377\35\36\27\377\37\40\31\377\37\36\30\377\36\37\30\377\37\37\26"
  "\377\37\40\27\377\36\37\26\377\33\32\24\377\35\36\24\377%%\34\377\"\""
  "\32\377''\35\377-*$\377+)#\377**\"\377##\33\377'%\36\377*(\"\377\37\40"
  "\32\377)&\40\37741+\37720)\377\30\31\26\377\5\5\5\37779:\377KOQ\377I"
  "P@\377|\204m\377gmd\377\226\233\221\377\265\266\257\377\232\237\225\377"
  "\203\205|\377AD;\377\231\236\204\377\\\\V\377889\377\31\32\30\377\20"
  "\21\16\37720(\37762)\37784+\37751(\37794+\37783+\37750,\377;5,\3421)"
  "%9\204\363\357\361\377\202\377\0\0\377\2\377\0\0\222\377\0\0n\203\377"
  "\0\0\377\205\363\357\361\377S-2'9^XN\261\77>8\374@>@\377LNM\377><4\377"
  "\204\211d\377\200\206o\377\221\226\213\377\236\236\224\377\224\231\224"
  "\377\214\227{\377_cX\377\\ZR\377szb\377GII\377.21\377\20\20\20\377\5"
  "\4\4\377@A<\377XYT\377[[X\377^_Y\377nli\377rrn\377wuo\377yvr\377yvs\377"
  "zzv\377zzw\377\177\177|\377\201\201}\377\205\205\202\377\206\206\202"
  "\377\214\212\205\377\226\223\214\377\232\225\216\377\231\226\220\377"
  "\234\233\225\377\237\237\230\377\242\241\231\377\246\244\236\377\244"
  "\242\236\377\246\243\240\377\252\247\237\377\257\253\245\377\257\252"
  "\245\377\260\255\246\377\257\254\247\377\255\252\243\377\256\253\244"
  "\377\261\251\245\377\257\250\243\377\250\246\243\377\251\246\237\377"
  "\263\255\245\377\267\260\252\377\270\260\246\377:76\377\12\13\7\377H"
  "HF\377HHI\377W^M\377\221\233f\377\233\245}\377\231\241\210\377\201\201"
  "x\377\201\204p\377z{m\377Z_W\377\252\257\231\377cb^\377665\377(\"\32"
  "\377+$\32\354972\340>;8\322941\245//,\207((%|65.XvjU,\267\256\224\6\205"
  "\363\357\361\377\202\377\0\0\377\2\377\0\0n\377\0\0F\203\377\0\0\377"
  "\207\363\357\361\377IHGMB^[[\323JLJ\377DB=\377~\201b\377y}b\377\214\215"
  "u\377~\204r\377ope\377\200\205s\377SYQ\377_^X\377~\205j\377EHG\377-."
  ".\377\24\24\24\377\3\2\3\377M\77:\377\325\274\245\377\314\265\242\377"
  "\315\270\243\377\326\275\243\377\331\302\250\377\337\306\255\377\334"
  "\302\252\377\324\272\242\377\314\263\234\377\325\276\244\377\330\277"
  "\245\377\333\277\244\377\332\301\242\377\332\277\242\377\334\300\245"
  "\377\335\301\245\377\336\304\254\377\337\306\255\377\336\305\253\377"
  "\341\310\252\377\337\306\253\377\332\302\250\377\325\275\243\377\332"
  "\301\250\377\331\301\247\377\332\301\250\377\322\270\242\377\315\270"
  "\243\377\313\266\243\377\304\256\234\377\303\255\233\377\273\250\225"
  "\377\261\235\212\377\251\227\205\377\250\225\202\377\260\235\210\377"
  "\257\233\211\377\241\214{\377\30\27\20\377\20\21\14\377GGE\377HHH\377"
  "PRJ\377\202\205l\377,.\37\377nqg\377\234\244|\377\252\243\224\377\256"
  "\253\212\377\302\277\270\377\274\277\271\377]Z^\377/..\377&'%8.14\1\215"
  "\363\357\361\377\202\377\0\0\377\2\377\0\0F\377\0\0\30\203\377\0\0\377"
  "\210\363\357\361\377F\263\264\241MHIF\377=;=\377\210\215~\377W]W\377"
  "63/\377\227\225x\377\215\212z\377\247\251\214\377\226\226{\377\261\261"
  "\244\377ovh\377HKJ\377**(\377\21\20\20\377\5\4\3\374\14\10\6\373]PF\373"
  "m]N\337lZL\333fVI\375eXH\377eVH\377_QC\377YL\77\375LE:\367MF:\341KC7"
  "\337C:0\372<6-\37793+\37271*\3512+%\370,(!\377($\36\377&\"\35\373#!\32"
  "\361$!\34\326%!\33\300%!\33\252#\40\30\243\36\33\25\311\36\32\25\263"
  "\35\32\26\252\31\27\22\300\27\25\22\312\23\22\17\330\20\17\14\330\16"
  "\17\14\322\15\16\12\305\14\14\10\261\16\16\12\252\15\15\12\236\16\16"
  "\11\240\16\15\12\267\13\12\11\251\6\7\4\376563\377III\377CCD\377\206"
  "\213v\377yzu\377\255\251\236\377\244\223\207\377(&$\377|\177x\377\324"
  "\324\325\377\232\231\234\377POS\377(()\242\217\363\357\361\377\202\377"
  "\0\0\377\3\377\0\0\30\0\0\0\0\377\0\0\337\203\377\0\0\377\207\363\357"
  "\361\377\17\364\352\330\4QRJ\357FEH\377tu{\377\216\225\224\377\212\210"
  "\203\377\325\304\254\377MOB\377;9,\377\222\213\201\377\260\265\251\377"
  "\\a^\377475\377&'\"\377\13\13\12\262\251\363\357\361\377\16\4\4\4\250"
  "\17\17\15\3779:5\377BAB\377UXX\377\246\253\243\377\316\316\311\377\245"
  "\244\243\377\270\272\270\377\343\343\340\377\313\301\310\377xv~\3775"
  "67\346'''\11\216\363\357\361\377\202\377\0\0\377\1\377\0\0\337\202\0"
  "\0\0\0\1\377\0\0\247\203\377\0\0\377\210\363\357\361\377\16HA<\211<;"
  ";\377OPU\377\216\217\230\377\310\313\316\377\321\323\323\377\245\247"
  "\237\377\266\266\260\377\266\271\263\377}\201\202\377MQR\377++'\377\33"
  "\34\27\351\11\11\10(\251\363\357\361\377\15\7\7\7\15\11\12\7\252\32\33"
  "\26\377663\377CCC\377[Z_\377\203\207\213\377\235\241\245\377\244\244"
  "\251\377\244\241\253\377}{\203\377\77:\77\375-+)H\217\363\357\361\377"
  "\202\377\0\0\377\1\377\0\0\247\202\0\0\0\0\1\377\0\0i\203\377\0\0\377"
  "\210\363\357\361\377\15\34\35\32\15""0--\276>><\377^]a\377\211\210\220"
  "\377\255\255\261\377\331\325\326\377\315\315\314\377\223\223\227\377"
  "VY[\37711,\377)(!\270\14\15\10\27\254\363\357\361\377\12\20\21\17I\35"
  "\36\32\252,-&\324997\374KLL\377WW\\\377VU[\377HFF\377321\317.*(T\220"
  "\363\357\361\377\202\377\0\0\377\1\377\0\0i\202\0\0\0\0\1\377\0\0'\203"
  "\377\0\0\377\211\363\357\361\377\13'%$\3""95/y752\355LKL\377nlo\377{"
  "y~\377dae\377@>=\377/-*\364*)$q\22\23\20\2\260\363\357\361\377\4(&\32"
  "\16+(\34""5-'\36--%\33\26\223\363\357\361\377\202\377\0\0\377\1\377\0"
  "\0'\203\0\0\0\0\1\377\0\0\324\203\377\0\0\377\212\363\357\361\377\7&"
  "\"\34\10/,!3\77:,62.%:41\"UEA2imk_*\310\363\357\361\377\202\377\0\0\377"
  "\1\377\0\0\324\204\0\0\0\0\1\377\0\0\210\203\377\0\0\377\331\363\357"
  "\361\377\202\377\0\0\377\1\377\0\0\210\204\0\0\0\0\1\377\0\0""6\203\377"
  "\0\0\377\331\363\357\361\377\202\377\0\0\377\1\377\0\0""6\205\0\0\0\0"
  "\1\377\0\0\317\203\377\0\0\377\327\363\357\361\377\202\377\0\0\377\1"
  "\377\0\0\317\206\0\0\0\0\1\377\0\0t\203\377\0\0\377\327\363\357\361\377"
  "\202\377\0\0\377\1\377\0\0t\206\0\0\0\0\2\377\0\0\23\377\0\0\375\203"
  "\377\0\0\377\325\363\357\361\377\202\377\0\0\377\2\377\0\0\375\377\0"
  "\0\23\207\0\0\0\0\1\377\0\0\227\203\377\0\0\377\325\363\357\361\377\202"
  "\377\0\0\377\1\377\0\0\227\210\0\0\0\0\1\377\0\0,\203\377\0\0\377\325"
  "\363\357\361\377\202\377\0\0\377\1\377\0\0,\211\0\0\0\0\1\377\0\0\241"
  "\203\377\0\0\377\323\363\357\361\377\202\377\0\0\377\1\377\0\0\241\212"
  "\0\0\0\0\1\377\0\0,\204\377\0\0\377\321\363\357\361\377\203\377\0\0\377"
  "\1\377\0\0,\213\0\0\0\0\1\377\0\0\222\203\377\0\0\377\321\363\357\361"
  "\377\202\377\0\0\377\1\377\0\0\222\214\0\0\0\0\2\377\0\0\23\377\0\0\356"
  "\203\377\0\0\377\317\363\357\361\377\202\377\0\0\377\2\377\0\0\356\377"
  "\0\0\23\215\0\0\0\0\1\377\0\0i\203\377\0\0\377\317\363\357\361\377\202"
  "\377\0\0\377\1\377\0\0i\217\0\0\0\0\1\377\0\0\266\203\377\0\0\377\315"
  "\363\357\361\377\202\377\0\0\377\1\377\0\0\266\220\0\0\0\0\2\377\0\0"
  "'\377\0\0\370\203\377\0\0\377\313\363\357\361\377\202\377\0\0\377\2\377"
  "\0\0\370\377\0\0'\221\0\0\0\0\1\377\0\0d\204\377\0\0\377\311\363\357"
  "\361\377\203\377\0\0\377\1\377\0\0d\223\0\0\0\0\1\377\0\0\227\203\377"
  "\0\0\377\311\363\357\361\377\202\377\0\0\377\1\377\0\0\227\225\0\0\0"
  "\0\1\377\0\0\300\203\377\0\0\377\307\363\357\361\377\202\377\0\0\377"
  "\1\377\0\0\300\226\0\0\0\0\2\377\0\0\35\377\0\0\337\203\377\0\0\377\305"
  "\363\357\361\377\202\377\0\0\377\2\377\0\0\337\377\0\0\35\227\0\0\0\0"
  "\2\377\0\0""6\377\0\0\363\203\377\0\0\377\303\363\357\361\377\202\377"
  "\0\0\377\2\377\0\0\363\377\0\0""6\231\0\0\0\0\2\377\0\0F\377\0\0\375"
  "\203\377\0\0\377\301\363\357\361\377\202\377\0\0\377\2\377\0\0\375\377"
  "\0\0F\233\0\0\0\0\2\377\0\0K\377\0\0\375\203\377\0\0\377\277\363\357"
  "\361\377\202\377\0\0\377\2\377\0\0\375\377\0\0K\235\0\0\0\0\2\377\0\0"
  "F\377\0\0\363\203\377\0\0\377\275\363\357\361\377\202\377\0\0\377\2\377"
  "\0\0\363\377\0\0F\237\0\0\0\0\2\377\0\0""6\377\0\0\337\204\377\0\0\377"
  "\271\363\357\361\377\203\377\0\0\377\2\377\0\0\337\377\0\0""6\241\0\0"
  "\0\0\2\377\0\0\35\377\0\0\300\204\377\0\0\377\267\363\357\361\377\203"
  "\377\0\0\377\2\377\0\0\300\377\0\0\35\244\0\0\0\0\1\377\0\0\227\204\377"
  "\0\0\377\265\363\357\361\377\203\377\0\0\377\1\377\0\0\227\247\0\0\0"
  "\0\2\377\0\0d\377\0\0\370\204\377\0\0\377\261\363\357\361\377\203\377"
  "\0\0\377\2\377\0\0\370\377\0\0d\251\0\0\0\0\2\377\0\0'\377\0\0\266\205"
  "\377\0\0\377\255\363\357\361\377\204\377\0\0\377\2\377\0\0\266\377\0"
  "\0'\254\0\0\0\0\2\377\0\0i\377\0\0\356\204\377\0\0\377\253\363\357\361"
  "\377\203\377\0\0\377\2\377\0\0\356\377\0\0i\257\0\0\0\0\2\377\0\0\23"
  "\377\0\0\222\206\377\0\0\377\245\363\357\361\377\205\377\0\0\377\2\377"
  "\0\0\222\377\0\0\23\262\0\0\0\0\2\377\0\0,\377\0\0\241\206\377\0\0\377"
  "\241\363\357\361\377\205\377\0\0\377\2\377\0\0\241\377\0\0,\266\0\0\0"
  "\0\3\377\0\0,\377\0\0\227\377\0\0\375\206\377\0\0\377\233\363\357\361"
  "\377\205\377\0\0\377\3\377\0\0\375\377\0\0\227\377\0\0,\272\0\0\0\0\3"
  "\377\0\0\23\377\0\0t\377\0\0\317\210\377\0\0\377\223\363\357\361\377"
  "\207\377\0\0\377\3\377\0\0\317\377\0\0t\377\0\0\23\277\0\0\0\0\3\377"
  "\0\0""6\377\0\0\210\377\0\0\324\234\377\0\0\377\3\377\0\0\324\377\0\0"
  "\210\377\0\0""6\305\0\0\0\0\4\377\0\0'\377\0\0i\377\0\0\247\377\0\0\337"
  "\224\377\0\0\377\4\377\0\0\337\377\0\0\247\377\0\0i\377\0\0'\314\0\0"
  "\0\0\24\377\0\0\30\377\0\0F\377\0\0n\377\0\0\222\377\0\0\261\377\0\0"
  "\312\377\0\0\337\377\0\0\356\377\0\0\370\377\0\0\377\377\0\0\375\377"
  "\0\0\370\377\0\0\356\377\0\0\337\377\0\0\312\377\0\0\261\377\0\0\222"
  "\377\0\0n\377\0\0F\377\0\0\30\250\0\0\0\0"};

    
    
    
    
    




/*************************************************
 * int get_file_type(char *fname, gboolean flag) *
 * Return the file type of fname.                *
 * flag FALSE means return type whatever         *
 * flag TRUE means NO_TYPE if not wanted         *
 *************************************************/
int RevHead::get_file_type(string filename, gboolean flag)
{
int type = NO_TYPE;  // default

char *fname = (char *)filename.c_str();

/*******************************************
NOTE:   Version files always return NO_TYPE.
   Not sure if I want this.
********************************************/

// C
if ( SUFEQ(fname,".c") )
  {
  type = C_TYPE;
  }

// .h files may be C_TYPE or CPP_TYPE
else if ( SUFIEQ(fname, ".h") && (!STREQ(fname, (char *)Update::hname.c_str() ) ) )
  {
  if (!flag) type = CPP_TYPE;
  else
    {
    if (flags.c_file) type = C_TYPE;
    if (flags.cpp_file) type = CPP_TYPE;
    }
  }

// C++
else if ( SUFIEQ(fname,".cc") || 
        SUFIEQ(fname,".cpp") ||
        SUFIEQ(fname,".cxx") ||
        SUFIEQ(fname,".HPP") ||
        SUFIEQ(fname,".hpp") ||
        SUFEQ(fname, ".C") )
  {
  if (!flag) type = CPP_TYPE;
  else if (flags.cpp_file) type = CPP_TYPE;  
  }

// PHP
else if ( ( SUFIEQ(fname, ".php") && !STREQ(fname, (char *)Update::phpname.c_str() ) )  )
  {
  if (!flag) type = PHP_TYPE;
  else if (flags.php_file) type = PHP_TYPE;
  }

// ADA
else if ( ( SUFEQ(fname, ".adb") ||  SUFEQ(fname, ".ads") ) && 
          !STREQ(fname, (char *)Update::adaname.c_str() ) )
  {
  if (!flag) type = ADA_TYPE;
  else if (flags.ada_file) type = ADA_TYPE;
  }
// JAVA
else if ( ( SUFIEQ(fname, ".java") && !STREQ(fname, (char *)Update::javaname.c_str() ) )  )
  {
  if (!flag) type = JAVA_TYPE;
  else if (flags.java_file) type = JAVA_TYPE;
  }
// PERL
else if ( ( SUFIEQ(fname, ".perl") || SUFIEQ(fname, ".pl") ) && 
          !STREQ(fname, (char *)Update::perlname.c_str() ) )
  {
  if (!flag) type = PERL_TYPE;
  else if (flags.perl_file) type = PERL_TYPE;
  }
// FORTRAN
else if ( ( SUFIEQ(fname, ".f90") || SUFIEQ(fname, ".f95") ) && 
          !STREQ(fname, (char *)Update::fortranname.c_str() ) )
  {
  if (!flag) type = FORTRAN_TYPE;
  else if (flags.fortran_file) type = FORTRAN_TYPE;
  }

return(type);
}


/************************************************
 * gboolean is_logged(string &fname)            *
 * Check if file fname has a RevHead header.    *
 * Only checks for RevHead_Start...             *
 * Return TRUE (OK) or FALSE (no header).       *
 ************************************************/
gboolean RevHead::is_logged(string &fname, int &comment_type)
{
fstream ifp;
string  buffer;
string  test_str;
int    type;
gboolean	rc = FALSE;


type = get_file_type(fname, FALSE);  // get type even if not "wanted"

if (type != NO_TYPE)
  {
  test_str = Header::in_comment_key_start[type] + RH_Start;

  ifp.open(fname.c_str(), ios::in | ios::binary);


  if (ifp.is_open())
	  {

	  if (find_string(ifp, test_str, buffer) == 1)
      {
      // type is logged, assume comment type is correct
      // C ADA and PERL have only one type available so they return here if OK
      rc = TRUE;
      }

  // CPP
    else if (type == CPP_TYPE)  // default was CPP_TYPE in Header
      {
      ifp.clear();
      ifp.seekg(0);
      type = C_TYPE;
      test_str = Header::in_comment_key_start[type] + RH_Start;
	    if (find_string(ifp, test_str, buffer) == 1)
        {
		    rc = TRUE;  // CPP_TYPE is logged, using C_TYPE comments
        }
      else
        {
        ifp.clear();
        ifp.seekg(0);
        type = CPP2_TYPE;
        test_str = Header::in_comment_key_start[type] + RH_Start;
        
	      if (find_string(ifp, test_str, buffer) == 1)
          {
		      rc = TRUE;  // CPP_TYPE is logged, using C_TYPE comments
          }
        else type = CPP_TYPE;
        }
      }

  // PHP
    else if (type == PHP_TYPE)  // default was C_TYPE in Header
      {
      ifp.clear();
      ifp.seekg(0);
      type = CPP_TYPE;
      test_str = Header::in_comment_key_start[type] + RH_Start;
	    if (find_string(ifp, test_str, buffer) == 1)
        {
		    rc = TRUE;  // CPP_TYPE is logged, using C_TYPE comments
        }
      else
        {
        ifp.clear();
        ifp.seekg(0);
        type = HASH_TYPE;
        test_str = Header::in_comment_key_start[type] + RH_Start;
	      if (find_string(ifp, test_str, buffer) == 1)
          {
		      rc = TRUE;  // using HASH_TYPE comments
          }
        else
          {
          ifp.clear();
          ifp.seekg(0);
          type = CPP2_TYPE;
          test_str = Header::in_comment_key_start[type] + RH_Start;
	        if (find_string(ifp, test_str, buffer) == 1)
            {
		        rc = TRUE;  // using CPP2_TYPE comments
            }
          else type = PHP_TYPE;
          }
        }
      }

  // JAVA
    else if (type == JAVA_TYPE)  // default was C_TYPE in Header
      {
      ifp.clear();
      ifp.seekg(0);
      type = CPP_TYPE;
      test_str = Header::in_comment_key_start[type] + RH_Start;
	    if (find_string(ifp, test_str, buffer) == 1)
        {
		    rc = TRUE;  // CPP_TYPE is logged, using C_TYPE comments
        }
      else
        {
        ifp.clear();
        ifp.seekg(0);
        type = CPP2_TYPE;
        test_str = Header::in_comment_key_start[type] + RH_Start;
	      if (find_string(ifp, test_str, buffer) == 1)
          {
		      rc = TRUE;  // using CPP2_TYPE comments
          }

        else type = JAVA_TYPE;
        }
      }
    ifp.close();
	  }
  }

comment_type = type;  // If RevHead::RH_Start not found this is the default type for that file type
return(rc);
}


/****************************************
 * gboolean write_string(fp,buffer)     *
 * Write a line from buffer to file.    *
 * Assumes line ended with '\n'.        *
 * Return TRUE (OK) or FALSE.           *
 ****************************************/
gboolean RevHead::write_string(fstream &ofp, string &buffer)
{
if (ofp)
    {
    // We need a check that it was actually written
    ofp.write( buffer.c_str(), buffer.length() );
    return(TRUE);
    }
return(FALSE);
}

/****************************************
 * gboolean read_line(fp,buffer)        *
 * Get a line into buffer.              *
 * Return TRUE (OK) or FALSE.           *
 ****************************************/
gboolean RevHead::read_line(fstream &ifp, string &buffer)
{
char c;

buffer.clear();

if (ifp)
  {
  while (ifp.get(c))
    {
    buffer += c;

    if (c == '\n')
      {
      return(TRUE);
      }
    }
  if (!ifp.eof()) cout << "RevHead::read_line - FAILED TO get(c) : " << strerror(errno) << endl;
  }
return(FALSE);
}


/****************************************
 * int find_string(ifp,string,buffer)   *
 * Find start of our header.            *
 * ifp      = file pointer              *
 * string   = string to find            *
 * buffer   = buffer                    *
 * Assumes string at start of line.     *
 * Return 1 if found.                   *
 ****************************************/
int RevHead::find_string(fstream &ifp, string &test_str, string &buffer)
{
while (read_line(ifp, buffer))
    {
    if (buffer.find(test_str)==0)    return 1;  // FOUND
    if (buffer.find(RevHead::RH_End) != string::npos) return -1; // END of header
    }
return 0;                                       // FAILED
}

// Get major minor and mico from buffer
void RevHead::get_version(string &buffer, string &major, string &minor, string &micro)
{
unsigned int n;
major = minor = micro = "";
for (n=0; n<buffer.length(); ++n)
  {
  if (buffer[n] == '.') break;
  major += buffer[n];
  }
remove_leading(major);
n++;
if ( n<buffer.length() )
  {
  for ( ; n<buffer.length(); ++n)
    {
    if (buffer[n] == '.') break;
    minor += buffer[n];
    }                  
  }
remove_leading(minor);
n++;
if ( n<buffer.length() )
  {
  micro += buffer.substr(n);
  remove_leading(micro);
  }  
if (major.empty()) major = '?';
if (minor.empty()) minor = '?';
if (micro.empty()) micro = '?';
}

// Append major minor micro to buffer
void RevHead::append_version(string &buffer, string major, string minor, string micro)
{
if (major.length()) buffer += major;
else buffer += '?';
buffer += '.';

if (minor.length()) buffer += minor;
else buffer += '?';

if (flags.p_want_micro)
  {
  buffer += '.';
  if (micro.length()) buffer += micro;
  else buffer += '?';
  }
}


// This should destroy ALL the widgets
void RevHead::DoDestroyWidget(GtkWidget *widget, RevHead *rh)
{
if (GTK_IS_WIDGET (widget))
    {
    UNLESS (widget==rh->status || widget==rh->title)
    gtk_widget_destroy(widget);
    }   
}

// Destroy all the widgets except status and first vbox
// We should be able to simply destroy one parent widget instead of this...
void RevHead::DestroyWidgets(void)
{
if (vbox)
  {
  gtk_container_foreach (GTK_CONTAINER (vbox), 
                (GtkCallback) DoDestroyWidget, this);
  }
}


/*
 * CloseAppWindow
 *
 * The window is closing down, end the gtk loop
 */
void RevHead::CloseAppWindow (GtkWidget *widget, gpointer data)
{
quit->cleanexit(NO_ERR);	/* Quit, bypass HandleError */
}


void RevHead::QuitFunc(GtkWidget *widget, Complain *cn)
{
cn->end_complain(widget, cn);
((RevHead *)(cn->data))->quit->cleanexit(NO_ERR);	/* Quit, bypass HandleError */
}

//--------------------------------------------------------------------//
//                          CALLBACKS                                 //
//--------------------------------------------------------------------//
void RevHead::callback_new_defs(GtkWidget *widget, RevHead *rh)
{
rh->destroy_menu(rh);
rh->nd = new Defaults(rh);  
rh->nd->get_new_defs();
}
void RevHead::callback_update(GtkWidget *widget, RevHead *rh)
{
rh->destroy_menu(rh);
rh->ud = new Update(rh);
rh->ud->DoUpdate();
}
void RevHead::callback_touch(GtkWidget *widget, RevHead *rh)
{
rh->destroy_menu(rh);
rh->ff = new Fixfiles(rh);
rh->ff->create_display(TOUCH);
}
void RevHead::callback_fixdate(GtkWidget *widget, RevHead *rh)
{
rh->destroy_menu(rh);
rh->ff = new Fixfiles(rh);
rh->ff->create_display(FIXDATE);
}

void RevHead::callback_env(GtkWidget *widget, RevHead *rh)
{
rh->destroy_menu(rh);
rh->ev = new Environment(rh);

// Needs to check that it is readable as well
// string pathname = rh->project_path + "/"  + Changelog::RH_ChangeLogTXT;

if ( isreadable_file(rh->ev->Env_FileTXT.c_str()) )  // returns 1 if file exists
  {
	  /*** Project has already been logged so get updated  details ***/            
  rh->ev->get_env(TRUE);
  }
else
  {
	  /*** Project not yet logged - get details from user ***/
  rh->ev->get_env(FALSE);
  }
}




void RevHead::callback_change_directory(GtkWidget *widget, RevHead *rh)
{
rh->destroy_menu(rh);
rh->cd = new ChangeDir(rh);
rh->cd->change_dir("CHANGE DIRECTORY", FALSE);
	
}
  


void RevHead::callback_cancel(GtkWidget *widget, RevHead *rh)
{
Complain *cn = new Complain(rh->icon_pixbuf, &black, NULL, GTK_STOCK_CANCEL, GTK_STOCK_OK);
cn->complain(CN_YES_NO, (char *)"FINISHED?", (char *)"Are you sure you have finished?", QuitFunc, NULL, rh);
}

void RevHead::callback_about(GtkWidget *widget, RevHead *rh)
{
rh->destroy_menu(rh);
cout << "RevHead::callback_about" << endl;
if (!rh->ab) rh->ab = new About(rh);
}

string RevHead::get_project_name_and_version(void)
{
p_name_and_version.clear();
if ( p_name.empty() ) p_name_and_version += "(PROJECT NOT YET NAMED)";
else p_name_and_version += p_name;

//cout << "RevHead::get_project_name_and_version p_name= \"" << p_name << "\"" << endl;

p_name_and_version += "-";
if (p_major_version.empty()) p_name_and_version += "?";
else p_name_and_version += p_major_version;

p_name_and_version += '.';
      
if (p_minor_version.empty()) p_name_and_version += "?";
else p_name_and_version += p_minor_version;

if (flags.p_want_micro)
  {
  p_name_and_version += '.';
      
  if (p_micro_version.empty()) p_name_and_version += "?";
  else p_name_and_version += p_micro_version;
  }

return p_name_and_version;
}


void RevHead::FrontPanel(void)
{
GtkWidget *button;

UNLESS (rc == EXIT_SUCCESS) quit->cleanexit(rc);
  
DestroyWidgets();
  
  
if (ff) delete ff;
ff = NULL;

if (nd) delete(nd);
nd = NULL;

if (ud) delete(ud);
ud = NULL;

if (ev) delete(ev);
ev = NULL;

if (cd) delete(cd);
cd = NULL;

if (ab) delete(ab);
ab = NULL;

if (flags.rh_starting)
  {
  string buffer;
  Defaults *nd = new Defaults(this);

  changelog_name = "ChangeLog";  // default
  init_flags_and_data();

  np_owner.clear();
  np_owner_email.clear();
  np_copyright.clear();
  np_address.clear();
  np_major_version.clear();
  np_minor_version.clear();
  np_micro_version.clear();

  nf_author.clear();
  nf_author_email.clear();
  nf_major_version.clear();
  nf_minor_version.clear();
  nf_micro_version.clear();

  path.clear();
  project_path.clear();
  flags.environment_required = 1;

// PROBLEM - current directory (this->path) has not been fixed at first startup
// but we do need the log

  if (path.empty())
    {
    char *cpath = get_current_dir_name();
    path = project_path = cpath;
    free(cpath);
    }

  logfile->open(TRUE);
  logfile->write2log( buffer = "Initializing RevHead" );

  get_prog_pathname();
  get_current_pathname();
  project_path = path;
   
  nd->load();

  buffer = "RevHead's pathname is: \"" + progname;
  buffer += '\"';
  logfile->write2log(buffer);
  

  buffer = "Current pathname is: \"" + path += '\"';
  logfile->write2log(buffer);  

  buffer = "Project pathname is: \"" + project_path += '\"';
  logfile->write2log(buffer);  
    
  Environment ev(this);
  ev.load_environment();
  
  cout << "RevHead::FrontPanel - done read_env()" << endl;

  flags.rh_starting = 0;
  buffer = "Initialization complete - ";
  if  (flags.newproject) buffer +=  "NEW PROJECT";
  else
    {
    buffer += "EXISTING PROJECT - " + ( get_project_name_and_version() );
    }
  logfile->report( buffer );

  cout << buffer << endl;
  logfile->changed = FALSE;
  FrontPanel();
  }
else
  {
  create_menu(this);
  menu_id = g_signal_connect(G_OBJECT(window), "button_press_event", G_CALLBACK(callback_menu), this);

  gtk_window_resize(GTK_WINDOW(window), width, height);


	GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
	gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);


	GtkWidget *vvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
	gtk_box_set_homogeneous (GTK_BOX (vvbox), TRUE);

		
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), vvbox, TRUE, TRUE, 50);
 
  cout << "RevHead::FrontPanel - AFTER initial start" << endl;
  Date dt(this);

  dt.get_current_time(); 
  
  gtk_label_set_text (GTK_LABEL (title), path.c_str());

  button = CreateButton(vvbox, (char *)DefaultsTXT.c_str() );
  decorate_widget(button, &black, NULL, &red, NULL, NULL);    
  g_signal_connect (G_OBJECT (button), "button_press_event",
                        G_CALLBACK(callback_menu), (gpointer)this);      
	gtk_widget_set_tooltip_text (GTK_WIDGET(button),
      "DEFAULTS\nDefaults for a new project");

		
  button = CreateButton(vvbox, (char *)Update::UpdateTXT.c_str() );
  decorate_widget(button, &black, NULL, &red, NULL, NULL);    
  g_signal_connect (G_OBJECT (button), "button_press_event",
                        G_CALLBACK(callback_menu), (gpointer)this);      
	gtk_widget_set_tooltip_text (GTK_WIDGET(button),
      "UPDATE\nUpdate a project with selected information");                      
  if (flags.environment_required)  gtk_widget_set_sensitive(button, FALSE); // Deactivate this button
      
  button = CreateButton(vvbox, (char *)Fixfiles::TouchTXT.c_str() );
  decorate_widget(button, &black, NULL, &red, NULL, NULL);      
  g_signal_connect (G_OBJECT (button), "button_press_event",
                        G_CALLBACK(callback_menu), (gpointer)this);      
	gtk_widget_set_tooltip_text (GTK_WIDGET(button),
      "TOUCH\nTouch selected files as if they had just been written");
                             
  button = CreateButton(vvbox, (char *)Fixfiles::FixDateTXT.c_str() );
  decorate_widget(button, &black, NULL, &red, NULL, NULL);    
  g_signal_connect (G_OBJECT (button), "button_press_event",
                        G_CALLBACK(callback_menu), (gpointer)this);      
	gtk_widget_set_tooltip_text (GTK_WIDGET(button),
      "FIXDATE\nMake the datestamp of selected files agree with their header datestamps");
		
  button = CreateButton(vvbox, (char *)P_EnvironmentTXT.c_str() );
  decorate_widget(button, &black, NULL, &red, NULL, NULL);                    
  g_signal_connect (G_OBJECT (button), "button_press_event",
                        G_CALLBACK(callback_menu), (gpointer)this);                    
	gtk_widget_set_tooltip_text (GTK_WIDGET(button),
      "ENVIRONMENT\nSelect programming language(s), logfile preferences etc. for this project");
		
  button = CreateButton(vvbox, "CHANGE CURRENT DIRECTORY");
  decorate_widget(button, &black, NULL, &red, NULL, NULL);                    
  g_signal_connect (G_OBJECT (button), "button_press_event",
                        G_CALLBACK(callback_menu), (gpointer)this);                    
	gtk_widget_set_tooltip_text (GTK_WIDGET(button),
      "CHANGE DIRECTORY\nChange to another project directory.\n"
      "Optionally create a new directory.");

  button = CreateButton(vvbox, "FINISHED");
  decorate_widget(button, &black, NULL, &red, NULL, NULL);                    
  g_signal_connect (G_OBJECT (button), "button_press_event",
                        G_CALLBACK(callback_menu), (gpointer)this);	
	gtk_widget_set_tooltip_text (GTK_WIDGET(button),
      "FINISHED\nQuit completely");
		
  gtk_widget_show_all(hbox);

  cout << "RevHead::FrontPanel - leaving starting" << endl;
  if ( ( gtk_main_level() ) > 0 )
    {
    gtk_main_quit(); // exit existing level
    gtk_main();      // Start new level : seems we have to have this  
    }
  }
}


/***********************************************
 * Fix log file when changing directories.     *
 * Close the existing log and start a new one. *
 ***********************************************/
void RevHead::fix_dir_change(char *newdir)
{
string buffer;
char olddir[PATH_MAX];
Date dt(this);

logfile->report( buffer="\nCHANGING DIRECTORIES\n" );

UNLESS (getcwd(olddir, PATH_MAX))
    {
    quit->HandleError(PATH_F_ERR, NULL, NULL);
    return;
    }

buffer = "FROM \"";
buffer += olddir;
buffer += "\" TO \"";
buffer += newdir;
buffer += "\"\n";;
logfile->report(buffer);
  
// If there is a log file, close it
dt.date_log_operation(&(buffer = "CLOSING THIS LOG FILE NOW"));
logfile->close();


// Now change to the new directory
if (chdir(newdir)) 
    {
    quit->HandleError(PATHLOCK_F_ERR, newdir, NULL);
    return;
    }

// Fix up new logfile
logfile->open(TRUE);  // default, but I'm not sure what else we need

buffer = "Changed to directory \"";
buffer += newdir;
buffer += "\" from\"";
buffer += olddir;
buffer += "\" OK";
logfile->report(buffer);
return;
}



/********************************************************
 * Read the args and if necessary fix the current	      *
 * directory to that.					                          *
 ********************************************************/
void RevHead::get_current_pathname(void)
{
int	n, numargs;
char	**arglist;
gboolean	PathFixed = FALSE;
  
if (argc)
    {
    arglist = &argv[1];
    numargs = argc - 1;
  
      
    char oldpath[PATH_MAX];

    UNLESS (getcwd(oldpath, PATH_MAX))
        {
        quit->HandleError(PATH_F_ERR, NULL, NULL);
        return;
        }

  
    for (n=0; n<numargs; n++)
        {
        if (strcmp(arglist[n], "?") == 0)
            {
            quit->Usage(0L, NULL, NULL);
            return;
            }
        
        else if ( STRIEQN(arglist[n], (char *)RevHead::PathTXT.c_str(), 4) &&
                ((arglist[n][4] == '=') || (arglist[n][4] == '\0')) )
            {
            if (PathFixed)
                {
                quit->Usage(ARG_TWICE_F_ERR, (char *)RevHead::PathTXT.c_str(), NULL);
                return;
                }

            //if (RevHead::rh->path) free(RevHead::rh->path);
            path.clear();
            if (arglist[n][4] == '=')
                {
                path = &arglist[n][5];
                }
            else if ((arglist[n][4] == '\0') && (++n < numargs))
                {
                path = arglist[n];
                }
            else
                {
                quit->Usage(MISSING_ARG_F_ERR, (char *)RevHead::PathTXT.c_str(), NULL);
                return;
                }

            PathFixed = TRUE;
            }
	
        else UNLESS (PathFixed)
            {
            //if (RevHead::rh->path) free(RevHead::rh->path);
            path.clear();
            path = arglist[n];
            PathFixed = TRUE;		
            }
	
	/* DEFAULT */
	
        else
            {
            quit->Usage(ARG_TWICE_F_ERR, (char *)RevHead::PathTXT.c_str(), NULL);
            return;
            }
        }
    }

  
if (PathFixed)   // We have to change to a new directory
    {
    // GET THE FULL PATHNAME!
    path = canonicalize_file_name(path.c_str());

    // Really need to check it is a directory as well
    if (!isdir(path.c_str()))
        {
        quit->Usage(PATHLOCK_F_ERR, (char *)path.c_str(), NULL);
        return;
        }
 
    fix_dir_change((char *)path.c_str());

    }
else             // We get this directory's name into RevHead.path
    {
    char *cpath;
      
      
    path.clear();
      
    cpath = get_current_dir_name();
    path = cpath;
    free(cpath);

    if (path.empty())
        {
        quit->HandleError(PATH_F_ERR, NULL, NULL);
        return;
        }
    }
return;
}



gboolean RevHead::get_pathname_from_path(char *fname, char *pathname)
{
char *path;
char *dir, *c, *t;   // directory entry, character, termination (':')
char *pn;

dir = path = getenv("PATH");
UNLESS (path) return(FALSE);

while (*dir)
    {
    pn = pathname;
    c = dir;

    FOREVER
        {
        if ((*c == ':') || (*c == 0))
            {
            *pn = 0;
            t = c;
            break;
            }
        *pn++ = *c++;
        }

        if (*pathname) // if there was a path in this slot!
            {
            UNLESS (pathname[strlen(pathname)-1] == '/')
                {
                strcpy(pn, "/");
                }
            strcat(pn, fname);

//cout << "RevHead::get_pathname_from_path - checking \"" << pathname << "\"" << endl;

        // Check this pathname
        if (isfile(pathname))	// if it exists, FOUND IT!
            {
            if ( *(t+1) ) // refix PATH
                *t = ':';
            return(TRUE);
            }
        }

    if ( *(t+1) ) // refix PATH
        *t = ':';
        
    dir = t + 1;
    }

return(FALSE); // failed!
}




// Must come AFTER quit has been fully initialized
void RevHead::get_prog_pathname(void)
{
char currentdir[PATH_MAX];
char progdir[PATH_MAX];
char arg[PATH_MAX];
char *c;
char fn;

if (getcwd(currentdir, PATH_MAX))
  {  
  strcpy(arg, argv[0]);
  c = &arg[strlen(arg) - 1];

  while (c >= arg)
    {
    if (*c == '/')
      {
      break;
      }
    c--;
      }

    // If this was a command without a directory path it is probably OK
    // to use just the name, but better to have the full pathname.
    // Need to do this from the PATH environment variable

//  if (c < arg)  // If this was a command without a directory path
  if (*c != '/')
    {
//    cout << "RevHead::get_prog_pathname - NO PATHNAME IN ARG" << endl;

    if (get_pathname_from_path(arg, progdir))
      {
      progname  = progdir;
      return;
      }


    // Can't get program's path!
    cerr << "RevHead::get_prog_pathname - Can't get program's path! - arg is \"" << arg << "\"" << endl;
    quit->HandleError(PATHLOCK_F_ERR, currentdir, NULL);
    return;
    }
    
  fn = *c;
  *c = 0;

  if (chdir(arg)) 
    {
    cerr << "RevHead::get_prog_pathname - Can't change directory to \"" << arg << "\"" << endl;
    quit->HandleError(PATHLOCK_F_ERR, arg, NULL);
    return;
    }
    
  UNLESS (getcwd(progdir, PATH_MAX)) 
        {
        cerr << "RevHead::get_prog_pathname - Can't get current directory (progdir)!" << endl;
        quit->HandleError(PATH_F_ERR, NULL, NULL);
        return;
        }

  if (chdir(currentdir)) 
    {
    cerr << "RevHead::get_prog_pathname - Can't change directory to \"" << currentdir << "\"" << endl;
    quit->HandleError(PATHLOCK_F_ERR, currentdir, NULL);
    return;
    }

  *c = fn;
  if ((progdir[strlen(progdir)-1] != '/') && (fn != '/')) strcat(progdir, "/");
    
  strcat(progdir, c);
  progname = progdir;
  return;
  }

cerr << "RevHead::get_prog_pathname - Can't get current directory (at function start)" << endl;
quit->HandleError(PATH_F_ERR, NULL, NULL); 
return;
}

void RevHead::init_flags_and_data(void)
{
    
// Zero flags
flags.newproject        =      // 1 if new project (no .revhead.project)
flags.newname           =      // 1 if project name changed
flags.newowner          =      // 1 if copyright owner changed
flags.newemail          =      // 1 if project email address changed
flags.new_major_version =      // 1 if version changed
flags.new_minor_version =      // 1 if version changed
flags.new_micro_version =      // 1 if version changed
flags.newcopyright      =      // 1 if copyright changed
flags.newaddress        =      // 1 if address changed
flags.newsummary        =      // 1 if summary changed
flags.p_changed         =      // 1 if any of above = 1
flags.pc_changed        =      // 1 if comment

flags.append_log        =      // 1 if new log appended to old
flags.log_to_cerr       =      // 1 if log to be sent also to cerr

flags.c_file            =      // 1 if C to be processed
flags.cpp_file          =      // 1 if C++ to be processed
flags.c_v               =      // 1 if C version file required
flags.cpp_t             =      // 1 if CPP 1 type comments
flags.cpp2_t            =      // 1 if CPP 2 type comments
    
flags.ada_file          =      // 1 if ADA to be processed
flags.ada_v             =      // 1 if ADA version file required
    
flags.php_file          =      // 1 if PHP to be processed
flags.php_c_t           =      // 1 if PHP C type comments
flags.php_cpp_t         =      // 1 if PHP CPP 1 type comments
flags.php_cpp2_t         =     // 1 if PHP CPP 2 type comments
flags.php_h_t           =      // 1 if PHP # type comments
flags.php_v             =      // 1 if PHP version file required

flags.java_file         =      // 1 if JAVA to be processed
flags.java_c_t          =      // 1 if JAVA C type comments
flags.java_cpp_t        =      // 1 if JAVA 1 CPP type comments
flags.java_cpp2_t       =      // 1 if JAVA 2 CPP type comments
flags.java_v            =      // 1 if JAVA version file required
    
flags.perl_file         =      // 1 if PERL to be processed
flags.perl_v            =      // 1 if PERL version file required

flags.fortran_file      =      // 1 if FORTRAN to be processed
flags.fortran_v         =      // 1 if FORTRAN version file required

        
flags.p_auto_inc        =      // 1 if auto increment project versions

flags.f_auto_inc        =      // 1 if auto increment file versions

flags.abort             = 0;   // 1 if fatal error or "abort"

// These flags default to 1
flags.p_want_micro      =      // default, 1 if micro project version wanted
flags.want_log          = 1;   // default, 1 if log file required
    
p_name.clear();
p_owner.clear();
p_owner_email.clear();
p_copyright.clear();
p_major_version.clear();
p_minor_version.clear();
p_micro_version.clear();
p_copyright.clear();
p_address.clear();
p_comment.clear();
p_summary.clear();

source_path.clear();   // list of sources

changelog_path.clear();
changelog.clear();
version_files.clear();
no_auto_files.clear();

}

// Popup Menu Stuff

gint RevHead::callback_menu(GtkWidget *widget, GdkEventButton *event, RevHead *rh)
{   
if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) )
  {
  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
                        event->button, event->time);
  return TRUE;
  }
else if (GTK_IS_BUTTON(widget))
  {
  string label = gtk_button_get_label(GTK_BUTTON(widget));

  if (label == DefaultsTXT) callback_new_defs(widget, rh);
  else if (label == Update::UpdateTXT) callback_update(widget, rh);
  else if (label == Fixfiles::TouchTXT) callback_touch(widget, rh);
  else if (label == Fixfiles::FixDateTXT) callback_fixdate(widget, rh);
  else if (label == P_EnvironmentTXT) callback_env(widget, rh);
  else if (label == "CHANGE CURRENT DIRECTORY") callback_change_directory(widget, rh);
  else if (label == "ABOUT") callback_about(widget, rh);
  else if (label == "FINISHED") callback_cancel(widget, rh);
  return TRUE;
  }

return FALSE;
}

void RevHead::destroy_menu(class RevHead *rh)
{
g_signal_handler_disconnect(G_OBJECT(rh->window), rh->menu_id);
gtk_widget_destroy(rh->menu);
rh->menu = NULL;
}

void RevHead::create_menu(RevHead *rh)
{
GtkWidget *menu_item;
    
static const char * action_txt[] =
    {
    DefaultsTXT.c_str(),
    Update::UpdateTXT.c_str(),
    Fixfiles::TouchTXT.c_str(),
    Fixfiles::FixDateTXT.c_str(),
    P_EnvironmentTXT.c_str(),
    "CHANGE CURRENT DIRECTORY",
    "ABOUT",
    "FINISHED"
    };

static void (*action[])(GtkWidget *widget, RevHead *rh) =
  {
  callback_new_defs,
  callback_update,
  callback_touch,
  callback_fixdate,
  callback_env,
  callback_change_directory,
  callback_about,
  callback_cancel
  };


menu = gtk_menu_new ();

for (int i = 0; i < 8; i++)
  {

      /* Create a new menu-item with action name... */
  menu_item = gtk_menu_item_new_with_label (action_txt[i]);

            /* ...and add it to the menu. */
  gtk_menu_shell_append (GTK_MENU_SHELL(menu), menu_item);

            /* do the action assciated with this action name */

//---------------------------------------------------------//
// rh doesn't get passed properly properly to the action   //
// Can't see what's wrong...                               //
//---------------------------------------------------------//
      //      g_signal_connect_swapped  (GTK_OBJECT (menu_item), "activate",
//                GTK_SIGNAL_FUNC (action[i]), rh);

//---------------------------------------------------------//
// THIS ONE WORKS!                                         //
//---------------------------------------------------------//
  g_signal_connect_after(G_OBJECT (menu_item), "activate",
                G_CALLBACK(action[i]), rh);

            /* Show the widget */
  gtk_widget_show (menu_item);
  }
}



// This is the entry point from main()
RevHead::RevHead(int numargs, char **args, Quit *qt)
{
GdkScreen *screen;
gint swidth;
gint sheight;

cout << "RevHead:" << endl;

flags.rh_starting = 1;

ff = NULL;
nd = NULL;
ud = NULL;
ev = NULL;
cd = NULL;
ab = NULL;

file_list.clear();
window = vbox = status = title = NULL;
quit = qt;
quit->set_revhead(this);  
logfile = new Logfile(this);  

argc = numargs;
argv = args;


//progname = args[0];    
//cout << "RevHead::RevHead -  progname= " << progname << endl;


dp = NULL;
direntry = NULL;

numerrs = 0;
rc = EXIT_SUCCESS;


// --- GTK initialization ---
gtk_init (&argc, &argv);

// Get current path into RevHead, Linux does this correctly from a link
// NOTE: this doesn't give a "/" terminator unless it is the "/" directory

// --- Create the top window --- 
window = gtk_window_new (GTK_WINDOW_TOPLEVEL);  // Assume success for now

  
// --- Give the window a title ---
gtk_window_set_title (GTK_WINDOW (window),  RH_PROJECT_NAME"-"RH_PROJECT_VERSION);

screen = gtk_window_get_screen(GTK_WINDOW(window));
swidth = gdk_screen_get_width(screen);
sheight = gdk_screen_get_height(screen);

//cout << "RevHead::RevHead - swidth,sheight= " << swidth << " : " << sheight << endl;

  
// --- Set the window size ---
//gtk_widget_set_usize (window, 600, 520); // deprecated

// Use height - aspect ratio is then unimportant
if (sheight >= 1200)
  {
  width =  2*sheight/3;
  height = 2*sheight/3;    
  }
else if (sheight >= 600)
  {
  width =  3*sheight/4;
  height = 3*sheight/4;    
  }

else
  {
  width =  swidth/2;
  height = 9*sheight/10;    
  }

//cout << "RevHead::RevHead - width,height= " << width << " : " << height << endl;
  
gtk_window_set_default_size(GTK_WINDOW(window), width, height);  // width, height

// --- Set the window icon
icon_pixbuf = gdk_pixbuf_new_from_inline(-1,
					   rh_pixbuf_source,
					   FALSE,
					   NULL);
gtk_window_set_icon( GTK_WINDOW(window), icon_pixbuf);

// --- Connect the delete_event to the main window ---
g_signal_connect (G_OBJECT (window), "delete_event",
                            G_CALLBACK(CloseAppWindow), NULL);

gtk_window_move(GTK_WINDOW(window),
    (swidth - width)/2,
    (sheight - height)/2 );


vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
gtk_box_set_homogeneous (GTK_BOX (vbox), FALSE);

// --- create a new label  ---
status = gtk_label_new ("STATUS: ");
	
// --- Align the label to the left side ---
gtk_misc_set_alignment (GTK_MISC (status), 0, 0);

  /* --- Pack the label into the vertical box (vbox box1).  --- */
gtk_box_pack_start (GTK_BOX (vbox), status, FALSE, FALSE, 0);
	
/* --- Show the label  --- */
gtk_widget_show (status);

title = gtk_label_new ("STARTING");
gtk_misc_set_alignment (GTK_MISC (title), .5, 0);
gtk_box_pack_start (GTK_BOX (vbox), title, FALSE, FALSE, 0);
gtk_widget_show (title);

/* --- Make them visible --- */
gtk_container_add (GTK_CONTAINER (window), vbox);

gtk_widget_show (vbox);


gtk_widget_add_events(window, GDK_BUTTON_PRESS_MASK);

gtk_widget_show (window);
FrontPanel();

cout << "RevHead::RevHead - starting gtk_main" << endl;

gtk_main();

}



RevHead::~RevHead(void)
{
cout << "~RevHead:" << endl;

if (nd) delete(nd);
nd = NULL;
if (ff) delete(ff);
ff = NULL;
if (ud) delete(ud);
ud = NULL;
if (ev) delete(ev);
ev = NULL;
if (cd) delete(cd);
cd = NULL;
if (ab) delete(ab);
ab = NULL;

if (icon_pixbuf)
  {
  g_object_unref(G_OBJECT(icon_pixbuf));
  icon_pixbuf = NULL;
  }

}

