/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       update.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       15-Apr-2013 13.12.41 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Most of the functions for updating file headers                           *
* 04-Jul-1999 09.18.06 -36000  0.00    David M. Balean                      *
* 04-Jul-1999 09.18.06 -36000  0.01    David M. Balean                      *
* Now checks that an update actually occurred before                        *
* re-writing RH_History.log                                                 *
* Also fixes the date problem in RH_version files where no change.          *
* 04-Jul-1999 09.18.06 -36000  0.02    David M. Balean                      *
* Dating the log file for project updates                                   *
* 04-Jul-1999 09.18.06 -36000  0.03    David M. Balean                      *
* Minor changes                                                             *
* 04-Jul-1999 09.18.06 -36000  0.04    David M. Balean                      *
* Added code to deal with RH_History.log for incrementing version numbers   *
* 04-Jul-1999 09.18.06 -36000  0.05    David M. Balean                      *
* Minor changes to give more info to the log file                           *
* 04-Jul-1999 09.18.06 -36000  0.06    David M. Balean                      *
* Minor changes                                                             *
* 04-Jul-1999 09.18.06 -36000  0.7     David M. Balean                      *
* Added extra function for the current datestamp                            *
* 15-Oct-2001 16.19.13 -36000  0.8     David M. Balean                      *
* 20-Aug-2004 12.10.29 -36000 0.12    David M. Balean                       *
* Vast improvement but same basic idea                                      *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <algorithm>

#include <libgen.h>

#include "update.h"
#include "logfile.h"
#include "filedetails.h"
#include "environment.h"
#include "date.h"
#include "tfile.h"
#include "misc.h"
#include "quit.h"
#include "header.h"

using namespace misc;
using namespace std;


string Update::hname         = "RH_version.h";       // For C & CPP files
string Update::phpname       = "RH_version.php";     // For PHP files
string Update::adaname       = "rh_version.ads";     // For ADA files - all lower case
string Update::javaname      = "RH_version.java";    // For JAVE files
string Update::perlname      = "RH_version.pl";      // For PERL files
string Update::fortranname   = "RH_version.f95";     // For FORTRAN files


string Update::UpdateTXT     = "UPDATE THIS PROJECT";


Update::Update(RevHead *revhead)
{
cout << "Update:\n";
rh = revhead;
fd_list = NULL;
gp = NULL;
f_ds = f_tz = 0;
}


Update::~Update(void)
{
GList *node;

cout << "~Update:\n";

for (node=fd_list; node; node=node->next)
    {
    delete( (FileDetails *)node->data );
    }
g_list_free(fd_list);
fd_list = NULL;
    
if (gp) delete(gp);
gp = NULL;
}
 

/********************************
 * Update()                     *
 ********************************/

void Update::DoUpdate(void)
{
Date dt(rh);
gp = new GetProject(rh);
  
cout << "Update::DoUpdate - ENTERED" << endl;

dt.date_log_operation(&UpdateTXT);
dt.get_current_time();
	
cout << "Update::DoUpdate - 1" << endl;

if (!change_directory(TRUE)) // initialize
  {
  string buffer = "UPDATE FAILED - NO SOURCE directory";
  rh->logfile->report(buffer);
  return;
  }
	
cout << "Update::DoUpdate - 2" << endl;

// Also get a list of which version file types are wanted
version_list.clear();
omit_version_list.clear();
	
cout << "Update::DoUpdate - 3" << endl;
  
//if ( (rh->flags.c_file || rh->flags.cpp_file) && rh->flags.c_v)
if (rh->flags.c_v)
  {
  version_list.push_back(C_TYPE);
  }
else
  {
  omit_version_list.push_back(C_TYPE);
  }
	
cout << "Update::DoUpdate - 4" << endl;
  
if (rh->flags.ada_v) version_list.push_back(ADA_TYPE);
else omit_version_list.push_back(ADA_TYPE);
  
if (rh->flags.php_v) version_list.push_back(PHP_TYPE);
else omit_version_list.push_back(PHP_TYPE);
  
if (rh->flags.java_v) version_list.push_back(JAVA_TYPE);
else omit_version_list.push_back(JAVA_TYPE);
  
if (rh->flags.perl_v) version_list.push_back(PERL_TYPE);
else omit_version_list.push_back(PERL_TYPE);
  
if (rh->flags.fortran_v) version_list.push_back(FORTRAN_TYPE);
else omit_version_list.push_back(FORTRAN_TYPE);
	
  
cout << "Update::DoUpdate - 5 -version_list:-" << endl;
	
list<int>::iterator i;  
for (i=version_list.begin(); i!=version_list.end(); ++i)
  {
  cout << "    " << *i << endl;
  }

cout << "Update::DoUpdate - 6 -omit_version_list:-" << endl; 
	
for (i=omit_version_list.begin(); i!=omit_version_list.end(); ++i)
  {
  cout << "    " << *i << endl;
  }
  
  
  
if (rh->flags.newproject)
  {
  // NOTE: To get here we have a new project
  gp->get_project(FALSE);
  }
else
  {
  // NOTE: To get here we have logged the project
  gp->get_project(TRUE);	
  }

cout << "Update::DoUpdate - source list:-" << endl;
list<string>::iterator s;  
for (s=rh->source_path.begin(); s!=rh->source_path.end(); ++s)
  {
  cout << "    " << *s << endl;
  }

cout << "Update::DoUpdate - LEAVING" << endl;

return;
}

bool Update::change_directory(bool start)
{
static  list<string>::iterator i;
string buffer;
  
if (start)
  {
  i=rh->source_path.begin();
//  return TRUE;
  }
if (rh->source_path.empty())
  {
  return FALSE;
  }
if (i==rh->source_path.end())
  {
  return FALSE;
  }

// Change to this directory

if (chdir( (*i).c_str()) )
  {
  rh->quit->HandleError(PATHLOCK_F_ERR, (char *)(*i).c_str(), NULL);
  return FALSE;
  }


cout << "Update::change_directory - " << buffer << endl;

rh->path = (*i);
  
++i;
return TRUE;
}


// Check if "This File" is the same as fname 
bool Update::filename_OK(string &temp, string &pathname, string &fname, int comment_type)
{
fstream ifp;
string buffer;

ifp.open(pathname.c_str(), ios::in | ios::binary);
if (ifp.is_open())
  {
  string tail     = Header::in_comment_end[comment_type];
  string key_head = Header::in_comment_key_start[comment_type];


  string test_str;

  // Check for RevHead_Start first, make sure it is logged...
  test_str = key_head + RevHead::RH_Start;

  if (rh->find_string(ifp, test_str, buffer) == 1)
    {
    test_str = key_head + RevHead::F_ThisFileTXT;

    if (rh->find_string(ifp, test_str, buffer) == 1)
      {
      remove_head_and_tail(buffer, test_str, tail);
      if (buffer == fname)
        {
        ifp.close();
        temp = buffer;
        return TRUE;
        }
      }
    }
  ifp.close();
  }
temp = buffer;
return FALSE;
}





/* 
   Return FALSE if more to check
   or TRUE when all files have been checked
*/

// this has to be recursive - what a pain
// We need to be already in rh->source_path when entering
gboolean Update::CheckFiles(void)
{
class Files  *f;
  
time_t      ds;          // These 3 as per file's datestamp
time_t      tz;
int         daylight;

struct stat statbuf;        // to get file dates
fstream     ifp;
string      buffer;
string      test_str;
string      fname;
string      pathname;
int         type;			    // Type of current file
int         version_type; // Type for version file
Date        dt(rh);
FileDetails *fd;
guint timer;
  
//cout << "Update::CheckFiles - ENTERED 1" << endl;

UNLESS (rh->rc == EXIT_SUCCESS)
  {
//cout << "Update::CheckFiles - ENTERED 2" << endl;
  return(TRUE);
  }
//cout << "Update::CheckFiles - ENTERED 3" << endl;

// I'm not convinced that this is required
// Why not simply "FileDetails fd(rh)" ?
// Then it would lapse of its own accord.
// Experiment later

if (fd_list)
    {
//    cout << "Update::CheckFiles - using existing fd_list" << endl;
    fd = (FileDetails *)fd_list->data;
    }
else
    {
//    cout << "Update::CheckFiles - using new fd_list" << endl;

    fd = new FileDetails(rh);  // We need to use new for this and maintain a list
    fd_list = g_list_append(fd_list, fd);
    }

// Open current directory if not already open
UNLESS (rh->dp)
  {
  UNLESS ( (rh->dp = opendir( (char *)rh->path.c_str() )) )
    {
    rh->rc = EXAMINE_F_ERR;
    rh->quit->HandleError(EXAMINE_F_ERR , (char *)rh->path.c_str(), NULL);
    return(TRUE);
    }

  cout << "Update::CheckFiles - Opened directory " << rh->path << endl;

  buffer = "\nCHECKING FILES - SOURCE directory \"" + rh->path + "\"";
  rh->logfile->report(buffer);
  }

//else cout << "Update::CheckFiles - using existing rh->dp" << endl;

  
// CHECK THE FILES - when no direntry we are done
if ( (rh->direntry = readdir(rh->dp)) )
  {
  fname = rh->direntry->d_name;
  pathname = rh->path + "/" + fname;

  cout << "Update::CheckFiles -  got file " << fname << endl;
      
  // Notify user
  buffer = "Checking \"" + fname + '\"';
  while (buffer.length() < 33) buffer += ' ';
  buffer += " : ";


  // TRUE returns type if this type is to be processed otherwise NO_TYPE
  // Version files are "NO_TYPE" - skip if "NO_TYPE"
  type = rh->get_file_type(fname, TRUE);

  if (type == NO_TYPE)
    {
    buffer += "SKIPPED";
    rh->logfile->write2log(buffer);	  
    return(CheckFiles());  // Continue checking the files
    }  

  version_type = (type==CPP_TYPE) ? C_TYPE : type;

  // make sure we can read and write and that it is a file
  // Could be better...
  if ( !isfile(pathname.c_str()) || (access( pathname.c_str(), R_OK | W_OK | F_OK ) ) )
    {
    rh->rc = ACCESS_F_ERR;
    rh->quit->HandleError(ACCESS_F_ERR , pathname.c_str(), NULL);
    return(TRUE);
    }

// Should this be lstat64 ?
  if ( stat(pathname.c_str(), &statbuf) )
    {
    rh->rc = ACCESS_F_ERR;
    rh->quit->HandleError(ACCESS_F_ERR, pathname.c_str(), NULL);  // Change this later to STAT_F_ERR
    return(TRUE);
    }

// ==================================================================
// PUT THIS FILE TYPE'S VERSION FILE HERE IF WANTED AND STILL IN LIST
// AND REMOVE IT FROM THE LIST
// ==================================================================
    
cout << "Update::CheckFiles - about to check" << endl;

// PUT THIS in Update Files I think
  if (binary_search(version_list.begin(), version_list.end(), version_type))
    {
    write_version(type);       // this should write to rh->path, not rh->project_path
    version_list.remove(type);
    if (type == CPP_TYPE)
      {
      version_list.remove(C_TYPE);
      omit_version_list.remove(C_TYPE);
      }
    if (type == C_TYPE)
      {
      //omit_version_list.remove(CPP_TYPE);
      version_list.remove(CPP_TYPE);
     
      omit_version_list.remove(CPP_TYPE);
      }
    }

  // The OS files values of this file for filedetails if needed
  f_ds = statbuf.st_mtime;   // UTC
  f_tz = rh->p_currenttz;    // Computer's time zone

// We also need daylight as per rh->f_ds, calculate later as needed


  // We have to make this assumption!
  // "tz" should really be obtained from the file but statbuf
  // is this computer's time so the computer's "tz" it is.
  // We also need "daylight", seconds of daylight saving if applicable.

  f_author.clear();
  f_author_email.clear();
  f_major_v.clear();
  f_minor_v.clear();
  f_micro_v.clear();
  f_summary.clear();

  f_want_version = binary_search(rh->version_files.begin(),
                                 rh->version_files.end(),
                                 pathname);

  f_no_auto_inc = binary_search(rh->no_auto_files.begin(),
                                rh->no_auto_files.end(),
                                pathname);

// Do we have the file logged?  Check for $RevHead_Start....
// NOTE: this doesn't check that the comments characters etc. are correct

  if (! ( rh->is_logged( fname, f_comment_type ) ) )
    {
// ------------------- FILE HAS NOT YET BEEN LOGGED ----------------
    rh->logfile->write2log(buffer+="NOT YET LOGGED");     

cout << "Update::CheckFiles - " << buffer << endl;

// THE PROBLEM IS:
//     this returns immediately so it has to be recursive from there to here...

    fd->get_file_details(this, (char *)fname.c_str(), FALSE);  // Comes back to start of this function...
    }

  else
    {
    string temp;

// ------------------- FILE HAS ALREADY BEEN LOGGED ----------------
    buffer += "HAS BEEN LOGGED : ";

cout << "Update::CheckFiles - " << buffer << endl;

// File is logged, get datestamp
      
    if (!(dt.get_date(&ds, &tz, &daylight, pathname, f_comment_type)))
      {
      cerr << "FAILED to obtain datestamp of \"" << pathname << "\"" << endl;
      rh->rc = UNDERSTAND_F_ERR;
      rh->quit->HandleError(UNDERSTAND_F_ERR, pathname.c_str(), NULL);
      return(TRUE);
      }

//================================================================
//
//   INSERT a check that "This File:" still refers to fname
//   Not sure what should be done it it doesn't!
//   WARN I think
//================================================================
    f_old_name.clear();
    if (!filename_OK(temp, pathname, fname, f_comment_type))
      {
      // only if name has changed
      f_old_name = temp;
      }


/* Is File Date later than RevHead Date? (i.e. has it been updated) */
/* ds is the datestamp from the header	*/
/* If fib_Date is later it must be processed */
/* If ds earlier, returns 1	*/
/* If equal returns 0		*/
/* if ds later, returns -1	*/


	  // In Linux we use the st_mtime field of stat
    // The problem here is that we don't know if &statbuf.st_mtime is UTC or local time
    // This assumes UTC. Unix time is measured using UTC so should be right.
    if ((dt.ds_cmp(&ds, &statbuf.st_mtime) <= 0) && f_old_name.empty())
      {
      rh->logfile->report(buffer+="is up to date");     

// FILE UNCHANGED AND UP TO DATE
// but the project details may have changed so record it
      f = new Files;
      UNLESS (f)
        {
        rh->quit->HandleError(MEMORY_F_ERR, pathname.c_str(), NULL);
        return(TRUE);
        }

// Fix up only the details we need details i.e. it's name
// We copy the datestamp but it may be changed later if header date OK
// Author & author's email should be unchanged.

      f->datestamp = ds;      // existing UTC time in datestamp
      f->timezone = tz;       // existing tz in datestamp
      f->daylight = daylight; // existing seconds of daylight saving
      f->name = fname;
      f->comment_type = f_comment_type;
      f->pathname = pathname;         // rh->path + "/" + fname;
      f->flags.logged = 1;
      f->flags.changed = 0;
      f->flags.newsummary = 0;
      f->flags.version_wanted = f_want_version;
      rh->file_list.push_back(*f);
      return(CheckFiles()); // Continue checking the the files
      }
    else
      {
      fstream::pos_type g;       // for current position - pos_type
      string tail     = Header::in_comment_end[f_comment_type];
      string key_head = Header::in_comment_key_start[f_comment_type];
      string head     = Header::in_comment_start[f_comment_type];

//=====================================================
//             FILE IS NOT UP TO DATE 
// As the file is already open, get header details here
//=====================================================
        
// File details as well as maybe the project details to be updated
      rh->logfile->write2log(buffer+="is not up to date");     
      ifp.open(pathname.c_str(), ios::in | ios::binary);
      UNLESS (ifp.is_open())
        {
        rh->rc = OPEN_F_ERR;
        rh->quit->HandleError(OPEN_F_ERR, pathname.c_str(), NULL);
        return(TRUE);
        }
//cout << "Update::CheckFiles - file opened " << endl;

      // Check for RevHead_Start first, make sure it is logged and we are in the header
      test_str = key_head + RevHead::RH_Start;

      if (rh->find_string(ifp, test_str, buffer) == 1)
        {
//cout << "Update::CheckFiles - got  RH_Start" << endl;
          
        g = ifp.tellg();  // start of header info
        
        // Current file version and author
        test_str = key_head + Header::AuthorTXT; 
            
        if (rh->find_string(ifp, test_str, f_author) == 1)
          {
//cout << "Update::CheckFiles - got  AuthorTXT" << endl;

          remove_head_and_tail(f_author, test_str, tail);

// Get the author's email address from author if present
//cout << "Update::CheckFiles - author with email= " << f_author << endl;
              
          for (int n=0; n<f_author.length(); n++)
            {
            if (f_author[n] == '<')
              {
              f_author_email = f_author.substr(n+1);
              f_author = f_author.substr(0, n-1);

              if (f_author_email.length() > 1)
                {
                if (f_author_email[f_author_email.length()-1] == '>')
                  {
                  f_author_email = f_author_email.substr(0, f_author_email.length()-1);
                  }
                }
              else f_author_email.clear();
              break;
              }
            }
          }

        ifp.clear();
        ifp.seekg(g); // Re-position to start of info
        
// File Version    
        test_str = key_head + RevHead::F_VersionTXT; 
        if (rh->find_string(ifp, test_str, buffer) == 1)
          {
          // There is a file version...
          remove_head_and_tail(buffer, test_str, tail);
          rh->get_version(buffer, f_major_v, f_minor_v, f_micro_v);
          }

// File Summary        
        ifp.clear();
        ifp.seekg(g); // Re-position to start of info

        test_str = key_head + RevHead::F_SummaryTXT;

        if ( rh->find_string(ifp, test_str, buffer) == 1)
          {
          FOREVER
            {
            UNLESS (rh->read_line(ifp, buffer))
              {
              ifp.close();
              rh->quit->HandleError(READ_F_ERR, pathname.c_str(), NULL);
              return TRUE;
              }
            remove_head_and_tail(buffer, head, tail);

            if (buffer == (key_head + "*")) break;       // GONE TOO FAR!

            // Now add it to the list
            if (buffer.length())
              {
              f_summary.push_back(buffer);
              }
            else break;                                  // Finished
            }
          }
        }

      ifp.close();

			fd->get_file_details(this, (char *)fname.c_str(), TRUE);
       
      }
    }
  }

  
// I think the extra directory change should come here
if (rh->direntry == NULL)
  {    
  if ((rh->dp) && (closedir(rh->dp)))
    {
    cout <<  "Update::CheckFiles -  Can't closedir() current directory " << rh->path << endl;
    rh->dp = NULL;
    rh->quit->HandleError(READDIR_F_ERR , (char *)rh->path.c_str(), NULL);
    return FALSE;
    }
  rh->dp = NULL;
  if (change_directory(FALSE))
    {
    return (CheckFiles());
    }
  }


// return to project directory here
if ( chdir(rh->project_path.c_str()) )
  {
  rh->quit->HandleError(PATHLOCK_F_ERR, (char *)rh->project_path.c_str(), NULL);
  return FALSE;
  }

rh->path = rh->project_path;
  
rh->dp = NULL;        // Just to make sure!
rh->direntry = NULL;
cout <<  "Update::CheckFiles - LEAVING - TRUE" << endl;
return TRUE;  // NO MORE

}


gboolean Update::process_php_tag(fstream &ifp, fstream &ofp)
{
string buffer;
string buf2;
  
string ::size_type p;
string intro;
fstream::pos_type g;
gboolean found = FALSE;

g = ifp.tellg();

while (rh->read_line(ifp, buffer))
  {
  buf2 = s_tolower(buffer);

  if ( (p=buf2.find("<?php")) != string::npos )       // Standard tag
    {
    intro += buffer.substr(0, p+5) + "\n\n";
    buffer = buffer.substr(p+5);
    g += p+5;
    found = TRUE;
    break;
    }

  else if ( (p=buf2.find("<?")) != string::npos )     // Short tag
    {
    intro += buffer.substr(0, p+2) + "\n\n";
    buffer = buffer.substr(p+2);
    g += p+2;
    found = TRUE;
    break;
    }

  else if ( (p=buf2.find("<script")) != string::npos ) // script tag
    {
    string::size_type n = buf2.length();
    gboolean dquote = FALSE;
    gboolean squote = FALSE;
      
    p += 7;  // the next character
    if (p<n && buf2[p] == ' ')
      {
      
      while (p<n && buf2[p] == ' ') { p++; } // skip blanks
       
      if ( buf2.substr(p, 8) == "language" )
        {          
        p += 8;  // the next character 
        while (p<n && buf2[p] == ' ') { p++; } // skip blanks

        if (p<n && buf2[p] == '=')
          {
          p++;
          while (p<n && buf2[p] == ' ') { p++; } // skip blanks

          if (p<n && buf2[p] == '"') // got double quote
            {
            p++;
            dquote = TRUE;
            }
          else if (p<n && buf2[p] == '\'') // got single quote
            {
            p++;
            squote = TRUE;
            }

          if ( buf2.substr(p, 3) == "php" )
            {          
            p+=3;
              
            if ((squote || dquote) && p<=n) p++;
              
            while (p<=n && buf2[p] == ' ') { p++; } // skip blanks
            if (p<=n && buf2[p] == '>')
              {
              p++;
              intro += buffer.substr(0, p) + "\n";
              buffer = buffer.substr(p);
              g += p;
              found = TRUE;
              break;
              }
            }
          }
        }
      }
    }
  
  intro += buffer;
  g = ifp.tellg();
  }  // end of while

if (found)
  {
  if (rh->write_string(ofp, intro))
    {
    ifp.seekg(g);
    return TRUE;
    }
  else
    {
    // ERROR
    rh->logfile->write2log(buffer="    FAILED to write intro to temporary file");     
    }
  }
else rh->logfile->write2log(buffer="    FAILED to find a php start tag");

return FALSE;
}



/********************************
 * gboolean new_file(f,type)    *
 * Log file from RevHead.       *
 * Return TRUE (OK) or FALSE.   *
 ********************************/

gboolean Update::new_file(class Files *f, int filetype, int comment_type)
{
fstream ofp;
fstream ifp;
string  buffer;
string *tname;
TFile   tf;
Header  hd(rh);

gboolean got_php_tag = FALSE;

// Notify user
buffer = "    Logging \"" + f->pathname;
buffer += '\"';
rh->logfile->report(buffer);     

// Open the source file for read only
ifp.open(f->pathname.c_str(), ios::in | ios::binary);
UNLESS (ifp.is_open())
    {
    rh->quit->HandleError(OPEN_F_ERR, (char *)f->name.c_str(), NULL);
    return(FALSE);
    }
buffer = "    Opened \"" + f->pathname;
buffer += "\" for read";
rh->logfile->write2log(buffer);     

// Open a temporary file for write
tname = tf.tfile(&f->pathname, &ofp);
UNLESS (tname)
    {
    ifp.close();
    rh->quit->HandleError(TEMP_OPEN_F_ERR, (char *)f->pathname.c_str(), NULL);
    return(FALSE);
    }

buffer = "    Opened \"" + *tname;
buffer += "\" for write";
rh->logfile->write2log(buffer);     

// If file is PHP_TYPE we have to go past the <?php I think
// It may also possibly start with other tags e.g.
// Standard tags   <?php                     ?>
// Short tag       <?                        ?>
// ASP tags        <%                        %>
// and script tags <script language="php">   </script>
// Also works:-    <script language=php>     </script>
// Only Standard tags are sure of working.

// Another method is to use with (x)html comments
// before and after: <!-- rows of comments --!>
// at the start of the file. Xemacs formats this correctly
// but gedit doesn't.
// I think all web servers should be able to manage it
// Check all in order except ASP - forget that one
// Write Project Details to temporary file
 
if (filetype == PHP_TYPE)
  {
  got_php_tag = process_php_tag(ifp, ofp);
  if (!got_php_tag)
    {
    ifp.clear();   // clear state so that we can continue
    ifp.seekg(0);  // Back to start of input

 
    buffer = "<!--\n";
    UNLESS ( rh->write_string(ofp, buffer) )
      {
      ifp.close();
      ofp.close();
      buffer = "    FAILED to write Project Header to temporary file \"" + *tname;
      buffer += '\"';
      rh->logfile->write2log(buffer);     
      return(FALSE);
      }
    }
  }

buffer.clear();
UNLESS ( hd.write_Project(&ofp, comment_type, buffer) )
    {
    ifp.close();
    ofp.close();
    buffer = "    FAILED to write Project Header to temporary file \"" + *tname;
    buffer += '\"';
    rh->logfile->write2log(buffer);     
    return(FALSE);
    }

// Wtite the new file details to the temporary file
UNLESS ( hd.write_file_info(&ofp, f, comment_type) )
    {
    ifp.close();
    ofp.close();
    buffer = "    FAILED to write File Details to temporary file \"" + *tname;
    buffer += '\"';
    rh->logfile->write2log(buffer);     
    return(FALSE);
    }

if ((filetype == PHP_TYPE) && (!got_php_tag))
  {
  buffer = "--!>\n";
  UNLESS ( rh->write_string(ofp, buffer) )
    {
    ifp.close();
    ofp.close();
    buffer = "    FAILED to write Project Header to temporary file \"" + *tname;
    buffer += '\"';
    rh->logfile->write2log(buffer);     
    return(FALSE);
    }
  } 

if ( tf.do_copy(&ofp, &ifp) ) // Copy file to temp
  {
  ofp.close();
  ifp.close();
// Now rename the temporary file as the original
// If successful the original is replaced by the temporary
  if (rename( (char *)(*tname).c_str(), (char *)f->pathname.c_str()) ) // returns zero if successful
    {
    rh->quit->HandleError(RENAME_F_ERR, (char *)(*tname).c_str(), (char *)f->name.c_str());
    return(FALSE);
    }
  return(TRUE);
  }

ofp.close();
ifp.close();
return(FALSE);
}




/*********************************************
 * THERE WERE NO FILE CHANGES                *
 * but a change in project portion of header *
 * or else update is forced                  *
 *********************************************/

// If the comment type has changed, the whole header needs to be re-written

void Update::update_project_only(class Files *f, int type)
{
string buffer;
string test_str;
string *tname;
fstream ifp;
fstream ofp;
Header  hd(rh);
Date    dt(rh);
TFile   tf;
int     p, t;  // previous line, this line

buffer = "    No file changes \"" + f->name;
  
buffer += "\", updating project and fixdate only";
rh->logfile->write2log(buffer);
  
// Open the source file for read only
ifp.open(f->pathname.c_str(), ios::in | ios::binary);
UNLESS (ifp.is_open())
    {
    rh->quit->HandleError(OPEN_F_ERR, (char *)f->pathname.c_str(), NULL);
    return;
    }
buffer = "    Opened \"" + f->pathname;
buffer += "\" for read";
rh->logfile->write2log(buffer);

// Open a temporary file for write
tname = tf.tfile(&f->pathname, &ofp);
UNLESS (tname)
    {
    ifp.close();
    rh->quit->HandleError(TEMP_OPEN_F_ERR, (char *)f->pathname.c_str(), NULL);
    return;
    }
buffer = "    Opened \"" + *tname;
buffer += "\" for write";
rh->logfile->write2log(buffer);

// Then copy up to $RH_Start
test_str = Header::in_comment_key_start[type];
test_str += RevHead::RH_Start;

p = t = ifp.tellg();  // position in file before read


while (rh->read_line(ifp, buffer))
    {
    if (buffer.find(test_str)==0)
        {
        break;
        }
    p = t;
    t = ifp.tellg();  // position in file before next read
    
    UNLESS (rh->write_string(ofp, buffer))
        {
        ifp.close();
        ofp.close();

        buffer = "    FAILED to write \"" + f->pathname;
        buffer += "\" to \"";
        buffer += *tname;
        buffer += '\"';
        rh->logfile->write2log(buffer);
        rh->quit->HandleError(WRITE_F_ERR, (char *)(*tname).c_str(), NULL);
        return;
        }
    }
    
if (ifp.eof())
  {
  ifp.close();
  ofp.close();
  buffer = "    FAILED to find \"" + test_str;
  buffer += "\" in \"";
  buffer += f->pathname;
  buffer += '\"';
  rh->logfile->write2log(buffer);
  rh->quit->HandleError(PROJ_UPDATE_F_ERR, (char *)f->pathname.c_str(), NULL);
  return;
  }

    
ofp.seekp(p);  // back to start of this line, we need the previous line

// NB: seekp and tellg for output, seekg and tellg for input

// Write Project Details to temporary file

  
buffer.clear();
UNLESS ( hd.write_Project(&ofp, type, buffer) )
  {
  ifp.close();
  ofp.close();
  buffer = "    FAILED to write Project Header to temporary file \"" + *tname;
  buffer += '\"';
  rh->logfile->report(buffer);
  rh->quit->HandleError(PROJ_UPDATE_F_ERR, (char *)f->pathname.c_str(), NULL);
  return;
  }


test_str = Header::get_in_separator(test_str, type);
if (rh->find_string(ifp, test_str, buffer) <= 0)
  {
  ifp.close();
  ofp.close();
  buffer = "    FAILED to write Project Header to temporary file \"" + *tname;
  buffer += "\"\n    Problem while reading \"";
  buffer += f->pathname;
  buffer += '\"';
  rh->logfile->report(buffer);
  rh->quit->HandleError(PROJ_UPDATE_F_ERR, (char *)f->pathname.c_str(), NULL);
  return;
  }

UNLESS (tf.do_copy(&ofp, &ifp) )
    {
    ifp.close();
    ofp.close();
    buffer = "    FAILED to write Project Header to temporary file \"" + *tname;
    buffer += '\"';
    rh->logfile->report(buffer);
    rh->quit->HandleError(COPY_F_ERR, (char *)f->pathname.c_str(), (char *)(*tname).c_str());
    return;
    }

ifp.close();
ofp.close();

// Now rename the temporary file as the original
// If successful the original is replaced by the temporary

if (rename( (char *)(*tname).c_str(), (char *)f->pathname.c_str()) ) // returns zero if successful
    {
    rh->quit->HandleError(RENAME_F_ERR, (char *)(*tname).c_str(), (char *)f->pathname.c_str());
    return;
    }

  

buffer =   "    UPDATED project details \"" + f->pathname + "\"";
buffer +=  "\n    Getting header date";
rh->logfile->write2log(buffer);

/* If there have been no file changes we fix the date of the file. */
/* This may have changed so we make sure it's OK.            */


/*****
We need to make sure that the dates are correct.
We have requested "OK - DO IT" on a file that hasn't changed
but it wouldn't be here if the file dates were correct.
We have a choice of dating the file to the header's datestamp
or changing the header's datestamp to the file's datestamp in RevHead->finfo.
******/
UNLESS (dt.get_date(&f->datestamp, &f->timezone, &f->daylight, f->pathname, type))
    {
    buffer = "    ERROR: Can't get header date \"" + f->pathname;
    buffer += '\"';
    rh->logfile->report(buffer);
    // NOTE: p1->f_datestamp should contain a default datestamp just in case!
    }

}



/*
 * The file has changed.
 * Needs full update.
*/
void Update::update_file_all(class Files *f, int type)
{
string buffer;
string test_str;
string rh_end;
string *tname;
fstream ifp;
fstream ofp;
TFile   tf;
Header  hd(rh);
gboolean found;
gboolean found_end;
fstream::pos_type g, p, t;  // start, previous line, this line in f->name (i.e. ifp)
string tail     = Header::in_comment_end[type];
string key_head = Header::in_comment_key_start[type];
string head     = Header::in_comment_start[type];

//cout <<  "<<<< Update::update_file_all 1" << endl;

// FILE CHANGES: Notify user we are updating
buffer = "    Updating file details \"" + f->pathname;
buffer += "\"";
rh->logfile->write2log(buffer);

// File has already been logged... Deal with update
// Open the source file for read only
ifp.open(f->pathname.c_str(), ios::in | ios::binary);
UNLESS (ifp.is_open())
    {
    rh->quit->HandleError(OPEN_F_ERR, (char *)f->name.c_str(), NULL);
    return;
    }
buffer = "    Opened \"" + f->pathname;
buffer += "\" for read";
rh->logfile->write2log(buffer);

// Open a temporary file for write
tname = tf.tfile(&f->pathname, &ofp);
UNLESS (tname)
    {
    ifp.close();
    rh->quit->HandleError(TEMP_OPEN_F_ERR, (char *)f->pathname.c_str(), NULL);
    return;
    }

buffer = "    Opened \"" + *tname;
buffer += "\" for write";
rh->logfile->write2log(buffer);

// Then copy up to line before $RH_Start

test_str = key_head + RevHead::RH_Start;

p = t = ifp.tellg();  // position in f->name before read

  
while (rh->read_line(ifp, buffer))
    {
    if (buffer.find(Header::start_comment_start[type]) == 0)
      {
      t = ifp.tellg();  
      if ( rh->read_line(ifp, buffer) && (buffer.find(test_str)==0) )
        {
        break;
        }
      else p = t;     // save this as previous line          
      }
    t = ifp.tellg();  // Found $RH_Start, position in file before next read
    
    UNLESS (rh->write_string(ofp, buffer)) // write buffer (up to $RH_Start) to tname
        {
        ifp.close();
        ofp.close();

        buffer = "    FAILED to write \"" + f->pathname;
        buffer += "\" to \"";
        buffer += *tname;
        buffer += '\"';
        rh->logfile->report(buffer);
        rh->quit->HandleError(WRITE_F_ERR, (char *)(*tname).c_str(), NULL);
        return;
        }
    }
    
if (ifp.eof())
  {
  ifp.close();
  ofp.close();
  buffer = "    FAILED to find \"" + test_str;
  buffer += "\" in \"";
  buffer += f->pathname;
  buffer += '\"';
  rh->logfile->report(buffer);
  rh->quit->HandleError(PROJ_UPDATE_F_ERR, (char *)f->pathname.c_str(), NULL);
  return;
  }

//cout <<  "<<<< Update::update_file_all 2 - got RevHead_Start" << endl;

// now look for RevHead::RH_End
ifp.seekg(p);  
test_str = key_head + RevHead::RH_End;
if (rh->find_string(ifp, test_str, buffer) != 1)
  {
  ifp.close();
  ofp.close();
  buffer = "    FAILED to find \"" + test_str;
  buffer += "\" in \"";
  buffer += f->pathname;
  buffer += '\"';
  rh->logfile->report(buffer);
  rh->quit->HandleError(PROJ_UPDATE_F_ERR, (char *)f->pathname.c_str(), NULL);
  return;
  }
UNLESS (rh->read_line(ifp, buffer)) // pass over the final separator line
  {
  cerr << "FAILED to read final separator line" << endl;
  ifp.close();
  ofp.close();
  rh->quit->HandleError(READ_F_ERR, f->pathname.c_str(), NULL);
  return;
  }


// ifp should now be at the end of the header
// WRITE our new header



// ** NB: seekp and tellp for output, seekg and tellg for input ********
  
// NOTE: seekp means move to the PUT position i.e. move the position for putting
//       tellp means return  the PUT position in the file
//       seekg means move to the GET position i.e. move the position for getting
//       tellg means return  the GET position in the file
  
// Write Project Details to temporary file
buffer.clear();
UNLESS ( hd.write_Project(&ofp, type, buffer) )
  {
  ifp.close();
  ofp.close();
  buffer = "    FAILED to write Project Header to temporary file \"" + *tname;
  buffer += '\"';
  rh->logfile->report(buffer);
  rh->quit->HandleError(PROJ_UPDATE_F_ERR, (char *)f->pathname.c_str(), NULL);
  return;
  }

//cout <<  "<<<< Update::update_file_all 3 - written Project portion of header" << endl;
//cout <<  "                               - input  at RH_End" << endl;
    
// Write the new file details to the temporary file
  // * This File:
  // * Author:
  // * DateStamp:
UNLESS ( hd.write_file_info(&ofp, f, type) )
  {
  ifp.close();
  ofp.close();
  buffer = "    FAILED to write File Details to temporary file \"" + *tname;
  buffer += '\"';
  rh->logfile->report(buffer);
  rh->quit->HandleError(PROJ_UPDATE_F_ERR, (char *)f->pathname.c_str(), NULL);
  return;
  }

//cout <<  "<<<< Update::update_file_all 4 - written File Info portion of header" << endl;


// Now copy the rest of the file
UNLESS (tf.do_copy(&ofp, &ifp))
    {
    ifp.close();
    ofp.close();
    rh->quit->HandleError(COPY_F_ERR, f->pathname.c_str(), (*tname).c_str() );
    return;
    }

ifp.close();
ofp.close();

// Now rename the temporary file as the original
// If successful the original is replaced by the temporary
if (rename( (char *)(*tname).c_str(), (char *)f->pathname.c_str()) ) // returns zero if successful
    {
    rh->quit->HandleError(RENAME_F_ERR, (char *)(*tname).c_str(), (char *)f->pathname.c_str());
    return;
    }
 
rh->flags.p_changed = TRUE;  // We re-wrote the file so a change occurred
}









// Get the parent directory from full pathname
string Update::parent_directory(string &pathname)
{
string dirname;
unsigned int p = pathname.length();
if (p) p--;
while (p)
  {
  if (pathname[p] == '/')
    {
    dirname = pathname.substr(0, p);
    break;
    }
  p--;
  }
if (dirname.empty()) dirname = '/';
return dirname;
}


string Update::type_to_string(int type)
{
static string nt    = "NO_TYPE";
//static string rh    = "RH_TYPE";
static string ct    = "C_TYPE";
static string cpp   = "CPP_TYPE";
static string cpp2   = "CPP2_TYPE";
static string ada   = "ADA_TYPE";
static string php   = "PHP_TYPE";
static string java  = "JAVA_TYPE";
static string perl  = "PERL_TYPE";
static string fort  = "FORTRAN_TYPE";
static string hash  = "HASH_TYPE";

switch (type)
  {
//    case RH_TYPE:
//    return rh;
      
    case C_TYPE:
    return ct;

    case CPP_TYPE:
    return cpp;

    case ADA_TYPE:
    return ada;

    case PHP_TYPE:
    return php;

    case JAVA_TYPE:
    return java;

    case PERL_TYPE:
    return perl;

    case FORTRAN_TYPE:
    return fort;
      
    case HASH_TYPE:
    return hash;

    case CPP2_TYPE:
    return cpp2;
 
  }
return nt;   
}




void Update::change_comment_type(class Files *f, int from_type, int to_type)
{
Header hd(rh);
string buffer;
TFile   tf;
fstream ifp;
fstream ofp;
string *tname;
string test_str1;
string test_str2;
fstream::pos_type g;
gboolean failed;
  
string from_tail     = Header::in_comment_end[from_type];
  
string from_key_head = Header::in_comment_key_start[from_type];
string to_key_head   = Header::in_comment_key_start[to_type];
  
string from_head = Header::in_comment_start[from_type];
string to_head   = Header::in_comment_start[to_type];
  
  
//cout << "Update::change_comment_type - ENTERED - " << f->name << endl;

// FILE CHANGES: Notify user we are updating
buffer = "    Changing comment style \"" + f->pathname;
buffer += "\" from ";
buffer += type_to_string(from_type);
buffer += " to " + type_to_string(to_type);

if (to_type == NO_TYPE)
  {
  buffer = "    Can't change comment style";
  rh->logfile->write2log(buffer);
  return;
  }
  
rh->logfile->write2log(buffer);
  
// File has already been logged... Deal with update
// Open the source file for read only
ifp.open(f->pathname.c_str(), ios::in | ios::binary);
UNLESS (ifp.is_open())
    {
    rh->quit->HandleError(OPEN_F_ERR, (char *)f->name.c_str(), NULL);
    return;
    }
buffer = "    Opened \"" + f->pathname;
buffer += "\" for read";
rh->logfile->write2log(buffer);

// Open a temporary file for write
tname = tf.tfile(&f->pathname, &ofp);
UNLESS (tname)
    {
    ifp.close();
    rh->quit->HandleError(TEMP_OPEN_F_ERR, (char *)f->pathname.c_str(), NULL);
    return;
    }

buffer = "    Opened \"" + *tname;
buffer += "\" for write";
rh->logfile->write2log(buffer);


test_str1 = Header::start_comment_start[from_type];
test_str2 = from_key_head + RevHead::RH_Start;

g = 0; // start of input
failed = TRUE;
FOREVER
  {
  g = ifp.tellg();
//cout << "Update::change_comment_type 1- g= " << g << endl;
  if ( (rh->read_line(ifp, buffer)) && (buffer.find(test_str1)==0) )
    {
    if ( (rh->read_line(ifp, buffer)) && (buffer.find(test_str2)==0) )
      {
      failed = FALSE;
      break; // Found RH_Start, g is start of comment line
      }
    }
  }


if (failed)
  {
  ifp.close();
  ofp.close();
  remove((*tname).c_str());
  rh->logfile->write2log(buffer);
  return;
  }

// Copy from start to g here to temp file
ifp.clear();
ifp.seekg(0);    // start of file
  
//cout << "Update::change_comment_type 2- g= " << g << endl;

UNTIL (ifp.tellg() == g)
  {
//  cout << "Update::change_comment_type 3- g= " << g << " : tellg= " << ifp.tellg() << endl;

  rh->read_line(ifp, buffer);

//  cout << "Update::change_comment_type 3- wrote " << buffer << endl;
  
  rh->write_string(ofp, buffer);

  }

//cout << "Update::change_comment_type 4- g= " << g << endl;

// The last pre-header line now written  
  
  
// Translate the whole header
  
// If we get here, the end of start is in g and we have copied up to this point

//test_str1 = Header::in_comment_start[from_type];  
test_str2 = Header::in_comment_key_start[from_type] + RevHead::RH_End;

  
  
// First pass over the start comment line and write a new one

if (rh->read_line(ifp, buffer) &&  hd.write_start_separator(&ofp, to_type)) // pass over the start comment line
  {
  while ( (!failed) && rh->read_line(ifp, buffer) )
    {
    if (buffer.find(test_str2)==0)  // If RH_End
      {
      if (!hd.write_RHEnd(&ofp, to_type))
        {
        failed = TRUE;
        }
      break;
      }

// NOTE: All those following are within the comment body

    // If it is an ordinary in_comment
    if ( buffer.substr(0, from_head.length())==from_head) 
      {
      // NOTE: The in_comment_end character is the same for C and C++ but not for # type
      buffer = to_head + buffer.substr(from_head.length());

      // Fix last character before the newline
      if (buffer.length() > (Header::in_comment_start[to_type]).length()+1)
        {
        // How about remove_tail?  
        remove_tail(buffer,  from_tail);
        }
      else failed = TRUE;
      }
    else // It must be a keyword
      {

//cout << "\nUpdate::change_comment_type - KEYWORD 1 buffer=" << buffer << endl;

      // If it is a keyword type  in_comment, change the lead-in
      buffer = to_key_head + buffer.substr(from_head.length()-1);

//cout << "Update::change_comment_type - KEYWORD 2 buffer=" << buffer << endl;

      // If this is a separator row
      if ( buffer[ to_key_head.length() ] == (Header::comment_separator[from_type])[0] )
        {
        if (!hd.write_in_separator(&ofp, to_type))
          {
           failed = TRUE;
          }
        continue;
        }

      else if (buffer.length() > to_key_head.length()+1)
        {
        remove_tail(buffer, from_tail);
        }
      else failed = TRUE;

          
//cout << "Update::change_comment_type - KEYWORD 3 buffer=" << buffer << endl;

      // We also have to check for the separator character and replace as necessary
      if ( buffer[ to_key_head.length() ] == (Header::comment_separator[from_type])[0] )
        {
//cout << "Update::change_comment_type - KEYWORD 4 buffer=" << buffer << endl;

        string::size_type pos =  to_key_head.length();
        while (pos < buffer.length())
          {
          buffer[pos] = (Header::comment_separator[to_type])[0];
          pos++;
          }
//cout << "Update::change_comment_type - KEYWORD 5 buffer=" << buffer << endl << endl;
        }     
    
      }

    if (!failed)
      {
      if (!hd.write2EOL(&ofp, buffer, to_type) )
        {
        failed = TRUE;
        }
      }

//    cout << "Update::change_comment_type - wrote\n" << buffer << endl;

    } // END OF while (rh->read_line(ifp, buffer))


  
  if (!failed)
    {
    if ( rh->read_line(ifp, buffer) )  // Found RevHead_End, read in final separator row
      {
      rh->logfile->write2log(buffer="    Converted RH comments, about to copy rest of file");
        
      if ( tf.do_copy(&ofp, &ifp) ) // Copy rest file to temp
        {
        ofp.close();
        ifp.close();
        // Now rename the temporary file as the original
        // If successful the original is replaced by the temporary
        if (!rename( (char *)(*tname).c_str(), (char *)f->pathname.c_str()) ) // returns zero if successful
          {
          buffer =  "    FINISHED, renamed \"" + (*tname) + "\"\n";
          buffer += "                   as \"" + f->name + "\"";
          rh->logfile->write2log(buffer);
//cout << "Update::change_comment_type - LEAVING "  << f->name << endl;

          return;
          }
        }
      }
    }

  ifp.close();
  ofp.close();
  remove((*tname).c_str());  
  rh->logfile->write2log(buffer="    FAILED to convert comments");
  }
}

// Return the comment type wanted for this file type
int Update::get_comment_type(int filetype)
{
int comment_type;

switch (filetype)
  {
  case CPP_TYPE:
  if (rh->flags.cpp_t)
    {
    comment_type = CPP_TYPE;
    }
  else if (rh->flags.cpp2_t)
    {
    comment_type = CPP2_TYPE;
    }        
  else comment_type = C_TYPE;
  break;
    
  case PHP_TYPE:
  if (rh->flags.php_cpp_t)
    {
    comment_type = CPP_TYPE;
    }
  else if (rh->flags.php_cpp2_t)
    {
    comment_type = CPP2_TYPE;
    }
  else if (rh->flags.php_h_t)
    {
    comment_type = HASH_TYPE;
    }
  else comment_type = PHP_TYPE;
  break;
  
  case JAVA_TYPE:
  if (rh->flags.java_cpp_t)
    {
    comment_type = CPP_TYPE;
    }
  else if (rh->flags.java_cpp2_t)
    {
    comment_type = CPP2_TYPE;
    }
  else comment_type = JAVA_TYPE;
  break;

  default:   // If there is only one comment type
  comment_type = filetype;
  break;
  }

return comment_type;
}


// Re-write all the files with new headers but same datestamp
void Update::update_files(void)
{
string      buffer;
string      test_str;
int         filetype;         // Type of current file
int         comment_type;     // The desired comment type
Date        dt(rh);
Header      hd(rh);
list<Files>::iterator f;


/**********
1) Write a copy of file to a tempfile
2) Write over the original from the tempfile,
      either new header or modified header at the start of the file
3) Fix up the datestamp, comments etc.
4) Delete the tempfile
5) If there is a failure, we still have the tempfile
6) May be better to fix this using rename...
**********/


buffer = "\nUPDATING FILES";
if (rh->file_list.empty())
  {
  buffer += "\nUPDATING FILES - no files to update";
  rh->logfile->report(buffer);
  }
else
  {
  cout << "Update::update_files : " << &buffer[1] << endl;
  string pathname;

    
// Clear "rh->version_files" and "rh->no_auto_files"
// Re-write as they may have changed
  rh->version_files.clear();
  rh->no_auto_files.clear();

  for (f=rh->file_list.begin(); f!=rh->file_list.end(); ++f)
    {
    string path = parent_directory(f->pathname); 
    if (pathname != path)
      {
      pathname = path;
      buffer = "\nUPDATING FILES - SOURCE directory \"" + pathname + "\"";
      rh->logfile->report(buffer);
      }

// 1st get the file type for later
// I think we can make PHP files the same as C files for this
    filetype = rh->get_file_type(f->pathname, FALSE); // i.e. if not wanted it returns no type
    comment_type = get_comment_type(filetype);

    buffer = "Updating \"" + f->name + "\"";
    rh->logfile->write2log(buffer);

    if (f->flags.version_wanted)
      {
      rh->version_files.push_back(f->pathname);
      if (f->flags.no_auto_inc)
        {
        rh->no_auto_files.push_back(f->pathname);
        }
      }


// Is file logged?
    if (f->flags.logged == FALSE)
      {
//      cout << "Update::update_files : " << f->name << "NOT LOGGED, NEW FILE" << endl;
// File not logged
      UNLESS (new_file( &(*f), filetype, comment_type ))   // NEW FILE
        {
        rh->quit->HandleError(LOG_F_ERR, (char *)f->name.c_str(), NULL);        
        return;
        }
//      rh->flags.p_changed = TRUE; // We wrote (or attempted to write) the header to this file
      }
    else // File has been logged
      {
      if (f->comment_type != comment_type)
        {
        // Convert from (old) f->comment_type to (new)  comment_type
        change_comment_type(&(*f), f->comment_type, comment_type);
        }
       
      if (f->flags.changed == FALSE)          // No file changes
        {
        if (rh->flags.p_changed)   // project changed or forced update 
          {                        // Should forced update do file info as well?    
          update_project_only(&(*f), comment_type);                  
          } // end of if (RevHead.flags.p_changed)
        
        else
          {
          // No file changes, no header changes, so forget it
                
          buffer = "\n    NO FILE OR HEADER CHANGES, \"" + f->name;
          buffer += "\" not updated";
          rh->logfile->write2log(buffer);
          }
        }
      else                                    // There were file changes
        {
        update_file_all(&(*f), comment_type);
        }
      }


// Now datestamp it...
// Set datestamp to header datestamp
    if (dt.date_file( f->pathname, &f->datestamp))
      {
      buffer = "    FIXDATE \"" + f->pathname;
      buffer += "\" OK";
      rh->logfile->write2log(buffer);
      }
    else
      {
      rh->quit->HandleError(DS_ERROR, (char *)f->pathname.c_str(), NULL);
      return;
      }
    }
  }
}

/********************************************
 * This does the actual update of the files *
 ********************************************/
void Update::FixFileData(void)
{
string      buffer;
time_t      ds;
time_t      tz;
int         daylight;
Date        dt(rh);
//Changelog   chlog(rh);
Environment ev(rh);
TFile       tf;
  
//========================================================================
// Now re-write all the files with new headers but keep same datestamp
//========================================================================
  
update_files();

 

/* -------------------------- ChangeLog --------------------------- */
  
// Delete and over-write ".revhead.project" and "ChangeLog"
ev.update_changelog();
rh->file_list.clear();
  
/* ------------------------ Version Files ------------------------- */


// The VERSION_DATE will need updating if nothing else!
// so re-write the version files....	
// BUT there may have been no changes.  At least check the file exists
// and only write it if missing.  The date should be the project date
// from RH_History.log

  
  
  
// NOT SURE ABOUT THIS so forget it for now  
/*******************************************
UNLESS (rh->flags.p_changed)
    {
    // Correct date needed in datebuffer if no changes, get it from RH_History.log
    string pathname = rh->project_path + "/"  + Changelog::RH_ChangeLogTXT;
    if (dt.get_date(&ds, &tz, &daylight, pathname, RH_TYPE))
        {  
        dt.ds2date(&ds, &tz, &daylight, &rh->datebuffer, D_M_Y);
        }
    }
*******************************************/

// We ALWAYS write the version files if required even if not processed...
  
list<int>::iterator i;
if (!version_list.empty())
  {
  buffer = "\nVersion Files requested:-";
  rh->logfile->write2log(buffer);
  }

// If both C and CPP it doesn't matter, one over-writes the other
for (i=version_list.begin(); i!=version_list.end(); ++i)
  {
  write_version( *i );       // this should write to rh->project_path 
  }
version_list.clear();
  
if (!omit_version_list.empty())
  {
  buffer = "\nVersion Files NOT requested:-";
  rh->logfile->write2log(buffer);
  }
  
for (i=omit_version_list.begin(); i!=omit_version_list.end(); ++i)
  {
  switch ( (*i) )
    {
    case C_TYPE:
    buffer = "    " +  hname;
    break;

    case ADA_TYPE:
    buffer = "    " +  adaname;
    break;

    case PHP_TYPE:
    buffer = "    " +  phpname;
    break;

    case JAVA_TYPE:
    buffer = "    " +  javaname;
    break;

    case PERL_TYPE:
    buffer = "    " +  perlname;
    break;

    case FORTRAN_TYPE:
    buffer = "    " +  fortranname;
    break;

    default:
    char t[128];
    sprintf(t, "%d", (*i));
    buffer = "    UNKNOWN VERSION FILE TYPE: ";
    buffer += t; 
    break;
    }
  rh->logfile->write2log(buffer);
  }
omit_version_list.clear();
  
  

rh->logfile->report(buffer = "Done Update");
  
while (gtk_events_pending ())
  {
  if (gtk_main_iteration ())
    {
    cout << "Update::FixFileData - gtk_main_iteration called gtk_main_quit" << endl;
    }
  else cout << "Update::FixFileData - gtk_main_iteration" << endl;
  }
  

// Can we get to FrontPanel from here?  
rh->FrontPanel(); 
  
}

// For C/C++
void Update::escape_double_quotes(string &buffer)
{
string::size_type p = 0;

while (p<buffer.length())
  {
  if (buffer[p] == '"')
    {
    buffer.insert(p, 1, '\\');
    p++;
    }
  p++;
  }
}

// For PERL
void Update::escape_single_quotes(string &buffer)
{
string::size_type p = 0;

while (p<buffer.length())
  {
  if (buffer[p] == '\'')
    {
    buffer.insert(p, 1, '\\');
    p++;
    }
  p++;
  }
}

// For ADA and FORTRAN
void Update::double_double_quotes(string &buffer)
{
string::size_type p = 0;

while (p<buffer.length())
  {
  if (buffer[p] == '"')
    {
    buffer.insert(p, 1, '"');
    p++;
    }
  p++;
  }
}


/********************************
 * void write_c_versiong()      *
 ********************************/
void Update::write_c_version(void)
{
size_t len;

string c_ver     = "\n#define RH_VERSION_STRING    ";
string name      = "\n#define RH_PROJECT_NAME      ";
string owner     = "\n#define RH_COPYRIGHT_OWNER   ";
string version   = "\n#define RH_PROJECT_VERSION   ";
string copyright = "\n#define RH_PROJECT_COPYRIGHT ";
string contact   = "\n#define RH_PROJECT_CONTACT   ";
string date      = "\n#define RH_PROJECT_DATE      ";

fstream ofp;
string temp;
int n;

string fname = ( rh->c_version_path.empty() ) ? rh->path : rh->c_version_path;
fname += "/" + hname;

string buffer = "Writing/Re-Writing \"" + fname + "\"";

rh->logfile->write2log(buffer);

ofp.open(fname.c_str(), ios::out | ios::binary);
if (!ofp.is_open())
  {
  buffer = "    Failed to open \"" + fname + "\" for write"; 
  rh->logfile->write2log(buffer);
  return;
  }
  
// If we want the file and we have at least one of these
if (  (!rh->p_name.empty()) || 
      (!rh->p_owner.empty()) ||
      (!rh->p_address.empty()) ||
      (!rh->p_major_version.empty()) ||
      (!rh->p_minor_version.empty()) ||
      ((!rh->p_micro_version.empty()) && rh->flags.p_want_micro) ||
      (!rh->p_copyright.empty()) )
  {
  unsigned int p;
  Date  dt(rh);

  string defname = "__" + hname + "__";
  s_toupper(&defname);

  p = defname.find('.');
  if (p>=0 && p<defname.length())
    {
    defname[p] = '_';
//    cout << "Update::write_c_version - REPLACED . with _" << endl;
    }
  else
    {
//    cout << "Update::write_c_version - FAILED TO REPLACE . with _" << endl;
    }
    
  buffer = "/* " + hname + " */\n/* Use-  #include \"";
  buffer += hname + "\"  -to include this in a source file */\n\n";
  ofp.write( buffer.c_str(), buffer.length());

  buffer = "#ifndef " + defname;
  ofp.write( buffer.c_str(), buffer.length());

  buffer = "\n#define " + defname + "\n";
  ofp.write( buffer.c_str(), buffer.length());

    
// RH_PROJECT_VERSION_STRING - we need all of these, always zero-terminated.
      
  if ( (!rh->p_name.empty()) && 
        (!rh->p_major_version.empty()) &&
        (!rh->p_minor_version.empty()) &&
        ((rh->flags.p_want_micro && (!rh->p_micro_version.empty())) || (!rh->flags.p_want_micro)) &&
        (!rh->p_owner.empty()) )
    {
    temp = rh->get_project_name_and_version();
    temp += " (C) " + rh->p_owner + " " + rh->datebuffer;
    escape_double_quotes(temp);
    buffer = c_ver + "\"" + temp += '\"';
    ofp.write( buffer.c_str(), buffer.length());
    }


// RH_PROJECT_NAME
  if (!rh->p_name.empty())
    {
    temp = rh->p_name;
    escape_double_quotes(temp);
    buffer = name + "\"" + temp + "\"";
    ofp.write( buffer.c_str(), buffer.length());
    }


// RH_PROJECT_VERSION    
  temp.clear(); 
  rh->append_version(temp, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);

  escape_double_quotes(temp);
  buffer = version + "\"" + temp + "\"";
  ofp.write( buffer.c_str(), buffer.length());

// RH_COPYRIGHT_OWNER
  if (!rh->p_owner.empty())
    {
    temp = rh->p_owner;
    escape_double_quotes(temp);                  
    buffer = owner + "\"" + temp + "\"";            
    ofp.write( buffer.c_str(), buffer.length());
    }


// RH_PROJECT_COPYRIGHT - use \ to concatenate
  if (!rh->p_copyright.empty())
    {
    int i;
    buffer = copyright;
    list<string>::iterator s;
    for (s=rh->p_copyright.begin(), i=rh->p_copyright.size();
                 s!=rh->p_copyright.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_double_quotes(temp);
      buffer += "\"" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\\n\" \\\n"; // concatenate the next line
        for (unsigned int n=copyright.length()-1; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "\"";
    ofp.write( buffer.c_str(), buffer.length());
    }
          
// RH_PROJECT_CONTACT          
  if (!rh->p_address.empty())
    {
    int i;

    buffer = contact;
    ofp.write( buffer.c_str(), buffer.length());
 
    buffer.clear();
    list<string>::iterator s;
    for (s=rh->p_address.begin(), i=rh->p_address.size();
                 s!=rh->p_address.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_double_quotes(temp);
      buffer += "\"" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\\n\" \\\n"; // concatenate the next line
        for (unsigned int n=contact.length()-1; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "\"";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_DATE
  temp = rh->datebuffer;
  escape_double_quotes(temp);
  buffer = date + "\"" + temp + "\"\n\n#endif\n\n";   
  ofp.write( buffer.c_str(), buffer.length());
  ofp.close();
    
  cout << "Update::write_c_version - Written " << endl;
    
  UNLESS (dt.date_file(fname, &rh->p_currentds))
    {
    rh->quit->HandleError(DS_ERROR, fname.c_str(), NULL);
    }

  chmod(fname.c_str(), 0664);
  buffer = "    Wrote \"" + fname + "\""; 
  rh->logfile->write2log(buffer);
  return; 
  }
ofp.close();
buffer = "    NOT written \"" + fname + "\" - missing information"; 
rh->logfile->write2log(buffer);
remove(fname.c_str());
return;
}


/********************************
 * void write_php_version(void) *
 * Write the php version file.  *
 ********************************/

// php uses escape_double_quotes so FIX IT!

void Update::write_php_version(void)
{
string php_ver   = "\n  define(\"RH_VERSION_STRING\",    \"";
string name      = "\n  define(\"RH_PROJECT_NAME\",      \"";
string owner     = "\n  define(\"RH_COPYRIGHT_OWNER\",   \"";
string version   = "\n  define(\"RH_PROJECT_VERSION\",   \"";
string copyright = "\n  define(\"RH_PROJECT_COPYRIGHT\", \"";
string contact   = "\n  define(\"RH_PROJECT_CONTACT\",   \"";
string date      = "\n  define(\"RH_PROJECT_DATE\",      \"";
  
  
  
fstream ofp;
string temp;

string fname = ( rh->php_version_path.empty() ) ? rh->path : rh->php_version_path;
fname += "/" + phpname;


string buffer = "Writing/Re-Writing \"" + fname + "\"";
rh->logfile->write2log(buffer);

ofp.open(fname.c_str(), ios::out | ios::binary);
if (!ofp.is_open())
  {
  buffer = "    Failed to open \"" + fname + "\" for write"; 
  rh->logfile->write2log(buffer);
  return;
  }
                             
// If we want the file and we have at least one of these
if (  (!rh->p_name.empty()) || 
      (!rh->p_owner.empty()) ||
      (!rh->p_address.empty()) ||
      (!rh->p_major_version.empty()) ||
      (!rh->p_minor_version.empty()) ||
      ((!rh->p_micro_version.empty()) && rh->flags.p_want_micro) ||
      (!rh->p_copyright.empty()) )
  {
  unsigned int p;
  Date  dt(rh);
  string defname = "__" + phpname + "__";

  s_toupper(&defname);

  p = defname.find('.');
  if (p>=0 && p<defname.length())
    {
    defname[p] = '_';
    }

  buffer = "<?php\n\n# " + phpname + "\n\n";
  ofp.write( buffer.c_str(), buffer.length());

  buffer = "if (!defined(\"" + defname + "\"))\n  {\n  define(\"" + defname + "\", \"" + defname + "\");\n\n";
  ofp.write( buffer.c_str(), buffer.length());
  
// RH_PROJECT_VERSION_STRING - we need all of these, always zero-terminated.
  if ( (!rh->p_name.empty()) && 
        (!rh->p_major_version.empty()) &&
        (!rh->p_minor_version.empty()) &&
        ((rh->flags.p_want_micro && (!rh->p_micro_version.empty())) || (!rh->flags.p_want_micro)) &&
        (!rh->p_owner.empty()) )
    {                
    temp = rh->get_project_name_and_version();
    temp += " (C) " + rh->p_owner + " " + rh->datebuffer;
    escape_double_quotes(temp);
    buffer = php_ver + temp + "\");";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_NAME 
  if (!rh->p_name.empty())
    {           
    temp = rh->p_name;
    escape_double_quotes(temp);
    buffer = name + temp + "\");";
    ofp.write( buffer.c_str(), buffer.length());
    }


// RH_PROJECT_VERSION    
  temp.clear();   
  rh->append_version(temp, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);

  escape_double_quotes(temp);
  buffer = version + temp + "\");";
  ofp.write( buffer.c_str(), buffer.length());

// RH_COPYRIGHT_OWNER
  if (!rh->p_owner.empty())
    {
    buffer = owner + rh->p_owner + "\");";
    ofp.write( buffer.c_str(), buffer.length());
    }


// RH_PROJECT_COPYRIGHT - use . to concatenate
  if (!rh->p_copyright.empty())
    {
    int i;
    buffer = copyright;
    list<string>::iterator s;
    for (s=rh->p_copyright.begin(), i=rh->p_copyright.size();
                 s!=rh->p_copyright.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_double_quotes(temp);
      buffer += temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "<br />\".\n"; // concatenate the next line

        for (unsigned int n=copyright.length()-2; n; --n)
          {
          buffer += ' ';
          }
        buffer += "\"";
        }
      }
    buffer += "\");";
    ofp.write( buffer.c_str(), buffer.length());
    }

  
// RH_PROJECT_CONTACT          
  if (!rh->p_address.empty())
    {
    int i;
    buffer = contact;
    list<string>::iterator s;
    for (s=rh->p_address.begin(), i=rh->p_address.size();
                 s!=rh->p_address.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_double_quotes(temp);
      buffer += temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "<br />\".\n"; // concatenate the next line
        for (unsigned int n = contact.length()-2; n; --n)
          {
          buffer += ' ';
          }
        buffer += "\"";
        }
      }
    buffer += "\");";
    ofp.write( buffer.c_str(), buffer.length());
    }


// RH_PROJECT_DATE 
  temp = rh->datebuffer;
  escape_double_quotes(temp);
  buffer = date + temp + "\");\n  }\n?>\n\n";
    
  ofp.write( buffer.c_str(), buffer.length());
  ofp.close();


  UNLESS (dt.date_file(fname, &rh->p_currentds))
    {
    rh->quit->HandleError(DS_ERROR, fname.c_str(), NULL);
    }

  chmod(fname.c_str(), 0664);

  buffer = "    Wrote \"" + fname + "\""; 
  rh->logfile->write2log(buffer);
  return; 
  }

ofp.close();
buffer = "    NOT written \"" + fname + "\" - missing information"; 
rh->logfile->write2log(buffer);
remove(fname.c_str());
return;
}


/********************************
 * void write_ada_version(void) *
 * Write the ada version file.  *
 ********************************/
void Update::write_ada_version(void)
{
string ada_ver   = "\nRH_VERSION_STRING: constant STRING:=    ";
string name      = "\nRH_PROJECT_NAME: constant STRING:=      ";
string owner     = "\nRH_COPYRIGHT_OWNER: constant STRING:=   ";
string version   = "\nRH_PROJECT_VERSION: constant STRING:=   ";
string copyright = "\nRH_PROJECT_COPYRIGHT: constant STRING:= ";
string contact   = "\nRH_PROJECT_CONTACT: constant STRING:=   ";
string date      = "\nRH_PROJECT_DATE: constant STRING:=      ";

fstream ofp;
string temp;

string fname = ( rh->ada_version_path.empty() ) ? rh->path : rh->ada_version_path;
fname += "/" + adaname;

string buffer = "Writing/Re-Writing \"" + fname + "\"";
rh->logfile->write2log(buffer);

ofp.open(fname.c_str(), ios::out | ios::binary);
if (!ofp.is_open())
  {
  buffer = "    Failed to open \"" + fname + "\" for write"; 
  rh->logfile->write2log(buffer);
  return;
  }

buffer =  "-- " + adaname + "\n\n"; 
buffer += "-- Insert-\n";
buffer += "--     with RH_version;\n";
buffer += "--     use RH_version;\n";
buffer += "-- to include this in a source file\n\n";    
buffer += "package RH_version is\n";
ofp.write( buffer.c_str(), buffer.length());

// If we want the file and we have at least one of these
if (  (!rh->p_name.empty()) || 
      (!rh->p_owner.empty()) ||
      (!rh->p_address.empty()) ||
      (!rh->p_major_version.empty()) ||
      (!rh->p_minor_version.empty()) ||
      ((!rh->p_micro_version.empty()) && rh->flags.p_want_micro) ||
      (!rh->p_copyright.empty()) )
  {
  Date  dt(rh);

// RH_PROJECT_VERSION_STRING - we need all of these, always zero-terminated.
  if ( (!rh->p_name.empty()) && 
        (!rh->p_major_version.empty()) &&
        (!rh->p_minor_version.empty()) &&
        ((rh->flags.p_want_micro && (!rh->p_micro_version.empty())) || (!rh->flags.p_want_micro)) &&
        (!rh->p_owner.empty()) )
    {
    temp = rh->get_project_name_and_version();
    temp += " (C) " + rh->p_owner + " " + rh->datebuffer;
    double_double_quotes(temp);
    buffer = ada_ver + "\"" + temp + "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_NAME 
  if (!rh->p_name.empty())
    {
    temp = rh->p_name;
    double_double_quotes(temp);
    buffer = name + "\"" + temp + "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_VERSION    
  temp.clear();   
  rh->append_version(temp, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);

  double_double_quotes(temp);    
  buffer = version + "\"" + temp + "\";";
  ofp.write( buffer.c_str(), buffer.length());

// RH_COPYRIGHT_OWNER
  if (!rh->p_owner.empty())
    {
    temp = rh->p_owner;
    double_double_quotes(temp);                  
    buffer = owner + "\"" + temp + "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }
    
// RH_PROJECT_COPYRIGHT - use & to concatenate and ASCII.LF for newline
  if (!rh->p_copyright.empty())
    {
    int i;
    buffer = copyright;

    list<string>::iterator s;
    for (s=rh->p_copyright.begin(), i=rh->p_copyright.size();
                 s!=rh->p_copyright.end();
                 ++s, --i)
      {
      temp = (*s);
      double_double_quotes(temp);                  
  
      buffer += "\"" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\" & ASCII.LF &\n"; // concatenate the next line
        for (unsigned int n = copyright.length()-1; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }
  
// RH_PROJECT_CONTACT          
  if (!rh->p_address.empty())
    {
    int i;
    buffer = contact;
    list<string>::iterator s;
    for (s=rh->p_address.begin(), i=rh->p_address.size();
                 s!=rh->p_address.end();
                 ++s, --i)
      {
      temp = (*s);
      double_double_quotes(temp);                  
  
      buffer += "\"" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\" & ASCII.LF &\n"; // concatenate the next line
        for (unsigned int n = contact.length()-1; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_DATE and end 
  temp = rh->datebuffer;
  double_double_quotes(temp);                  
  
  buffer = date + "\"" + temp + "\";\n\nend RH_version;\n";    
  ofp.write( buffer.c_str(), buffer.length());
  ofp.close();

  UNLESS (dt.date_file(fname, &rh->p_currentds))
    {
    rh->quit->HandleError(DS_ERROR, fname.c_str(), NULL);
    return;
    }  
  chmod(fname.c_str(), 0664);
  buffer = "    Wrote \"" + fname + "\""; 
  rh->logfile->write2log(buffer);
  return; 
  }

ofp.close();
buffer = "    NOT written \"" + fname + "\" - missing information"; 
rh->logfile->write2log(buffer);
remove(fname.c_str());
return;
}

/*********************************
 * void write_java_version(void) *
 * Write the java version file.  *
 *********************************/
void Update::write_java_version(void)
{
string j_start   = "\npublic static final String ";
string java_ver  = "RH_VERSION_STRING    = ";
string name      = "RH_PROJECT_NAME      = ";
string owner     = "RH_COPYRIGHT_OWNER   = ";
string version   = "RH_PROJECT_VERSION   = ";
string copyright = "RH_PROJECT_COPYRIGHT = ";
string contact   = "RH_PROJECT_CONTACT   = ";
string date      = "RH_PROJECT_DATE      = ";

fstream ofp;
string temp;

string fname = ( rh->java_version_path.empty() ) ? rh->path : rh->java_version_path;
fname += "/" + javaname;

string buffer = "Writing/Re-Writing \"" + fname + "\"";
rh->logfile->write2log(buffer);

ofp.open(fname.c_str(), ios::out | ios::binary);
if (!ofp.is_open())
  {
  buffer = "    Failed to open \"" + fname + "\" for write"; 
  rh->logfile->write2log(buffer);
  return;
  }

buffer = "/* " + javaname + " */\n\n" + "public class RH_version\n{\n";
ofp.write( buffer.c_str(), buffer.length());

// If we want the file and we have at least one of these
if (  (!rh->p_name.empty()) || 
      (!rh->p_owner.empty()) ||
      (!rh->p_address.empty()) ||
      (!rh->p_major_version.empty()) ||
      (!rh->p_minor_version.empty()) ||
      ((!rh->p_micro_version.empty()) && rh->flags.p_want_micro) ||
      (!rh->p_copyright.empty()) )
  {
  Date  dt(rh);

// RH_PROJECT_VERSION_STRING- we need all of these, always zero-terminated.
  if ( (!rh->p_name.empty()) && 
        (!rh->p_major_version.empty()) &&
        (!rh->p_minor_version.empty()) &&
        ((rh->flags.p_want_micro && (!rh->p_micro_version.empty())) || (!rh->flags.p_want_micro)) &&
        (!rh->p_owner.empty()) )
    {                
    temp = rh->get_project_name_and_version();
    temp += " (C) " + rh->p_owner + " " + rh->datebuffer;
    escape_double_quotes(temp);
    buffer = j_start + java_ver + "\"" + temp += "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }
    
// RH_PROJECT_NAME 
  if (!rh->p_name.empty())
    {           
    temp = rh->p_name;
    escape_double_quotes(temp);
    buffer = j_start + name + "\"" + temp + "\";";
    ofp.write( buffer.c_str(), buffer.length());

    }

// RH_PROJECT_VERSION
  temp.clear();  
  rh->append_version(temp, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);

  escape_double_quotes(temp);
  buffer = j_start + version + "\"" + temp + "\";";
  ofp.write( buffer.c_str(), buffer.length());

    
// RH_COPYRIGHT_OWNER
  if (!rh->p_owner.empty())
    {
    temp = rh->p_owner;
    escape_double_quotes(temp);
    buffer = j_start + owner + "\"" + temp + "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }
cout << "Update::write_java_version - written RH_COPYRIGHT_OWNER" << endl;
    
// RH_PROJECT_COPYRIGHT - use + to concatenate
  if (!rh->p_copyright.empty())
    {
    int i;
    unsigned int p;
    buffer = j_start + copyright;
    p = buffer.length() - 1;
    list<string>::iterator s;
    for (s=rh->p_copyright.begin(), i=rh->p_copyright.size();
                 s!=rh->p_copyright.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_double_quotes(temp);
      buffer += "\"" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\\n\" +\n"; // concatenate the next line

        for (unsigned int n = p; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_CONTACT          
  if (!rh->p_address.empty())
    {
    int i;
    unsigned int p;
    buffer = j_start + contact;
    p = buffer.length() - 1;
    list<string>::iterator s;
    for (s=rh->p_address.begin(), i=rh->p_address.size();
                 s!=rh->p_address.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_double_quotes(temp);
      buffer += "\"" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\\n\" +\n"; // concatenate the next line
        for (unsigned int n = p; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "\";";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_DATE and end 
  temp = rh->datebuffer;
  escape_double_quotes(temp);
  buffer = j_start + date + "\"" + temp + "\";\n\n}\n\n";   
  ofp.write( buffer.c_str(), buffer.length());
  ofp.close();

  UNLESS (dt.date_file(fname, &rh->p_currentds))
    {
    rh->quit->HandleError(DS_ERROR, fname.c_str(), NULL);
    return;
    }  
  chmod(fname.c_str(), 0664);  
  buffer = "    Wrote \"" + fname + "\""; 
  rh->logfile->write2log(buffer);
  return; 
  }

ofp.close();
buffer = "    NOT written \"" + fname + "\" - missing information"; 
rh->logfile->write2log(buffer);
remove(fname.c_str());
return;
}



/*********************************
 * void write_perl_version(void) *
 * Write the perl version file.  *
 *********************************/
void Update::write_perl_version(void)
{
string perl_start =    "#!/usr/bin/perl -w\n\n";

string perl_ver  = "\n$RH_VERSION_STRING    = ";
string name      = "\n$RH_PROJECT_NAME      = ";
string owner     = "\n$RH_COPYRIGHT_OWNER   = ";
string version   = "\n$RH_PROJECT_VERSION   = ";
string copyright = "\n$RH_PROJECT_COPYRIGHT = ";
string contact   = "\n$RH_PROJECT_CONTACT   = ";
string date      = "\n$RH_PROJECT_DATE      = ";



fstream ofp;
string temp;

string fname = ( rh->perl_version_path.empty() ) ? rh->path : rh->perl_version_path;
fname += "/" + perlname;

string buffer = "Writing/Re-Writing \"" + fname + "\"";
rh->logfile->write2log(buffer);

ofp.open(fname.c_str(), ios::out | ios::binary);
if (!ofp.is_open())
  {
  buffer = "    Failed to open \"" + fname + "\" for write"; 
  rh->logfile->write2log(buffer);
  return;
  }

buffer = perl_start + "# " + perlname + "\n";
buffer += "# Insert-  require '" + perlname + "';  -to include this in a source file\n\n";
ofp.write( buffer.c_str(), buffer.length());

// If we want the file and we have at least one of these
if (  (!rh->p_name.empty()) || 
      (!rh->p_owner.empty()) ||
      (!rh->p_address.empty()) ||
      (!rh->p_major_version.empty()) ||
      (!rh->p_minor_version.empty()) ||
      ((!rh->p_micro_version.empty()) && rh->flags.p_want_micro) ||
      (!rh->p_copyright.empty()) )

  {
  Date  dt(rh);

// RH_PROJECT_VERSION_STRING- we need all of these, always zero-terminated.
  if ( (!rh->p_name.empty()) && 
        (!rh->p_major_version.empty()) &&
        (!rh->p_minor_version.empty()) &&
        ((rh->flags.p_want_micro && (!rh->p_micro_version.empty())) || (!rh->flags.p_want_micro)) &&
        (!rh->p_owner.empty()) )
    {
    temp = rh->get_project_name_and_version();
    temp += " (C) ";
    temp += rh->p_owner;        
    temp += ' ';
    temp += rh->datebuffer; 
    escape_single_quotes(temp);
    buffer = perl_ver + "'" + temp + "';";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_NAME 
  if (!rh->p_name.empty())
    {
    temp = rh->p_name;
    escape_single_quotes(temp);      
    buffer = name + "'" + temp + "';";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_VERSION    
  temp.clear();    
  rh->append_version(temp, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);

  escape_single_quotes(temp);
  buffer = version + "\'" + temp + "';";
  ofp.write( buffer.c_str(), buffer.length());

// RH_COPYRIGHT_OWNER
  if (!rh->p_owner.empty())
    {
    temp = rh->p_owner;
    escape_single_quotes(temp);                  
    buffer = owner + "'" + temp + "';";            
    ofp.write( buffer.c_str(), buffer.length());
    }


// RH_PROJECT_COPYRIGHT - use . to concatenate
  if (!rh->p_copyright.empty())
    {
    int i;
    buffer = copyright;
    ofp.write( buffer.c_str(), buffer.length());

    buffer.clear();
    list<string>::iterator s;
    for (s=rh->p_copyright.begin(), i=rh->p_copyright.size();
                 s!=rh->p_copyright.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_single_quotes(temp);
      buffer += "'" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\\n' .\n"; // concatenate the next line
        for (unsigned int n=copyright.length()-1; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "';";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_CONTACT          
  if (!rh->p_address.empty())
    {
    int i;

    buffer = contact;
    ofp.write( buffer.c_str(), buffer.length());
 
    buffer.clear();
    list<string>::iterator s;
    for (s=rh->p_address.begin(), i=rh->p_address.size();
                 s!=rh->p_address.end();
                 ++s, --i)
      {
      temp = (*s);
      escape_single_quotes(temp);
      buffer += "'" + temp;

      if (i > 1)          // if there is another line after this one
        {
        buffer += "\\n' .\n"; // concatenate the next line
        for (unsigned int n=contact.length()-1; n; --n)
          {
          buffer += ' ';
          }
        }
      }
    buffer += "';";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_DATE and end 
  temp = rh->datebuffer;
  escape_single_quotes(temp);
  buffer = date + "'" + temp + "';\n";    
  ofp.write( buffer.c_str(), buffer.length());
  ofp.close();

  UNLESS (dt.date_file(fname, &rh->p_currentds))
    {
    rh->quit->HandleError(DS_ERROR, fname.c_str(), NULL);
    return;
    }  
  
  chmod(fname.c_str(), 0664);
  buffer = "    Wrote \"" + fname + "\""; 
  rh->logfile->write2log(buffer);
  return; 
  }

ofp.close();
buffer = "    NOT written \"" + fname + "\" - missing information"; 
rh->logfile->write2log(buffer);
remove(fname.c_str());
return;
}

/************************************
 * void write_fortran_version(void) *
 * Write the perl version file.     *
 ************************************/
// NOTE: maximum line width is 132 characters
// Use 112 to play safe, then add "//CHAR(0)//& if required and lead_in with " at the start 
// making 130 altogether with the newline
// I think there is a limit to the number of & characters in a statement
void Update::write_fortran_version(void)
{
string line_start = "  CHARACTER(len=*), PARAMETER :: ";
string fort_ver   = "RH_VERSION_STRING    = &\n";
string name       = "RH_PROJECT_NAME      = &\n";
string owner      = "RH_COPYRIGHT_OWNER   = &\n";
string version    = "RH_PROJECT_VERSION   = &\n";
string copyright  = "RH_PROJECT_COPYRIGHT = &\n";
string contact    = "RH_PROJECT_CONTACT   = &\n";
string date       = "RH_PROJECT_DATE      = &\n";
string cont       = "//CHAR(10)//&";
string cont_nl    = "\"//CHAR(10)//&\n";

fstream ofp;
string temp;

// This is the length of the start of all statements
string lead_in = "    ";
string::size_type lead_len = lead_in.length();
string::size_type pos;
string::size_type maxlen = 112;

string fname = ( rh->fortran_version_path.empty() ) ? rh->path : rh->fortran_version_path;
fname += "/" + fortranname;

string buffer = "Writing/Re-Writing \"" + fname + "\"";
rh->logfile->write2log(buffer);

ofp.open(fname.c_str(), ios::out | ios::binary);
if (!ofp.is_open())
  {
  buffer = "    Failed to open \"" + fname + "\" for write"; 
  rh->logfile->write2log(buffer);
  return;
  }

buffer = "! " + fortranname + "\n";
buffer += "! Insert-\n!    USE RH_version\n! to include this in a source file\n\nMODULE RH_version\n\n";
ofp.write( buffer.c_str(), buffer.length());

// If we want the file and we have at least one of these
if (  (!rh->p_name.empty()) || 
      (!rh->p_owner.empty()) ||
      (!rh->p_address.empty()) ||
      (!rh->p_major_version.empty()) ||
      (!rh->p_minor_version.empty()) ||
      ((!rh->p_micro_version.empty()) && rh->flags.p_want_micro) ||
      (!rh->p_copyright.empty()) )

  {
  Date  dt(rh);

// RH_PROJECT_VERSION_STRING- we need all of these, always zero-terminated.
  if ( (!rh->p_name.empty()) && 
        (!rh->p_major_version.empty()) &&
        (!rh->p_minor_version.empty()) &&
        ((rh->flags.p_want_micro && (!rh->p_micro_version.empty())) || (!rh->flags.p_want_micro)) &&
        (!rh->p_owner.empty()) )
    {
    buffer = line_start + fort_ver;
    ofp.write( buffer.c_str(), buffer.length());
      
    temp = rh->get_project_name_and_version();
    temp += " (C) " + rh->p_owner + " " + rh->datebuffer;
    double_double_quotes(temp);

    while ( (lead_len + temp.length()) > maxlen )
      {
      buffer = lead_in + "\"temp.substr(0,maxlen)" + cont_nl;
      temp = "\"" + temp.substr(maxlen);
      ofp.write( buffer.c_str(), buffer.length());
      }

    buffer = lead_in + "\"" + temp += "\"\n";
    ofp.write( buffer.c_str(), buffer.length());
    }
 
// RH_PROJECT_NAME 
  if (!rh->p_name.empty())
    {
    buffer = line_start + name;
    ofp.write( buffer.c_str(), buffer.length());

    temp = rh->p_name;
    double_double_quotes(temp);
    while ( (lead_len + temp.length()) > maxlen )
      {
      buffer = lead_in + "\"temp.substr(0, maxlen)"  + cont_nl;
      temp = "\"" + temp.substr(maxlen);
      ofp.write( buffer.c_str(), buffer.length());
      }

    buffer = lead_in + "\"" + temp += "\"\n";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_VERSION
  buffer = line_start + version;
  ofp.write( buffer.c_str(), buffer.length());

  temp.clear();    
  rh->append_version(temp, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);
  double_double_quotes(temp);
    
  while ( (lead_len + temp.length()) > maxlen )
    {
    buffer = lead_in + "\"temp.substr(0, maxlen)"  + cont_nl;
    temp = "\"" + temp.substr(maxlen);
    ofp.write( buffer.c_str(), buffer.length());
    }

  buffer = lead_in + "\"" + temp += "\"\n";
  ofp.write( buffer.c_str(), buffer.length());

// RH_COPYRIGHT_OWNER
  if (!rh->p_owner.empty())
    {
    buffer = line_start + owner;
    ofp.write( buffer.c_str(), buffer.length());

    temp = rh->p_owner;
    double_double_quotes(temp);
    while ( (lead_len + temp.length()) > maxlen )
      {
      buffer = lead_in + "\"temp.substr(0, maxlen)"  + cont_nl;
      temp = "\"" + temp.substr(maxlen);
      ofp.write( buffer.c_str(), buffer.length());
      }

    buffer = lead_in + "\"" + temp += "\"\n";
    ofp.write( buffer.c_str(), buffer.length());
    }

// RH_PROJECT_COPYRIGHT - use . to concatenate
  if (!rh->p_copyright.empty())
    {
    int i;
    buffer = line_start + copyright;
    ofp.write( buffer.c_str(), buffer.length());

    buffer.clear();
    list<string>::iterator s;
    for (s=rh->p_copyright.begin(), i=rh->p_copyright.size();
                 s!=rh->p_copyright.end();
                 ++s, --i)
      {
      // Write each line as we go...
      temp = (*s);
      double_double_quotes(temp);

      while ( (lead_len + temp.length()) > maxlen )
        {
        buffer = lead_in + "\"temp.substr(0, maxlen)"  + cont_nl;
        temp = "\"" + temp.substr(maxlen);
        ofp.write( buffer.c_str(), buffer.length());
        }

      buffer = lead_in + "\"" + temp += "\"";
      if (i > 1)  // if there is another line after this one
        {
        buffer += cont;
        }
      buffer += "\n";
      ofp.write( buffer.c_str(), buffer.length());
      }
    }

// RH_PROJECT_CONTACT          
  if (!rh->p_address.empty())
    {
    int i;

    buffer = line_start + contact;
    ofp.write( buffer.c_str(), buffer.length());
 
    buffer.clear();
    list<string>::iterator s;
    for (s=rh->p_address.begin(), i=rh->p_address.size();
                 s!=rh->p_address.end();
                 ++s, --i)
      {
      temp = (*s);
      double_double_quotes(temp);

      while ( (lead_len + temp.length()) > maxlen )
        {
        buffer = lead_in + "\"temp.substr(0, maxlen)"  + cont_nl;
        temp = "\"" + temp.substr(maxlen);
        ofp.write( buffer.c_str(), buffer.length());
        }

      buffer = lead_in + "\"" + temp + "\"";
      if (i > 1)  // if there is another line after this one
        {
        buffer +=  cont;
        }
      buffer += "\n";
      ofp.write( buffer.c_str(), buffer.length());
      }
    }

// RH_PROJECT_DATE and end
  buffer = line_start + date;
  ofp.write( buffer.c_str(), buffer.length());

  temp = rh->datebuffer;
  double_double_quotes(temp);
  while ( (lead_len + temp.length()) > maxlen )
    {
    buffer = lead_in + "\"temp.substr(0, maxlen)"  + cont_nl;
    temp = "\"" + temp.substr(maxlen);
    ofp.write( buffer.c_str(), buffer.length());
    }

  buffer = lead_in + "\"" + temp + "\"\n" + "\nend MODULE RH_version\n\n";
  ofp.write( buffer.c_str(), buffer.length());
  ofp.close();

  UNLESS (dt.date_file(fname, &rh->p_currentds))
    {
    ofp.close();
    rh->quit->HandleError(DS_ERROR, fname.c_str(), NULL);
    return;
    }  
  
  chmod(fname.c_str(), 0664);
  buffer = "    Wrote \"" + fname + "\""; 
  rh->logfile->write2log(buffer);
  return; 
  }

ofp.close();
buffer = "    NOT written \"" + fname + "\" - missing information"; 
rh->logfile->write2log(buffer);
remove(fname.c_str());
return;

}

void Update::write_version(int type)
{
switch (type)
  {
  case C_TYPE:
  case CPP_TYPE:
  write_c_version();
  break;

  case ADA_TYPE:
  write_ada_version();
  break;

  case PHP_TYPE:
  write_php_version();
  break;

  case JAVA_TYPE:
  write_java_version();
  break;

  case PERL_TYPE:
  write_perl_version();
  break;

  case FORTRAN_TYPE:
  write_fortran_version();
  break;

  default:
  string buffer = "    Version file NOT written, unknown type"; 
  rh->logfile->write2log(buffer);
  break;  
  }
}

/************************* END OF UPDATE ******************************/

