/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       update.h                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       28-Mar-2009 14.24.51 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Header file for update.cpp                                                *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __UPDATE_H__
#define __UPDATE_H__

#include "revhead.h"
#include "getproject.h"

class Update
{
public:
  Update(RevHead *revhead);
  ~Update(void);

  void DoUpdate(void);
  gboolean CheckFiles(void);

  void FixFileData(void);


  static string hname;       // For C / CPP files
  static string phpname;     // For PHP files
  static string adaname;     // For ADA files
  static string javaname;    // For JAVA files
  static string perlname;    // For PERL files
  static string fortranname; // For FORTRAN files

  static string UpdateTXT;

  string       f_old_name;
  string       f_author;
  string       f_author_email;
  string       f_major_v;
  string       f_minor_v;
  string       f_micro_v;  
  list<string> f_summary;
  time_t       f_ds;               // This file's datestamp
  time_t       f_tz;               // This file's timezone
  int          f_comment_type;     // This file's comment type
  bool         f_want_version;
  bool         f_no_auto_inc;
        
private:
  bool change_directory(bool start);
  bool filename_OK(string &temp, string &pathname, string &fname, int);
  gboolean process_php_tag(fstream &ifp, fstream &ofp);
  gboolean new_file(Files *f, int filetype, int comment_type);

  string parent_directory(string &pathname);

  void update_project_only(Files *f, int type);
  void update_file_all(Files *f, int type);
  string type_to_string(int type);
  int get_comment_type(int filetype);
  void change_comment_type(Files *f, int from_type, int to_type);
  void update_files(void);        

  void escape_double_quotes(string &buffer);
  void escape_single_quotes(string &buffer);
  void double_double_quotes(string &buffer);

  void write_c_version(void);
  void write_php_version(void);
  void write_ada_version(void);
  void write_java_version(void);
  void write_perl_version(void);
  void write_fortran_version(void);
  void write_version(int type);

  RevHead *rh;
  GetProject *gp;
  list<int> version_list;
  list<int> omit_version_list;
  GList *fd_list;
        
protected:
        
};

       
#endif  // __UPDATE_H__
